@@ -51,6 +51,7 @@
 #include "log.h"
 #include "guid_cmp.h"
 #include "dimage.h"
+#include "fat_adv.h"
 
 extern const arch_fnct_t arch_gpt;
 extern const arch_fnct_t arch_i386;
@@ -89,6 +90,7 @@ static int is_linux(const partition_t *partition)
     case UP_CRAMFS:
     case UP_EXT2:
     case UP_EXT3:
+    case UP_EXT4:
     case UP_JFS:
     case UP_RFS:
     case UP_RFS2:
@@ -190,7 +192,8 @@ void interface_adv(disk_t *disk_car, const int verbose,const int dump_ind, const
       {'t',"Type","Change type, this setting will not be saved on disk"},
       {'b',"Boot","Boot sector recovery"},
       {'s',"Superblock",NULL},
-      {'c',"Image creation", "Create an image"},
+      {'c',"Image Creation", "Create an image"},
+      {'u',"Undelete", "File undelete"},
 //      {'a',"Add", "Add temporary partition (Expert only)"},
       {'q',"Quit","Return to main menu"},
       {0,NULL,NULL}
@@ -237,40 +240,54 @@ void interface_adv(disk_t *disk_car, const int verbose,const int dump_ind, const
     }
     else
     {
-      if(is_part_fat(current_element->part) ||
-	  is_part_ntfs(current_element->part))
+      if(is_part_fat(current_element->part))
+      {
+	options="tubcq";
+	menu=1;
+      }
+      else if(is_part_ntfs(current_element->part))
       {
 	options="tbcq";
 	menu=1;
       }
       else if(is_part_linux(current_element->part))
       {
-	options="tscq";
-	menuAdv[2].desc="Locate EXT2/EXT3 backup superblock";
+	if(current_element->part->upart_type==UP_EXT2)
+	  options="tuscq";
+	else
+	  options="tscq";
+	menuAdv[2].desc="Locate ext2/ext3/ext4 backup superblock";
 	menu=1;
       }
       else if(is_part_hfs(current_element->part) || is_part_hfsp(current_element->part))
       {
 	options="tscq";
-	menuAdv[2].desc="Locate HFS/HFS+ backup superblock";
+	menuAdv[2].desc="Locate HFS/HFS+ backup volume header";
 	menu=1;
       }
-      else if(is_fat(current_element->part) ||
-	  is_ntfs(current_element->part))
+      else if(is_fat(current_element->part))
+      {
+	options="tubcq";
+	menu=1;
+      }
+      else if(is_ntfs(current_element->part))
       {
 	options="tbcq";
 	menu=1;
       }
       else if(is_linux(current_element->part))
       {
-	options="tscq";
-	menuAdv[2].desc="Locate EXT2/EXT3 backup superblock";
+	if(current_element->part->upart_type==UP_EXT2)
+	  options="tuscq";
+	else
+	  options="tscq";
+	menuAdv[2].desc="Locate ext2/ext3/ext4 backup superblock";
 	menu=1;
       }
       else if(is_hfs(current_element->part) || is_hfsp(current_element->part))
       {
 	options="tscq";
-	menuAdv[2].desc="Locate HFS/HFS+ backup superblock";
+	menuAdv[2].desc="Locate HFS/HFS+ backup volume header";
 	menu=1;
       }
       else
@@ -301,6 +318,11 @@ void interface_adv(disk_t *disk_car, const int verbose,const int dump_ind, const
 	  (*current_cmd)+=4;
 	  command='c';
 	}
+	else if(strncmp(*current_cmd,"list",4)==0)
+	{
+	  (*current_cmd)+=4;
+	  command='l';
+	}
 	else if(strncmp(*current_cmd,"superblock",10)==0)
 	{
 	  (*current_cmd)+=10;
@@ -324,7 +346,7 @@ void interface_adv(disk_t *disk_car, const int verbose,const int dump_ind, const
     else
     {
 #ifdef HAVE_NCURSES
-      command = wmenuSelect(stdscr,INTER_ADV_Y, INTER_ADV_X, menuAdv, 8, options,
+      command = wmenuSelect(stdscr, 24, INTER_ADV_Y, INTER_ADV_X, menuAdv, 8, options,
 	  MENU_HORIZ | MENU_BUTTON | MENU_ACCEPT_OTHERS, menu);
 #else
       command = 'q';
@@ -364,6 +386,7 @@ void interface_adv(disk_t *disk_car, const int verbose,const int dump_ind, const
 	break;
       case 'a':
       case 'A':
+	if(disk_car->arch->add_partition!=NULL)
 	{
 	  list_part=disk_car->arch->add_partition(disk_car,list_part, verbose, current_cmd);
 	  current_element=list_part;
@@ -416,7 +439,7 @@ void interface_adv(disk_t *disk_car, const int verbose,const int dump_ind, const
 	  image_dd=ask_location("Do you want to save disk file image.dd in %s%s ? [Y/N]","");
 	  if(image_dd!=NULL)
 	  {
-	    char *new_recup_dir=MALLOC(strlen(image_dd)+1+strlen(DEFAULT_IMAGE_NAME)+1);
+	    char *new_recup_dir=(char *)MALLOC(strlen(image_dd)+1+strlen(DEFAULT_IMAGE_NAME)+1);
 	    strcpy(new_recup_dir,image_dd);
 	    strcat(new_recup_dir,"/");
 	    strcat(new_recup_dir,DEFAULT_IMAGE_NAME);
@@ -430,6 +453,15 @@ void interface_adv(disk_t *disk_car, const int verbose,const int dump_ind, const
 	  }
 	}
 	break;
+      case 'u':
+      case 'U':
+      case 'l':
+      case 'L':
+	if(current_element!=NULL)
+	{
+	  dir_partition(disk_car, current_element->part, 0, current_cmd);
+	}
+	break;
       case 's':
       case 'S':
 	if(current_element!=NULL)
@@ -499,7 +531,7 @@ int fat1x_boot_sector(disk_t *disk_car, partition_t *partition, const int verbos
     { 'P', "Previous",""},
     { 'N', "Next","" },
     { 'Q', "Quit","Return to Advanced menu"},
-    { 'L', "List", "List directories and files" },
+    { 'L', "List", "List directories and files, copy and undelete data from FAT" },
     { 'R', "Rebuild BS","Rebuild boot sector"},
     { 'D', "Dump","Dump boot sector and backup boot sector"},
     { 'C', "Repair FAT","Very Dangerous! Expert only"},
@@ -511,7 +543,7 @@ int fat1x_boot_sector(disk_t *disk_car, partition_t *partition, const int verbos
   {
     unsigned int menu=0;
     int command;
-    aff_buffer(BUFFER_RESET,"Q");
+    screen_buffer_reset();
     if(rescan==1)
     {
 #ifdef HAVE_NCURSES
@@ -524,15 +556,15 @@ int fat1x_boot_sector(disk_t *disk_car, partition_t *partition, const int verbos
 #endif
       log_info("\nfat1x_boot_sector\n");
       log_partition(disk_car,partition);
-      aff_buffer(BUFFER_ADD,"Boot sector\n");
+      screen_buffer_add("Boot sector\n");
       if(disk_car->read(disk_car,FAT1x_BOOT_SECTOR_SIZE, buffer_bs, partition->part_offset)!=0)
       {
-	aff_buffer(BUFFER_ADD,"fat1x_boot_sector: Can't read boot sector.\n");
+	screen_buffer_add("fat1x_boot_sector: Can't read boot sector.\n");
 	memset(buffer_bs,0,FAT1x_BOOT_SECTOR_SIZE);
       }
       if(test_FAT(disk_car,(const struct fat_boot_sector *)buffer_bs,partition,verbose,0)==0)
       {
-	aff_buffer(BUFFER_ADD,"OK\n");
+	screen_buffer_add("OK\n");
 	if(expert==0)
 	  options="DRCL";
 	else
@@ -540,12 +572,12 @@ int fat1x_boot_sector(disk_t *disk_car, partition_t *partition, const int verbos
       }
       else
       {
-	aff_buffer(BUFFER_ADD,"Bad\n");
+	screen_buffer_add("Bad\n");
 	options="DRC";
       }
-      aff_buffer(BUFFER_ADD,"\n");
-      aff_buffer(BUFFER_ADD,"A valid FAT Boot sector must be present in order to access\n");
-      aff_buffer(BUFFER_ADD,"any data; even if the partition is not bootable.\n");
+      screen_buffer_add("\n");
+      screen_buffer_add("A valid FAT Boot sector must be present in order to access\n");
+      screen_buffer_add("any data; even if the partition is not bootable.\n");
       rescan=0;
     }
     screen_buffer_to_log();
@@ -658,7 +690,7 @@ int fat32_boot_sector(disk_t *disk_car, partition_t *partition, const int verbos
     { 'P', "Previous",""},
     { 'N', "Next","" },
     { 'Q', "Quit","Return to Advanced menu"},
-    { 'L', "List", "List directories and files" },
+    { 'L', "List", "List directories and files, copy and undelete data from FAT" },
     { 'O', "Org. BS","Copy boot sector over backup sector"},
     { 'B', "Backup BS","Copy backup boot sector over boot sector"},
     { 'R', "Rebuild BS","Rebuild boot sector"},
@@ -672,7 +704,7 @@ int fat32_boot_sector(disk_t *disk_car, partition_t *partition, const int verbos
   {
     unsigned int menu=0;
     int command;
-    aff_buffer(BUFFER_RESET,"Q");
+    screen_buffer_reset();
     if(rescan==1)
     {
       int opt_over=0;
@@ -689,15 +721,15 @@ int fat32_boot_sector(disk_t *disk_car, partition_t *partition, const int verbos
 #endif
       log_info("\nfat32_boot_sector\n");
       log_partition(disk_car,partition);
-      aff_buffer(BUFFER_ADD,"Boot sector\n");
+      screen_buffer_add("Boot sector\n");
       if(disk_car->read(disk_car,3*disk_car->sector_size, buffer_bs, partition->part_offset)!=0)
       {
-	aff_buffer(BUFFER_ADD,"fat32_boot_sector: Can't read boot sector.\n");
+	screen_buffer_add("fat32_boot_sector: Can't read boot sector.\n");
 	memset(buffer_bs,0,3*disk_car->sector_size);
       }
       if(test_FAT(disk_car,(struct fat_boot_sector *)buffer_bs,partition,verbose,0)==0)
       {
-        aff_buffer(BUFFER_ADD,"OK\n");
+        screen_buffer_add("OK\n");
         if(partition->upart_type==UP_FAT32)
         {
           opt_O=1;
@@ -705,22 +737,22 @@ int fat32_boot_sector(disk_t *disk_car, partition_t *partition, const int verbos
         }
         else
         {
-          aff_buffer(BUFFER_ADD,"Warning: valid FAT bootsector but not a FAT32 one!");
+          screen_buffer_add("Warning: valid FAT bootsector but not a FAT32 one!");
         }
       }
       else
       {
-        aff_buffer(BUFFER_ADD,"Bad\n");
+        screen_buffer_add("Bad\n");
       }
-      aff_buffer(BUFFER_ADD,"\nBackup boot sector\n");
+      screen_buffer_add("\nBackup boot sector\n");
       if(disk_car->read(disk_car,3*disk_car->sector_size, buffer_backup_bs, partition->part_offset+6*disk_car->sector_size)!=0)
       {
-	aff_buffer(BUFFER_ADD,"fat32_boot_sector: Can't read backup boot sector.\n");
+	screen_buffer_add("fat32_boot_sector: Can't read backup boot sector.\n");
 	memset(buffer_backup_bs,0,3*disk_car->sector_size);
       }
       if(test_FAT(disk_car,(struct fat_boot_sector *)buffer_backup_bs,partition,verbose,0)==0)
       {
-	aff_buffer(BUFFER_ADD,"OK\n");
+	screen_buffer_add("OK\n");
 	if(partition->upart_type==UP_FAT32)
 	{
 	  opt_B=1;
@@ -728,32 +760,32 @@ int fat32_boot_sector(disk_t *disk_car, partition_t *partition, const int verbos
 	}
 	else
 	{
-	  aff_buffer(BUFFER_ADD,"Warning: valid FAT backup bootsector but not a FAT32 one!");
+	  screen_buffer_add("Warning: valid FAT backup bootsector but not a FAT32 one!");
 	}
       }
       else
       {
-        aff_buffer(BUFFER_ADD,"Bad\n");
+        screen_buffer_add("Bad\n");
       }
-      aff_buffer(BUFFER_ADD,"\n");
+      screen_buffer_add("\n");
       if((memcmp(buffer_bs,buffer_backup_bs,0x3E8)==0)&&(memcmp(buffer_bs+0x3F0,buffer_backup_bs+0x3F0,0x600-0x3F0))==0)
       {
-	aff_buffer(BUFFER_ADD,"Sectors are identical.\n");
+	screen_buffer_add("Sectors are identical.\n");
 	opt_over=0;
       }
       else
       {
 	if(memcmp(buffer_bs,buffer_backup_bs,0x200)!=0)
-	  aff_buffer(BUFFER_ADD,"First sectors (Boot code and partition information) are not identical.\n");
+	  screen_buffer_add("First sectors (Boot code and partition information) are not identical.\n");
 	if((memcmp(buffer_bs+disk_car->sector_size, buffer_backup_bs+disk_car->sector_size,0x1E8)!=0)||
 	    (memcmp(buffer_bs+disk_car->sector_size+0x1F0, buffer_backup_bs+disk_car->sector_size+0x1F0,0x200-0x1F0)!=0))
-	  aff_buffer(BUFFER_ADD,"Second sectors (cluster information) are not identical.\n");
+	  screen_buffer_add("Second sectors (cluster information) are not identical.\n");
 	if(memcmp(buffer_bs+2*disk_car->sector_size, buffer_backup_bs+2*disk_car->sector_size,0x200)!=0)
-	  aff_buffer(BUFFER_ADD,"Third sectors (Second part of boot code) are not identical.\n");
+	  screen_buffer_add("Third sectors (Second part of boot code) are not identical.\n");
       }
-      aff_buffer(BUFFER_ADD,"\n");
-      aff_buffer(BUFFER_ADD,"A valid FAT Boot sector must be present in order to access\n");
-      aff_buffer(BUFFER_ADD,"any data; even if the partition is not bootable.\n");
+      screen_buffer_add("\n");
+      screen_buffer_add("A valid FAT Boot sector must be present in order to access\n");
+      screen_buffer_add("any data; even if the partition is not bootable.\n");
       if(opt_over!=0)
       {
 	if(opt_B!=0 && opt_O!=0)
@@ -922,7 +954,7 @@ int ntfs_boot_sector(disk_t *disk_car, partition_t *partition, const int verbose
     { 'P', "Previous",""},
     { 'N', "Next","" },
     { 'Q', "Quit","Return to Advanced menu"},
-    { 'L', "List", "List directories and files" },
+    { 'L', "List", "List directories and files, copy data from NTFS" },
     { 'O', "Org. BS","Copy boot sector over backup sector"},
     { 'B', "Backup BS","Copy backup boot sector over boot sector"},
     { 'R', "Rebuild BS","Rebuild boot sector"},
@@ -940,7 +972,7 @@ int ntfs_boot_sector(disk_t *disk_car, partition_t *partition, const int verbose
   {
     unsigned int menu=0;
     int command;
-    aff_buffer(BUFFER_RESET,"Q");
+    screen_buffer_reset();
     if(rescan==1)
     {
       int identical_sectors=0;
@@ -956,52 +988,52 @@ int ntfs_boot_sector(disk_t *disk_car, partition_t *partition, const int verbose
 #endif
       log_info("\nntfs_boot_sector\n");
       log_partition(disk_car,partition);
-      aff_buffer(BUFFER_ADD,"Boot sector\n");
+      screen_buffer_add("Boot sector\n");
       if(disk_car->read(disk_car,NTFS_BOOT_SECTOR_SIZE, buffer_bs, partition->part_offset)!=0)
       {
-	aff_buffer(BUFFER_ADD,"ntfs_boot_sector: Can't read boot sector.\n");
+	screen_buffer_add("ntfs_boot_sector: Can't read boot sector.\n");
 	memset(buffer_bs,0,NTFS_BOOT_SECTOR_SIZE);
       }
       if(test_NTFS(disk_car,(struct ntfs_boot_sector*)buffer_bs,partition,verbose,0)==0)
       {
-	aff_buffer(BUFFER_ADD,"Status: OK\n");
+	screen_buffer_add("Status: OK\n");
 	opt_O=1;
       }
       else
       {
-	aff_buffer(BUFFER_ADD,"Status: Bad\n");
+	screen_buffer_add("Status: Bad\n");
       }
-      aff_buffer(BUFFER_ADD,"\nBackup boot sector\n");
+      screen_buffer_add("\nBackup boot sector\n");
       if(disk_car->read(disk_car,NTFS_BOOT_SECTOR_SIZE, buffer_backup_bs, partition->part_offset+partition->part_size-disk_car->sector_size)!=0)
       {
-	aff_buffer(BUFFER_ADD,"ntfs_boot_sector: Can't read backup boot sector.\n");
+	screen_buffer_add("ntfs_boot_sector: Can't read backup boot sector.\n");
 	memset(buffer_backup_bs,0,NTFS_BOOT_SECTOR_SIZE);
       }
       if(test_NTFS(disk_car,(struct ntfs_boot_sector*)buffer_backup_bs,partition,verbose,0)==0)
       {
-	aff_buffer(BUFFER_ADD,"Status: OK\n");
+	screen_buffer_add("Status: OK\n");
 	opt_B=1;
       }
       else
       {
-	aff_buffer(BUFFER_ADD,"Status: Bad\n");
+	screen_buffer_add("Status: Bad\n");
       }
-      aff_buffer(BUFFER_ADD,"\n");
+      screen_buffer_add("\n");
       if(memcmp(buffer_bs,buffer_backup_bs,NTFS_BOOT_SECTOR_SIZE)==0)
       {
 	log_ntfs_info((const struct ntfs_boot_sector *)buffer_bs);
-	aff_buffer(BUFFER_ADD,"Sectors are identical.\n");
+	screen_buffer_add("Sectors are identical.\n");
 	identical_sectors=1;
       }
       else
       {
 	log_ntfs2_info((const struct ntfs_boot_sector *)buffer_bs, (const struct ntfs_boot_sector *)buffer_backup_bs);
-	aff_buffer(BUFFER_ADD,"Sectors are not identical.\n");
+	screen_buffer_add("Sectors are not identical.\n");
 	identical_sectors=0;
       }
-      aff_buffer(BUFFER_ADD,"\n");
-      aff_buffer(BUFFER_ADD,"A valid NTFS Boot sector must be present in order to access\n");
-      aff_buffer(BUFFER_ADD,"any data; even if the partition is not bootable.\n");
+      screen_buffer_add("\n");
+      screen_buffer_add("A valid NTFS Boot sector must be present in order to access\n");
+      screen_buffer_add("any data; even if the partition is not bootable.\n");
       if(opt_B!=0 && opt_O!=0)
       {
 //      options="DRMLU";
@@ -1186,7 +1218,7 @@ int HFS_HFSP_boot_sector(disk_t *disk_car, partition_t *partition, const int ver
   {
     unsigned int menu=0;
     int command;
-    aff_buffer(BUFFER_RESET,"Q");
+    screen_buffer_reset();
     if(rescan==1)
     {
       int opt_over=0;
@@ -1203,55 +1235,55 @@ int HFS_HFSP_boot_sector(disk_t *disk_car, partition_t *partition, const int ver
 #endif
       log_info("\nHFS_HFSP_boot_sector\n");
       log_partition(disk_car,partition);
-      aff_buffer(BUFFER_ADD,"Superblock\n");
+      screen_buffer_add("Volume header\n");
       if(disk_car->read(disk_car,HFSP_BOOT_SECTOR_SIZE, buffer_bs, partition->part_offset+0x400)!=0)
       {
-	aff_buffer(BUFFER_ADD,"Bad: can't read HFS/HFS+ superblock.\n");
+	screen_buffer_add("Bad: can't read HFS/HFS+ volume header.\n");
 	memset(buffer_bs,0,HFSP_BOOT_SECTOR_SIZE);
       }
       else if(test_HFSP(disk_car,(const struct hfsp_vh*)buffer_bs,partition,verbose,0)==0)
       {
-	aff_buffer(BUFFER_ADD,"HFS+ OK\n");
+	screen_buffer_add("HFS+ OK\n");
 	opt_O=1;
 	opt_over=1;
       }
       else if(test_HFS(disk_car,(const hfs_mdb_t*)buffer_bs,partition,verbose,0)==0)
       {
-	aff_buffer(BUFFER_ADD,"HFS Ok\n");
+	screen_buffer_add("HFS Ok\n");
 	opt_O=1;
 	opt_over=1;
       }
       else
-	aff_buffer(BUFFER_ADD,"Bad\n");
-      aff_buffer(BUFFER_ADD,"\nBackup superblock\n");
+	screen_buffer_add("Bad\n");
+      screen_buffer_add("\nBackup volume header\n");
       if(disk_car->read(disk_car,HFSP_BOOT_SECTOR_SIZE, buffer_backup_bs, partition->part_offset+partition->part_size-0x400)!=0)
       {
-	aff_buffer(BUFFER_ADD,"Bad: can't read HFS/HFS+ backup superblock.\n");
+	screen_buffer_add("Bad: can't read HFS/HFS+ backup volume header.\n");
 	memset(buffer_backup_bs,0,HFSP_BOOT_SECTOR_SIZE);
       }
       else if(test_HFSP(disk_car,(const struct hfsp_vh*)buffer_backup_bs,partition,verbose,0)==0)
       {
-	aff_buffer(BUFFER_ADD,"HFS+ OK\n");
+	screen_buffer_add("HFS+ OK\n");
 	opt_B=1;
 	opt_over=1;
       }
       else if(test_HFS(disk_car,(const hfs_mdb_t*)buffer_backup_bs,partition,verbose,0)==0)
       {
-	aff_buffer(BUFFER_ADD,"HFS Ok\n");
+	screen_buffer_add("HFS Ok\n");
 	opt_B=1;
 	opt_over=1;
       }
       else
-	aff_buffer(BUFFER_ADD,"Bad\n");
-      aff_buffer(BUFFER_ADD,"\n");
+	screen_buffer_add("Bad\n");
+      screen_buffer_add("\n");
       if(memcmp(buffer_bs,buffer_backup_bs,HFSP_BOOT_SECTOR_SIZE)==0)
       {
-	aff_buffer(BUFFER_ADD,"Sectors are identical.\n");
+	screen_buffer_add("Sectors are identical.\n");
 	opt_over=0;
       }
       else
       {
-	aff_buffer(BUFFER_ADD,"Sectors are not identical.\n");
+	screen_buffer_add("Sectors are not identical.\n");
       }
       if(opt_over!=0)
       {
@@ -1304,24 +1336,24 @@ int HFS_HFSP_boot_sector(disk_t *disk_car, partition_t *partition, const int ver
         free(buffer_backup_bs);
 	return 0;
       case 'O': /* O : copy original superblock over backup boot */
-	if(ask_confirmation("Copy original HFS/HFS+ superblock over backup boot, confirm ? (Y/N)")!=0)
+	if(ask_confirmation("Copy original HFS/HFS+ volume header over backup, confirm ? (Y/N)")!=0)
 	{
 	  log_info("copy original superblock over backup boot\n");
 	  if(disk_car->write(disk_car,HFSP_BOOT_SECTOR_SIZE, buffer_bs, partition->part_offset+partition->part_size-0x400)!=0)
 	  {
-	    display_message("Write error: Can't overwrite HFS/HFS+ backup superblock\n");
+	    display_message("Write error: Can't overwrite HFS/HFS+ backup volume header\n");
 	  }
           disk_car->sync(disk_car);
 	  rescan=1;
 	}
 	break;
       case 'B': /* B : copy backup superblock over main superblock */
-	if(ask_confirmation("Copy backup HFS/HFS+ superblock over main superblock, confirm ? (Y/N)")!=0)
+	if(ask_confirmation("Copy backup HFS/HFS+ volume header over main volume header, confirm ? (Y/N)")!=0)
 	{
 	  log_info("copy backup superblock over main superblock\n");
 	  if(disk_car->write(disk_car,HFSP_BOOT_SECTOR_SIZE, buffer_backup_bs, partition->part_offset+0x400)!=0)
 	  {
-	    display_message("Write error: Can't overwrite HFS/HFS+ main superblock\n");
+	    display_message("Write error: Can't overwrite HFS/HFS+ main volume header\n");
 	  }
           disk_car->sync(disk_car);
 	  rescan=1;
