@@ -212,7 +212,7 @@ struct efi_guid_s
 #define TESTDISK_O_READAHEAD_32K 010
 #define TESTDISK_O_ALL		020
 
-enum upart_type { UP_UNK, UP_BEOS, UP_CRAMFS, UP_EXT2, UP_EXT3, UP_EXTENDED, UP_FAT12, UP_FAT16, UP_FAT32, UP_FATX, UP_FREEBSD,  UP_HFS, UP_HFSP, UP_HFSX, UP_HPFS, UP_JFS, UP_LINSWAP, UP_LINSWAP2, UP_LUKS, UP_LVM, UP_LVM2, UP_MD, UP_MD1, UP_NETWARE, UP_NTFS, UP_OPENBSD, UP_OS2MB, UP_RFS, UP_RFS2, UP_RFS3, UP_RFS4, UP_SUN, UP_SYSV4, UP_UFS, UP_UFS2, UP_XFS, UP_XFS2, UP_XFS3, UP_XFS4};
+enum upart_type { UP_UNK, UP_BEOS, UP_CRAMFS, UP_EXT2, UP_EXT3, UP_EXT4, UP_EXTENDED, UP_FAT12, UP_FAT16, UP_FAT32, UP_FATX, UP_FREEBSD,  UP_HFS, UP_HFSP, UP_HFSX, UP_HPFS, UP_JFS, UP_LINSWAP, UP_LINSWAP2, UP_LUKS, UP_LVM, UP_LVM2, UP_MD, UP_MD1, UP_NETWARE, UP_NTFS, UP_OPENBSD, UP_OS2MB, UP_RFS, UP_RFS2, UP_RFS3, UP_RFS4, UP_SUN, UP_SYSV4, UP_UFS, UP_UFS2, UP_XFS, UP_XFS2, UP_XFS3, UP_XFS4};
 typedef enum upart_type upart_type_t;
 enum status_type { STATUS_DELETED, STATUS_PRIM, STATUS_PRIM_BOOT, STATUS_LOG, STATUS_EXT, STATUS_EXT_IN_EXT};
 typedef enum status_type status_type_t;
@@ -356,7 +356,6 @@ char test_key(void);
 unsigned int up2power(const unsigned int number);
 void my_sort(void *base, size_t nmemb, size_t size, int (*compar)(const void *, const void *));
 void set_part_name(partition_t *partition,const char *src,const int max_size);
-int rebuild_FAT_BS(disk_t *disk_car,partition_t *partition, const int verbose, const int dump_ind,const int interface, const unsigned int expert, char**current_cmd);
 #ifndef BSD_MAXPARTITIONS
 #define	BSD_MAXPARTITIONS	8
 #endif
@@ -417,23 +416,22 @@ int vsnprintf(char *str, size_t size, const char *format, va_list ap);
 int strncasecmp(const char * s1, const char * s2, size_t len);
 #endif
 void create_dir(const char *dir_name, const unsigned int is_dir_name);
-unsigned int filename_convert(char *dst, const char*src, const unsigned int n);
+char *gen_local_filename(const char *dir, const char*src);
+/*
+ * td_min()/td_max() macros that also do
+ * strict type-checking.. See the
+ * "unnecessary" pointer comparison.
+ * Comes from Linux kernel
+ */
+#define td_min(x,y) ({ \
+        typeof(x) _x = (x);     \
+        typeof(y) _y = (y);     \
+        (void) (&_x == &_y);            \
+        _x < _y ? _x : _y; })
 
-#ifdef DJGPP
-    #define TESTDISK_OS "Dos version"
-#elif defined(TARGET_BSD)
-    #define TESTDISK_OS "BSD version"
-#elif defined(TARGET_LINUX)
-    #define TESTDISK_OS "Linux version"
-#elif defined(TARGET_SOLARIS)
-    #define TESTDISK_OS "Solaris version"
-#elif defined(__CYGWIN__) || defined(__MINGW32__)
-    #define TESTDISK_OS "Windows version"
-#elif defined(__APPLE__)
-    #define TESTDISK_OS "Apple version"
-#elif defined(__OS2__)
-    #define TESTDISK_OS "OS2 version"
-#else
-    #define TESTDISK_OS "Undefined OS"
-#endif
+#define td_max(x,y) ({ \
+        typeof(x) _x = (x);     \
+        typeof(y) _y = (y);     \
+        (void) (&_x == &_y);            \
+        _x > _y ? _x : _y; })
 #endif
