@@ -52,7 +52,7 @@ static void interface_editor_ncurses(disk_t *disk_car)
 {
   int done = 0;
   uint64_t hd_offset=0;
-  unsigned char *buffer=MALLOC(disk_car->sector_size);
+  unsigned char *buffer=(unsigned char *)MALLOC(disk_car->sector_size);
   log_info("%s\n",disk_car->description(disk_car));
   while (done==0)
   {
@@ -63,7 +63,7 @@ static void interface_editor_ncurses(disk_t *disk_car)
       { 'Q', "Quit",""},
       { 0, NULL, NULL }
     };
-    switch ( wmenuSelect(stdscr,INTER_DUMP_Y, INTER_DUMP_X, menuEditor, 8, "CDQ", MENU_HORIZ | MENU_BUTTON, 0))
+    switch ( wmenuSelect(stdscr, 24, INTER_DUMP_Y, INTER_DUMP_X, menuEditor, 8, "CDQ", MENU_HORIZ | MENU_BUTTON, 0))
     {
       case 'c':
       case 'C':
@@ -233,7 +233,7 @@ static int dump_editor(const unsigned char *nom_dump,const unsigned int lng, con
 		  wprintw(stdscr,"%c",  car);
 	  }
 	}
-	switch (wmenuSelect(stdscr,INTER_DUMP_Y, INTER_DUMP_X, menuDump, 8, "PNQ", MENU_HORIZ | MENU_BUTTON | MENU_ACCEPT_OTHERS, menu))
+	switch (wmenuSelect(stdscr, 24, INTER_DUMP_Y, INTER_DUMP_X, menuDump, 8, "PNQ", MENU_HORIZ | MENU_BUTTON | MENU_ACCEPT_OTHERS, menu))
 	{
 	  case 'p':
 	  case 'P':
