@@ -2,7 +2,7 @@
 
     File: ext2_dir.c
 
-    Copyright (C) 1998-2007 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2008 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -27,6 +27,9 @@
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
+#ifdef HAVE_ERRNO_H
+#include <errno.h>
+#endif
 #ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
 #endif
@@ -66,6 +69,7 @@ static errcode_t my_flush(io_channel channel);
 
 static io_channel alloc_io_channel(disk_t *disk_car,my_data_t *my_data);
 static void dir_partition_ext2_close(dir_data_t *dir_data);
+static int ext2_copy(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const file_data_t *file);
 
 static struct struct_io_manager my_struct_manager = {
         magic: EXT2_ET_MAGIC_IO_MANAGER,
@@ -198,10 +202,12 @@ static int list_dir_proc2(ext2_ino_t dir,
   ext2_ino_t		ino;
   unsigned int		thislen;
   struct ext2_dir_struct *ls = (struct ext2_dir_struct *) private;
-  file_data_t *new_file=MALLOC(sizeof(*new_file));
+  file_data_t *new_file;
+  if(entry==DIRENT_DELETED_FILE && (ls->dir_data->param & FLAG_LIST_DELETED)==0)
+    return 0;
+  new_file=(file_data_t *)MALLOC(sizeof(*new_file));
   new_file->prev=ls->current_file;
   new_file->next=NULL;
-
   thislen = ((dirent->name_len & 0xFF) < EXT2_NAME_LEN) ?
     (dirent->name_len & 0xFF) : EXT2_NAME_LEN;
   if(thislen>DIR_NAME_LEN)
@@ -220,6 +226,10 @@ static int list_dir_proc2(ext2_ino_t dir,
   } else {
     memset(&inode, 0, sizeof(struct ext2_inode));
   }
+  if(entry==DIRENT_DELETED_FILE)
+    new_file->status=FILE_STATUS_DELETED;
+  else
+    new_file->status=0;
   new_file->filestat.st_dev=0;
   new_file->filestat.st_ino=ino;
   new_file->filestat.st_mode=inode.i_mode;
@@ -264,6 +274,71 @@ static void dir_partition_ext2_close(dir_data_t *dir_data)
   /* ext2fs_close call the close function that freed my_data */
   free(ls);
 }
+
+static int ext2_copy(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const file_data_t *file)
+{
+  int error=0;
+  FILE *f_out;
+  struct ext2_dir_struct *ls = (struct ext2_dir_struct *)dir_data->private_dir_data;
+  char *new_file;
+  new_file=gen_local_filename(dir_data->local_dir, dir_data->current_directory);
+  f_out=create_file(new_file);
+  if(!f_out)
+  {
+    log_critical("Can't create file %s: %s\n", new_file, strerror(errno));
+    free(new_file);
+    return -4;
+  }
+  {
+    errcode_t retval;
+    struct ext2_inode       inode;
+    char            buffer[8192];
+    ext2_file_t     e2_file;
+    int             nbytes; 
+    unsigned int    got;
+
+    if (ext2fs_read_inode(ls->current_fs, file->filestat.st_ino, &inode)!=0)
+    {
+      free(new_file);
+      return -1;
+    }
+
+    retval = ext2fs_file_open(ls->current_fs, file->filestat.st_ino, 0, &e2_file);
+    if (retval) {
+      log_error("Error while opening ext2 file %s\n", dir_data->current_directory);
+      free(new_file);
+      return -2;
+    }
+    while (1)
+    {
+      retval = ext2fs_file_read(e2_file, buffer, sizeof(buffer), &got);
+      if (retval)
+      {
+	log_error("Error while reading ext2 file %s\n", dir_data->current_directory);
+	error=-3;
+      }
+      if (got == 0)
+	break;
+      nbytes = fwrite(buffer, 1, got, f_out);
+      if ((unsigned) nbytes != got)
+      {
+	log_error("Error while writing file %s\n", new_file);
+      error=-5;
+      }
+    }
+    retval = ext2fs_file_close(e2_file);
+    if (retval)
+    {
+      log_error("Error while closing ext2 file\n");
+      error=-6;
+    }
+    fclose(f_out);
+    set_date(new_file, file->filestat.st_atime, file->filestat.st_mtime);
+    set_mode(new_file, file->filestat.st_mode);
+  }
+  free(new_file);
+  return error;
+}
 #endif
 
 int dir_partition_ext2_init(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const int verbose)
@@ -275,8 +350,9 @@ int dir_partition_ext2_init(disk_t *disk_car, const partition_t *partition, dir_
   ls->dir_list=NULL;
   ls->current_file=NULL;
   /*  ls->flags = DIRENT_FLAG_INCLUDE_EMPTY; */
-  ls->flags = 0;
-  my_data=MALLOC(sizeof(*my_data));
+  ls->flags = DIRENT_FLAG_INCLUDE_REMOVED;
+  ls->dir_data=dir_data;
+  my_data=(my_data_t *)MALLOC(sizeof(*my_data));
   my_data->partition=partition;
   my_data->disk_car=disk_car;
   ioch=alloc_io_channel(disk_car,my_data);
@@ -289,9 +365,11 @@ int dir_partition_ext2_init(disk_t *disk_car, const partition_t *partition, dir_
   }
   strncpy(dir_data->current_directory,"/",sizeof(dir_data->current_directory));
   dir_data->current_inode=EXT2_ROOT_INO;
+  dir_data->param=FLAG_LIST_DELETED;
   dir_data->verbose=verbose;
+  dir_data->capabilities=CAPA_LIST_DELETED;
   dir_data->get_dir=ext2_dir;
-  dir_data->copy_file=NULL;
+  dir_data->copy_file=ext2_copy;
   dir_data->close=&dir_partition_ext2_close;
   dir_data->local_dir=NULL;
   dir_data->private_dir_data=ls;
