@@ -68,7 +68,7 @@ static void fat_set_part_name(partition_t *partition,const unsigned char *src,co
 static int log_fat_info(const struct fat_boot_sector*fh1, const upart_type_t upart_type, const unsigned int sector_size)
 {
   log_info("sector_size  %u\n", fat_sector_size(fh1));
-  log_info("cluster_size %u\n", fh1->cluster_size);
+  log_info("cluster_size %u\n", fh1->sectors_per_cluster);
   log_info("reserved     %u\n", le16(fh1->reserved));
   log_info("fats         %u\n", fh1->fats);
   log_info("dir_entries  %u\n", get_dir_entries(fh1));
@@ -117,7 +117,7 @@ static int dump_fat_info_ncurses(const struct fat_boot_sector*fh1, const upart_t
       wprintw(stdscr,"Not a FAT\n");
       return 0;
   }
-  wprintw(stdscr,"cluster_size %u\n", fh1->cluster_size);
+  wprintw(stdscr,"cluster_size %u\n", fh1->sectors_per_cluster);
   wprintw(stdscr,"reserved     %u\n", le16(fh1->reserved));
   if(sectors(fh1)!=0)
     wprintw(stdscr,"sectors      %u\n", sectors(fh1));
@@ -175,7 +175,7 @@ static int dump_2fat_info_ncurses(const struct fat_boot_sector*fh1, const struct
       wprintw(stdscr,"Not a FAT\n");
       return 1;
   }
-  wprintw(stdscr,"cluster_size %u %u\n", fh1->cluster_size, fh2->cluster_size);
+  wprintw(stdscr,"cluster_size %u %u\n", fh1->sectors_per_cluster, fh2->sectors_per_cluster);
   wprintw(stdscr,"reserved     %u %u\n", le16(fh1->reserved),le16(fh2->reserved));
   if(sectors(fh1)!=0 || sectors(fh2)!=0)
     wprintw(stdscr,"sectors      %u %u\n", sectors(fh1), sectors(fh2));
@@ -237,7 +237,7 @@ int log_fat2_info(const struct fat_boot_sector*fh1, const struct fat_boot_sector
       return 1;
   }
   log_info("sector_size  %u %u\n", fat_sector_size(fh1),fat_sector_size(fh2));
-  log_info("cluster_size %u %u\n", fh1->cluster_size,fh2->cluster_size);
+  log_info("cluster_size %u %u\n", fh1->sectors_per_cluster,fh2->sectors_per_cluster);
   log_info("reserved     %u %u\n", le16(fh1->reserved),le16(fh2->reserved));
   log_info("fats         %u %u\n", fh1->fats,fh2->fats);
   log_info("dir_entries  %u %u\n", get_dir_entries(fh1),get_dir_entries(fh2));
@@ -281,10 +281,10 @@ int log_fat2_info(const struct fat_boot_sector*fh1, const struct fat_boot_sector
 int check_FAT(disk_t *disk_car,partition_t *partition,const int verbose)
 {
   unsigned char *buffer;
-  buffer=MALLOC(3*disk_car->sector_size);
+  buffer=(unsigned char *)MALLOC(3*disk_car->sector_size);
   if(disk_car->read(disk_car,3*disk_car->sector_size, buffer, partition->part_offset)!=0)
   {
-    aff_buffer(BUFFER_ADD,"check_FAT: can't read FAT boot sector\n");
+    screen_buffer_add("check_FAT: can't read FAT boot sector\n");
     log_error("check_FAT: can't read FAT boot sector\n");
     free(buffer);
     return 1;
@@ -301,7 +301,7 @@ int check_FAT(disk_t *disk_car,partition_t *partition,const int verbose)
     return 1;
   }
   set_FAT_info(disk_car,(const struct fat_boot_sector *)buffer,partition,verbose);
-  /*  aff_buffer(BUFFER_ADD,"Ok\n"); */
+  /*  screen_buffer_add("Ok\n"); */
   free(buffer);
   return 0;
 }
@@ -481,7 +481,7 @@ unsigned int fat32_get_prev_cluster(disk_t *disk_car,const partition_t *partitio
   const uint32_t *p32;
   uint64_t hd_offset=partition->part_offset+(uint64_t)fat_offset*disk_car->sector_size;
   unsigned int prev_cluster;
-  unsigned char *buffer=MALLOC(disk_car->sector_size);
+  unsigned char *buffer=(unsigned char *)MALLOC(disk_car->sector_size);
   p32=(const uint32_t*)buffer;
 
   for(prev_cluster=2;prev_cluster<=no_of_cluster+1;prev_cluster++)
@@ -541,17 +541,17 @@ int test_FAT(disk_t *disk_car,const struct fat_boot_sector *fat_header, partitio
 #endif
   if(!((fat_header->ignored[0]==0xeb && fat_header->ignored[2]==0x90)||fat_header->ignored[0]==0xe9))
   {
-    aff_buffer(BUFFER_ADD,msg_CHKFAT_BAD_JUMP);
+    screen_buffer_add(msg_CHKFAT_BAD_JUMP);
     log_error(msg_CHKFAT_BAD_JUMP);
     return 1;
   }
   if(fat_sector_size(fat_header)!=disk_car->sector_size)
   {
-    aff_buffer(BUFFER_ADD,"check_FAT: Incorrect number of bytes per sector %u (FAT) != %u (HD)\n",fat_sector_size(fat_header),disk_car->sector_size);
+    screen_buffer_add("check_FAT: Incorrect number of bytes per sector %u (FAT) != %u (HD)\n",fat_sector_size(fat_header),disk_car->sector_size);
     log_error("check_FAT: Incorrect number of bytes per sector %u (FAT) != %u (HD)\n",fat_sector_size(fat_header),disk_car->sector_size);
     return 1;
   }
-  switch(fat_header->cluster_size)
+  switch(fat_header->sectors_per_cluster)
   {
     case 1:
     case 2:
@@ -563,32 +563,32 @@ int test_FAT(disk_t *disk_car,const struct fat_boot_sector *fat_header, partitio
     case 128:
       break;
     default:
-      aff_buffer(BUFFER_ADD,msg_CHKFAT_SECT_CLUSTER);
+      screen_buffer_add(msg_CHKFAT_SECT_CLUSTER);
       log_error(msg_CHKFAT_SECT_CLUSTER);
       return 1;
   }
   switch(fat_header->fats)
   {
     case 1:
-      aff_buffer(BUFFER_ADD,"check_FAT: Unusual, only one FAT\n");
+      screen_buffer_add("check_FAT: Unusual, only one FAT\n");
       log_warning("check_FAT: Unusual, only one FAT\n");
       break;
     case 2:
       break;
     default:
-      aff_buffer(BUFFER_ADD,"check_FAT: Bad number %u of FAT\n", fat_header->fats);
+      screen_buffer_add("check_FAT: Bad number %u of FAT\n", fat_header->fats);
       log_error("check_FAT: Bad number %u of FAT\n", fat_header->fats);
       return 1;
   }
   if(fat_header->media!=0xF0 && fat_header->media<0xF8)
   {	/* Legal values are 0xF0, 0xF8-0xFF */
-    aff_buffer(BUFFER_ADD,"check_FAT: Bad media descriptor (0x%2x!=0xf8)\n",fat_header->media);
+    screen_buffer_add("check_FAT: Bad media descriptor (0x%2x!=0xf8)\n",fat_header->media);
     log_error("check_FAT: Bad media descriptor (0x%2x!=0xf8)\n",fat_header->media);
     return 1;
   }
   if(fat_header->media!=0xF8)
   { /* the only value I have ever seen is 0xF8 */
-    aff_buffer(BUFFER_ADD,"check_FAT: Unusual media descriptor (0x%2x!=0xf8)\n",fat_header->media);
+    screen_buffer_add("check_FAT: Unusual media descriptor (0x%2x!=0xf8)\n",fat_header->media);
     log_warning("check_FAT: Unusual media descriptor (0x%2x!=0xf8)\n",fat_header->media);
   }
   fat_length=le16(fat_header->fat_length)>0?le16(fat_header->fat_length):le32(fat_header->fat32_length);
@@ -596,8 +596,8 @@ int test_FAT(disk_t *disk_car,const struct fat_boot_sector *fat_header, partitio
   start_fat1=le16(fat_header->reserved);
   start_fat2=start_fat1+(fat_header->fats>1?fat_length:0);
   start_data=start_fat1+fat_header->fats*fat_length+(get_dir_entries(fat_header)*32+disk_car->sector_size-1)/disk_car->sector_size;
-  no_of_cluster=(part_size-start_data)/fat_header->cluster_size;
-  end_data=start_data+no_of_cluster*fat_header->cluster_size-1;
+  no_of_cluster=(part_size-start_data)/fat_header->sectors_per_cluster;
+  end_data=start_data+no_of_cluster*fat_header->sectors_per_cluster-1;
   if(verbose>1)
   {
     log_info("number of cluster = %lu\n",no_of_cluster);
@@ -614,23 +614,23 @@ int test_FAT(disk_t *disk_car,const struct fat_boot_sector *fat_header, partitio
     }
     if(sectors(fat_header)==0)
     {
-      aff_buffer(BUFFER_ADD,msg_CHKFAT_SIZE);
+      screen_buffer_add(msg_CHKFAT_SIZE);
       log_error(msg_CHKFAT_SIZE);
     }
     if(le16(fat_header->reserved)!=1)
     {
-      aff_buffer(BUFFER_ADD,"check_FAT: Unusual number of reserved sectors %u (FAT), should be 1.\n",le16(fat_header->reserved));
+      screen_buffer_add("check_FAT: Unusual number of reserved sectors %u (FAT), should be 1.\n",le16(fat_header->reserved));
       log_warning("check_FAT: Unusual number of reserved sectors %u (FAT), should be 1.\n",le16(fat_header->reserved));
     }
     if((get_dir_entries(fat_header)==0)||(get_dir_entries(fat_header)%16!=0))
     {
-      aff_buffer(BUFFER_ADD,msg_CHKFAT_ENTRY);
+      screen_buffer_add(msg_CHKFAT_ENTRY);
       log_error(msg_CHKFAT_ENTRY);
       return 1;
     }
     if((le16(fat_header->fat_length)>256)||(le16(fat_header->fat_length)==0))
     {
-      aff_buffer(BUFFER_ADD,msg_CHKFAT_SECTPFAT);
+      screen_buffer_add(msg_CHKFAT_SECTPFAT);
       log_error(msg_CHKFAT_SECTPFAT);
       return 1;
     }
@@ -639,7 +639,7 @@ int test_FAT(disk_t *disk_car,const struct fat_boot_sector *fat_header, partitio
     partition->upart_type=UP_FAT12;
     if(memcmp(buffer+FAT_NAME1,"FAT12   ",8)!=0) /* 2 Mo max */
     {
-      aff_buffer(BUFFER_ADD,"Should be marked as FAT12\n");
+      screen_buffer_add("Should be marked as FAT12\n");
       log_warning("Should be marked as FAT12\n");
     }
   }
@@ -655,18 +655,18 @@ int test_FAT(disk_t *disk_car,const struct fat_boot_sector *fat_header, partitio
     }
     if(le16(fat_header->reserved)!=1)
     {
-      aff_buffer(BUFFER_ADD,"check_FAT: Unusual number of reserved sectors %u (FAT), should be 1.\n",le16(fat_header->reserved));
+      screen_buffer_add("check_FAT: Unusual number of reserved sectors %u (FAT), should be 1.\n",le16(fat_header->reserved));
       log_warning("check_FAT: Unusual number of reserved sectors %u (FAT), should be 1.\n",le16(fat_header->reserved));
     }
     if(le16(fat_header->fat_length)==0)
     {
-      aff_buffer(BUFFER_ADD,msg_CHKFAT_SECTPFAT);
+      screen_buffer_add(msg_CHKFAT_SECTPFAT);
       log_error(msg_CHKFAT_SECTPFAT);
       return 1;
     }
     if((get_dir_entries(fat_header)==0)||(get_dir_entries(fat_header)%16!=0))
     {
-      aff_buffer(BUFFER_ADD,msg_CHKFAT_ENTRY);
+      screen_buffer_add(msg_CHKFAT_ENTRY);
       log_error(msg_CHKFAT_ENTRY);
       return 1;
     }
@@ -675,7 +675,7 @@ int test_FAT(disk_t *disk_car,const struct fat_boot_sector *fat_header, partitio
     partition->upart_type=UP_FAT16;
     if(memcmp(buffer+FAT_NAME1,"FAT16   ",8)!=0)
     {
-      aff_buffer(BUFFER_ADD,"Should be marked as FAT16\n");
+      screen_buffer_add("Should be marked as FAT16\n");
       log_warning("Should be marked as FAT16\n");
     }
   }
@@ -691,33 +691,33 @@ int test_FAT(disk_t *disk_car,const struct fat_boot_sector *fat_header, partitio
     }
     if(sectors(fat_header)!=0)
     {
-      aff_buffer(BUFFER_ADD,msg_CHKFAT_SIZE);
+      screen_buffer_add(msg_CHKFAT_SIZE);
       log_error(msg_CHKFAT_SIZE);
       return 1;
     }
     if(get_dir_entries(fat_header)!=0)
     {
-      aff_buffer(BUFFER_ADD,msg_CHKFAT_ENTRY);
+      screen_buffer_add(msg_CHKFAT_ENTRY);
       log_error(msg_CHKFAT_ENTRY);
       return 1;
     }
     if((fat_header->version[0]!=0) || (fat_header->version[1]!=0))
     {
-      aff_buffer(BUFFER_ADD,msg_CHKFAT_BADFAT32VERSION);
+      screen_buffer_add(msg_CHKFAT_BADFAT32VERSION);
       log_error(msg_CHKFAT_BADFAT32VERSION);
     }
     if((le32(fat_header->root_cluster)<2) ||(le32(fat_header->root_cluster)>=2+no_of_cluster))
     {
-      aff_buffer(BUFFER_ADD,"Bad root_cluster\n");
+      screen_buffer_add("Bad root_cluster\n");
       log_error("Bad root_cluster\n");
       return 1;
     }
-    start_rootdir=start_data+(uint64_t)(le32(fat_header->root_cluster)-2)*fat_header->cluster_size;
+    start_rootdir=start_data+(uint64_t)(le32(fat_header->root_cluster)-2)*fat_header->sectors_per_cluster;
     fat_length_calc=((no_of_cluster+2+disk_car->sector_size/4-1)*4/disk_car->sector_size);
     partition->upart_type=UP_FAT32;
     if(memcmp(buffer+FAT_NAME2,"FAT32   ",8)!=0)
     {
-      aff_buffer(BUFFER_ADD,"Should be marked as FAT32\n");
+      screen_buffer_add("Should be marked as FAT32\n");
       log_warning("Should be marked as FAT32\n");
     }
   }
@@ -725,7 +725,7 @@ int test_FAT(disk_t *disk_car,const struct fat_boot_sector *fat_header, partitio
   {
     if(part_size > partition->part_size/disk_car->sector_size)
     {
-      aff_buffer(BUFFER_ADD, "Error: size boot_sector %lu > partition %lu\n",
+      screen_buffer_add( "Error: size boot_sector %lu > partition %lu\n",
           (long unsigned)part_size,
           (long unsigned)(partition->part_size/disk_car->sector_size));
       log_error("test_FAT size boot_sector %lu > partition %lu\n",
@@ -750,25 +750,25 @@ int test_FAT(disk_t *disk_car,const struct fat_boot_sector *fat_header, partitio
       log_info(" root cluster : %u",(unsigned int)le32(fat_header->root_cluster));
     log_info("\nData : %lu-%lu\n", (long unsigned)start_data, (long unsigned)end_data);
     log_info("sectors : %lu\n", (long unsigned)part_size);
-    log_info("cluster_size : %u\n",fat_header->cluster_size);
+    log_info("cluster_size : %u\n", fat_header->sectors_per_cluster);
     log_info("no_of_cluster : %lu (2 - %lu)\n", no_of_cluster,no_of_cluster+1);
     log_info("fat_length %lu calculated %lu\n",fat_length,fat_length_calc);
   }
   if(fat_length<fat_length_calc)
   {
-    aff_buffer(BUFFER_ADD,msg_CHKFAT_SECTPFAT);
+    screen_buffer_add(msg_CHKFAT_SECTPFAT);
     return 1;
   }
   if(fat_header->fats>1)
     comp_FAT(disk_car,partition,fat_length,le16(fat_header->reserved));
   if(le16(fat_header->heads)!=disk_car->CHS.head+1)
   {
-    aff_buffer(BUFFER_ADD,"Warning: Incorrect number of heads/cylinder %u (FAT) != %u (HD)\n",le16(fat_header->heads),disk_car->CHS.head+1);
+    screen_buffer_add("Warning: Incorrect number of heads/cylinder %u (FAT) != %u (HD)\n",le16(fat_header->heads),disk_car->CHS.head+1);
     log_warning("heads/cylinder %u (FAT) != %u (HD)\n",le16(fat_header->heads),disk_car->CHS.head+1);
   }
   if(le16(fat_header->secs_track)!=disk_car->CHS.sector)
   {
-    aff_buffer(BUFFER_ADD,"Warning: Incorrect number of sectors per track %u (FAT) != %u (HD)\n",le16(fat_header->secs_track),disk_car->CHS.sector);
+    screen_buffer_add("Warning: Incorrect number of sectors per track %u (FAT) != %u (HD)\n",le16(fat_header->secs_track),disk_car->CHS.sector);
     log_warning("sect/track %u (FAT) != %u (HD)\n",le16(fat_header->secs_track),disk_car->CHS.sector);
   }
   return 0;
@@ -848,6 +848,7 @@ int recover_FAT(disk_t *disk_car, const struct fat_boot_sector*fat_header, parti
   partition->sborg_offset=0;
   partition->sb_size=512;
   partition->sb_offset=0;
+  set_FAT_info(disk_car,fat_header,partition,verbose);
   switch(partition->upart_type)
   {
     case UP_FAT12:
@@ -890,7 +891,10 @@ int recover_FAT(disk_t *disk_car, const struct fat_boot_sector*fat_header, parti
       else
         partition->part_type_i386=P_32FAT_LBA;
       partition->part_type_mac=PMAC_FAT32;
-      partition->part_type_gpt=GPT_ENT_TYPE_MS_BASIC_DATA;
+      if(memcmp(partition->fsname,"EFI",4)==0)
+	partition->part_type_gpt=GPT_ENT_TYPE_EFI;
+      else
+	partition->part_type_gpt=GPT_ENT_TYPE_MS_BASIC_DATA;
       if(backup)
       {
         partition->sb_offset=6*512;
@@ -901,21 +905,20 @@ int recover_FAT(disk_t *disk_car, const struct fat_boot_sector*fat_header, parti
       log_critical("recover_FAT unknown FAT type\n");
       return 1;
   }
-  set_FAT_info(disk_car,fat_header,partition,verbose);
   return 0;
 }
 
 int fat32_set_part_name(disk_t *disk_car, partition_t *partition, const struct fat_boot_sector*fat_header)
 {
   partition->fsname[0]='\0';
-  if((fat_header->cluster_size>0)&&(fat_header->cluster_size<=128))
+  if((fat_header->sectors_per_cluster>0)&&(fat_header->sectors_per_cluster<=128))
   {
-    unsigned char *buffer=(unsigned char*)MALLOC(fat_header->cluster_size*disk_car->sector_size);
+    unsigned char *buffer=(unsigned char*)MALLOC(fat_header->sectors_per_cluster*disk_car->sector_size);
     if(disk_car->read(disk_car,
-          fat_header->cluster_size*disk_car->sector_size,
+          fat_header->sectors_per_cluster*disk_car->sector_size,
           buffer,
           partition->part_offset +
-          (le16(fat_header->reserved)+fat_header->fats*le32(fat_header->fat32_length)+(uint64_t)(le32(fat_header->root_cluster)-2)*fat_header->cluster_size) * disk_car->sector_size))
+          (le16(fat_header->reserved)+fat_header->fats*le32(fat_header->fat32_length)+(uint64_t)(le32(fat_header->root_cluster)-2)*fat_header->sectors_per_cluster) * disk_car->sector_size))
     {
       log_error("fat32_set_part_name() cannot read FAT32 root cluster.\n");
     }
@@ -923,7 +926,7 @@ int fat32_set_part_name(disk_t *disk_car, partition_t *partition, const struct f
     {
       int i;
       int stop=0;
-      for(i=0;(i<16*fat_header->cluster_size)&&(stop==0);i++)
+      for(i=0;(i<16*fat_header->sectors_per_cluster)&&(stop==0);i++)
       { /* Test attribut volume name and check if the volume name is erased or not */
         if(((buffer[i*0x20+0xB] & ATTR_EXT) !=ATTR_EXT) && ((buffer[i*0x20+0xB] & ATTR_VOLUME) !=0) && (buffer[i*0x20]!=0xE5))
         {
@@ -955,7 +958,7 @@ int check_HPFS(disk_t *disk_car,partition_t *partition,const int verbose)
   unsigned char buffer[512];
   if(disk_car->read(disk_car,disk_car->sector_size, &buffer, partition->part_offset)!=0)
   {
-    aff_buffer(BUFFER_ADD,"check_HPFS: Read error\n");
+    screen_buffer_add("check_HPFS: Read error\n");
     log_error("check_HPFS: Read error\n");
     return 1;
   }
@@ -1016,7 +1019,7 @@ int check_OS2MB(disk_t *disk_car,partition_t *partition,const int verbose)
   unsigned char buffer[0x200];
   if(disk_car->read(disk_car,disk_car->sector_size, &buffer, partition->part_offset)!=0)
   {
-    aff_buffer(BUFFER_ADD,"check_OS2MB: Read error\n");
+    screen_buffer_add("check_OS2MB: Read error\n");
     log_error("check_OS2MB: Read error\n");
     return 1;
   }
