@@ -2,7 +2,7 @@
 
     File: file_txt.c
 
-    Copyright (C) 2005-2007 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 2005-2008 Christophe GRENIER <grenier@cgsecurity.org>
 
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -29,6 +29,9 @@
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
+#ifdef HAVE_TIME_H
+#include <time.h>
+#endif
 #include <ctype.h>      /* tolower */
 #include <stdio.h>
 #include "types.h"
@@ -56,21 +59,23 @@ static void file_check_html(file_recovery_t *file_recovery);
 static void file_check_emlx(file_recovery_t *file_recovery);
 static void file_check_xml(file_recovery_t *file_recovery);
 
-file_hint_t file_hint_fasttxt= {
+const file_hint_t file_hint_fasttxt= {
   .extension="tx?",
-  .description="Text files with header: rtf,xml,xhtml,imm,pm,reg,sh,slk,ram",
+  .description="Text files with header: rtf,xml,xhtml,imm,pm,ram,reg,sh,slk,stp",
   .min_header_distance=0,
   .max_filesize=PHOTOREC_MAX_FILE_SIZE,
   .recover=1,
+  .enable_by_default=1,
   .register_header_check=&register_header_check_fasttxt
 };
 
-file_hint_t file_hint_txt= {
+const file_hint_t file_hint_txt= {
   .extension="txt",
   .description="Other text files: txt,html,asp,bat,C,jsp,perl,php,py... scripts",
   .min_header_distance=0,
   .max_filesize=PHOTOREC_MAX_FILE_SIZE,
   .recover=1,
+  .enable_by_default=1,
   .register_header_check=&register_header_check_txt
 };
 
@@ -83,9 +88,14 @@ static const unsigned char header_rtf[5]	= { '{','\\','r','t','f'};
 static const unsigned char header_reg[8]  	= "REGEDIT4";
 static const unsigned char header_sh[9]  	= "#!/bin/sh";
 static const unsigned char header_slk[10]  	= "ID;PSCALC3";
+static const unsigned char header_stp[13]  	= "ISO-10303-21;";
 static const unsigned char header_ram[7]	= "rtsp://";
 static const unsigned char header_xml[14]	= "<?xml version=";
-static const char sign_html[5]	= "<html";
+static const unsigned char header_dc[6]		= "SC V10";
+static const unsigned char header_ics[15]	= "BEGIN:VCALENDAR";
+static const unsigned char header_msf[19]	= "// <!-- <mdb:mork:z";
+static const unsigned char header_adr[25]	= "Opera Hotlist version 2.0";
+static const unsigned char header_stl[6]	= "solid ";
 
 static void register_header_check_txt(file_stat_t *file_stat)
 {
@@ -103,8 +113,14 @@ static void register_header_check_fasttxt(file_stat_t *file_stat)
   register_header_check(0, header_reg,sizeof(header_reg), &header_check_fasttxt, file_stat);
   register_header_check(0, header_sh,sizeof(header_sh), &header_check_fasttxt, file_stat);
   register_header_check(0, header_slk,sizeof(header_slk), &header_check_fasttxt, file_stat);
+  register_header_check(0, header_stp,sizeof(header_stp), &header_check_fasttxt, file_stat);
+  register_header_check(0, header_stl,sizeof(header_stl), &header_check_fasttxt, file_stat);
   register_header_check(0, header_ram,sizeof(header_ram), &header_check_fasttxt, file_stat);
   register_header_check(0, header_xml,sizeof(header_xml), &header_check_fasttxt, file_stat);
+  register_header_check(4, header_dc, sizeof(header_dc), &header_check_fasttxt, file_stat);
+  register_header_check(0, header_ics, sizeof(header_ics), &header_check_fasttxt, file_stat);
+  register_header_check(0, header_msf, sizeof(header_msf), &header_check_fasttxt, file_stat);
+  register_header_check(0, header_adr, sizeof(header_adr), &header_check_fasttxt, file_stat);
 }
 
 // #define DEBUG_FILETXT
@@ -298,7 +314,11 @@ static int UTF2Lat(unsigned char *buffer_lower, const unsigned char *buffer, con
 static int header_check_fasttxt(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
   const char sign_grisbi[14]		= "Version_grisbi";
-  const char sign_fst[5]                 = "QBFSD";
+  const char sign_fst[5]                = "QBFSD";
+  const char sign_html[5]		= "<html";
+  static const unsigned char spaces[16]={
+    ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ',
+    ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ' };
   if(memcmp(buffer,header_cls,sizeof(header_cls))==0)
   {
     reset_file_recovery(file_recovery_new);
@@ -362,6 +382,30 @@ static int header_check_fasttxt(const unsigned char *buffer, const unsigned int
     file_recovery_new->extension="slk";
     return 1;
   }
+  if(memcmp(buffer, header_stl, sizeof(header_stl))==0 &&
+      memcmp(buffer+0x40, spaces, sizeof(spaces))!=0)
+  {
+    /* StereoLithography - STL Ascii format
+     * http://www.ennex.com/~fabbers/StL.asp	*/
+    reset_file_recovery(file_recovery_new);
+    file_recovery_new->data_check=&data_check_txt;
+    file_recovery_new->file_check=&file_check_size;
+    file_recovery_new->extension="stl";
+    return 1;
+  }
+  if(memcmp(buffer,header_stp,sizeof(header_stp))==0)
+  {
+    /* ISO 10303 is an ISO standard for the computer-interpretable
+     * representation and exchange of industrial product data.
+     * - Industrial automation systems and integration - Product data representation and exchange
+     * - Standard for the Exchange of Product model data.
+     * */
+    reset_file_recovery(file_recovery_new);
+    file_recovery_new->data_check=&data_check_txt;
+    file_recovery_new->file_check=&file_check_size;
+    file_recovery_new->extension="stp";
+    return 1;
+  }
   if(memcmp(buffer,header_ram,sizeof(header_ram))==0)
   {
     reset_file_recovery(file_recovery_new);
@@ -375,7 +419,10 @@ static int header_check_fasttxt(const unsigned char *buffer, const unsigned int
     reset_file_recovery(file_recovery_new);
     file_recovery_new->data_check=&data_check_txt;
     if(td_memmem(buffer, buffer_size, sign_grisbi, sizeof(sign_grisbi))!=NULL)
+    {
+      /* Grisbi - Personal Finance Manager XML data */
       file_recovery_new->extension="gsb";
+    }
     else if(td_memmem(buffer, buffer_size, sign_fst, sizeof(sign_fst))!=NULL)
       file_recovery_new->extension="fst";
     else if(td_memmem(buffer, buffer_size, sign_html, sizeof(sign_html))!=NULL)
@@ -388,6 +435,70 @@ static int header_check_fasttxt(const unsigned char *buffer, const unsigned int
     file_recovery_new->file_check=&file_check_xml;
     return 1;
   }
+  if(buffer[0]=='0' && buffer[1]=='0' && memcmp(&buffer[4],header_dc,sizeof(header_dc))==0)
+  { /*
+       TSCe Survey Controller DC v10.0
+     */
+    reset_file_recovery(file_recovery_new);
+    file_recovery_new->data_check=&data_check_txt;
+    file_recovery_new->file_check=&file_check_size;
+    file_recovery_new->extension="dc";
+    return 1;
+  }
+  if(memcmp(buffer, header_ics, sizeof(header_ics))==0)
+  {
+    const char *date_asc;
+    char *buffer2;
+    reset_file_recovery(file_recovery_new);
+    file_recovery_new->data_check=&data_check_txt;
+    file_recovery_new->file_check=&file_check_size;
+    file_recovery_new->extension="ics";
+    /* DTSTART:19970714T133000            ;Local time
+     * DTSTART:19970714T173000Z           ;UTC time
+     * DTSTART;TZID=US-Eastern:19970714T133000    ;Local time and time
+     */
+    buffer2=(char *)MALLOC(buffer_size+1);
+    buffer2[buffer_size]='\0';
+    memcpy(buffer2, buffer, buffer_size);
+    date_asc=strstr(buffer2, "DTSTART");
+    if(date_asc!=NULL)
+      date_asc=strchr(date_asc, ':');
+    if(date_asc!=NULL && date_asc-buffer2<=buffer_size-14)
+    {
+      struct tm tm_time;
+      memset(&tm_time, 0, sizeof(tm_time));
+      date_asc++;
+      tm_time.tm_sec=(date_asc[13]-'0')*10+(date_asc[14]-'0');      /* seconds 0-59 */
+      tm_time.tm_min=(date_asc[11]-'0')*10+(date_asc[12]-'0');      /* minutes 0-59 */
+      tm_time.tm_hour=(date_asc[9]-'0')*10+(date_asc[10]-'0');      /* hours   0-23*/
+      tm_time.tm_mday=(date_asc[6]-'0')*10+(date_asc[7]-'0');	/* day of the month 1-31 */
+      tm_time.tm_mon=(date_asc[4]-'0')*10+(date_asc[5]-'0')-1;	/* month 0-11 */
+      tm_time.tm_year=(date_asc[0]-'0')*1000+(date_asc[1]-'0')*100+
+	(date_asc[2]-'0')*10+(date_asc[3]-'0')-1900;        	/* year */
+      tm_time.tm_isdst=-1;       /* unknown daylight saving time */
+      file_recovery_new->time=mktime(&tm_time);
+    }
+    free(buffer2);
+    return 1;
+  }
+  /* Mozilla, firefox, thunderbird msf (Mail Summary File) */
+  if(memcmp(buffer, header_msf, sizeof(header_msf))==0)
+  {
+    reset_file_recovery(file_recovery_new);
+    file_recovery_new->data_check=&data_check_txt;
+    file_recovery_new->file_check=&file_check_size;
+    file_recovery_new->extension="msf";
+    return 1;
+  }
+  /* Opera Hotlist bookmark/contact list/notes */
+  if(memcmp(buffer, header_adr, sizeof(header_adr))==0)
+  {
+    reset_file_recovery(file_recovery_new);
+    file_recovery_new->data_check=&data_check_txt;
+    file_recovery_new->file_check=&file_check_size;
+    file_recovery_new->extension="adr";
+    return 1;
+  }
   return 0;
 }
 
@@ -409,6 +520,8 @@ static int header_check_txt(const unsigned char *buffer, const unsigned int buff
   const char sign_jsp2[]		= "<%=";
   const char sign_php[]			= "<?php";
   const char sign_tex[]			= "\\begin{";
+  const char sign_html[]		= "<html";
+  const unsigned int buffer_size_test=(buffer_size < 2048 ? buffer_size : 2048);
   {
     unsigned int tmp=0;
     for(i=0;i<10 && isdigit(buffer[i]);i++)
@@ -426,7 +539,7 @@ static int header_check_txt(const unsigned char *buffer, const unsigned int buff
       return 1;
     }
   }
-  if(buffer_lower_size<buffer_size+16)
+  if(buffer_lower_size<buffer_size_test+16)
   {
     free(buffer_lower);
     buffer_lower=NULL;
@@ -434,10 +547,10 @@ static int header_check_txt(const unsigned char *buffer, const unsigned int buff
   /* Don't malloc/free memory every time, small memory leak */
   if(buffer_lower==NULL)
   {
-    buffer_lower_size=buffer_size+16;
-    buffer_lower=MALLOC(buffer_lower_size);
+    buffer_lower_size=buffer_size_test+16;
+    buffer_lower=(char *)MALLOC(buffer_lower_size);
   }
-  i=UTF2Lat(buffer_lower,buffer,buffer_size);
+  i=UTF2Lat(buffer_lower,buffer,buffer_size_test);
   /* strncasecmp */
   if(memcmp(buffer_lower,header_bat,sizeof(header_bat))==0)
   {
@@ -509,7 +622,36 @@ static int header_check_txt(const unsigned char *buffer, const unsigned int buff
   }
   {
     const char *ext=NULL;
-    if(strstr(buffer_lower, sign_tex)!=NULL)
+    /* ind=~0: random
+     * ind=~1: constant	*/
+    double ind=1;
+    unsigned int nbr=0;
+    /* Detect Fortran */
+    {
+      char *str=buffer_lower;
+      while((str=strstr(str, "\n      "))!=NULL)
+      {
+	nbr++;
+	str++;
+      }
+    }
+    if(i>1)
+    {
+      unsigned int stats[256];
+      unsigned int j;
+      memset(&stats, 0, sizeof(stats));
+      for(j=0;j<i;j++)
+        stats[buffer[j]]++;
+      ind=0;
+      for(j=0;j<256;j++)
+        if(stats[j]>0)
+          ind+=stats[j]*(stats[j]-1);
+      ind=ind/i/(i-1);
+    }
+    if(nbr>10 && i<=0.90)
+      ext="f";
+    /* Detect LaTeX, C, PHP, JSP, ASP, HTML, C header */
+    else if(strstr(buffer_lower, sign_tex)!=NULL)
       ext="tex";
     else if(strstr(buffer_lower, sign_c)!=NULL)
       ext="c";
@@ -525,25 +667,10 @@ static int header_check_txt(const unsigned char *buffer, const unsigned int buff
       ext="html";
     else if(strstr(buffer_lower, sign_h)!=NULL && i>50)
       ext="h";
-    else if(i<100)
+    else if(i<100 || ind<0.03 || ind>0.90)
       ext=NULL;
     else
-    {
-      unsigned int stats[256];
-      unsigned int j;
-      double ind=0;
-      memset(&stats, 0, sizeof(stats));
-      for(j=0;j<i;j++)
-        stats[buffer[j]]++;
-      for(j=0;j<256;j++)
-        if(stats[j]>0)
-          ind+=stats[j]*(stats[j]-1);
-      ind=ind/i/(i-1);
-      if(ind<0.03 || ind>0.90)
-        ext=NULL;
-      else
-        ext=file_hint_txt.extension;
-    }
+      ext=file_hint_txt.extension;
     if(ext!=NULL && strcmp(ext,"txt")==0 &&
         (strstr(buffer_lower,"<br>")!=NULL || strstr(buffer_lower,"<p>")!=NULL))
     {
@@ -594,10 +721,11 @@ Doc: \r (0xD)
             strstr(file_recovery->filename,".html")!=NULL) ||
           /* Text should not be found in JPEG */
           (file_recovery->file_stat->file_hint==&file_hint_jpg &&
-           td_memmem(buffer, buffer_size, "8BIM", 4)==NULL &&
-           td_memmem(buffer, buffer_size, "adobe", 5)==NULL) ||
+           td_memmem(buffer, buffer_size_test, "8BIM", 4)==NULL &&
+           td_memmem(buffer, buffer_size_test, "adobe", 5)==NULL) ||
           /* Text should not be found in zip because of compression */
-          (file_recovery->file_stat->file_hint==&file_hint_zip && td_memmem(buffer, buffer_size, zip_header, 4)==NULL))
+          (file_recovery->file_stat->file_hint==&file_hint_zip &&
+	  td_memmem(buffer, buffer_size_test, zip_header, 4)==NULL))
       {
         reset_file_recovery(file_recovery_new);
         file_recovery_new->data_check=&data_check_txt;
@@ -623,7 +751,7 @@ Doc: \r (0xD)
 static int data_check_txt(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery)
 {
   unsigned int i;
-  char *buffer_lower=MALLOC(buffer_size+16);
+  char *buffer_lower=(char *)MALLOC(buffer_size+16);
   i=UTF2Lat(buffer_lower,&buffer[buffer_size/2],buffer_size/2);
   if(i<buffer_size/2)
   {
@@ -658,7 +786,7 @@ static void file_check_html(file_recovery_t *file_recovery)
     int i;
     if(fseek(file_recovery->handle,0,SEEK_SET)<0)
       return;
-    buffer_lower=MALLOC(read_size);
+    buffer_lower=(char *)MALLOC(read_size);
     taille=fread(buffer_lower,1,read_size,file_recovery->handle);
     if(taille<0)
     {
@@ -669,7 +797,7 @@ static void file_check_html(file_recovery_t *file_recovery)
     /* TODO: use strcasestr if available */
     for(i=0;i<taille;i++)
       buffer_lower[i]=tolower(buffer_lower[i]);
-    if(strstr(buffer_lower, sign_html)==NULL)
+    if(strstr(buffer_lower, "<html")==NULL)
     {
       log_warning("%s: no header\n",file_recovery->filename);
       file_recovery->file_size=0;
