@@ -26,6 +26,9 @@
 #else
 #define PHOTOREC_MAX_FILE_SIZE (((uint64_t)1<<41)-1)
 #endif
+#define PHOTOREC_MAX_SIZE_16 (((uint64_t)1<<15)-1)
+#define PHOTOREC_MAX_SIZE_32 (((uint64_t)1<<31)-1)
+
 typedef struct file_hint_struct file_hint_t;
 typedef struct file_recovery_struct file_recovery_t;
 typedef struct file_enable_struct file_enable_t;
@@ -60,6 +63,7 @@ struct file_recovery_struct
   file_stat_t *file_stat;
   FILE *handle;
   char filename[2048];
+  time_t time;
   uint64_t file_size;
   uint64_t file_size_on_disk;
   alloc_list_t location;
@@ -83,6 +87,7 @@ struct file_hint_struct
   /* needed by tar header */
   const uint64_t max_filesize;
   const int recover;
+  const int enable_by_default;
   void (*register_header_check)(file_stat_t *file_stat);
 };
 
@@ -96,7 +101,7 @@ struct alloc_data_struct
 
 void file_search_footer(file_recovery_t *file_recovery, const unsigned char*footer, const unsigned int footer_length);
 void file_search_lc_footer(file_recovery_t *file_recovery, const unsigned char*footer, const unsigned int footer_length);
-alloc_data_t *del_search_space(alloc_data_t *list_search_space, uint64_t start, uint64_t end);
+alloc_data_t *del_search_space(alloc_data_t *list_search_space, const uint64_t start, const uint64_t end);
 int data_check_size(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery);
 void file_check_size(file_recovery_t *file_recovery);
 void reset_file_recovery(file_recovery_t *file_recovery);
