@@ -57,7 +57,7 @@ extern const arch_fnct_t arch_mac;
 extern const arch_fnct_t arch_none;
 extern const arch_fnct_t arch_sun;
 extern const arch_fnct_t arch_xbox;
-static void align_structure(const disk_t *disk_car, list_part_t *list_part,const unsigned int location_boundary);
+static void align_structure(list_part_t *list_part,const unsigned int location_boundary);
 static list_part_t *reduce_structure(list_part_t *list_part);
 static int use_backup(disk_t *disk_car, const list_part_t *list_part, const int verbose,const int dump_ind, const unsigned int expert, char**current_cmd);
 static int interface_part_bad_log(disk_t *disk_car,list_part_t *list_part_bad);
@@ -67,7 +67,7 @@ static void warning_geometry_ncurses(disk_t *disk_car, const unsigned int recomm
 static void ask_mbr_order_i386(disk_t *disk_car,list_part_t *list_part);
 #endif
 static list_part_t *add_ext_part_i386(disk_t *disk_car, list_part_t *list_part, const int max_ext, const int align,const int verbose);
-static unsigned int tab_insert(uint64_t *tab, uint64_t offset, unsigned int tab_nbr);
+static unsigned int tab_insert(uint64_t *tab, const uint64_t offset, unsigned int tab_nbr);
 /* Optimization */
 static inline uint64_t CHS2offset_inline(const disk_t *disk_car,const CHS_t*CHS);
 static list_part_t *search_part(disk_t *disk_car, const list_part_t *list_part_org, const int verbose, const int dump_ind, const int fast_mode, const int interface, const int search_vista_part, char **current_cmd);
@@ -87,7 +87,7 @@ static inline uint64_t CHS2offset_inline(const disk_t *disk_car,const CHS_t*CHS)
 }
 /* Optimization end */
 
-static void align_structure(const disk_t *disk_car, list_part_t *list_part, const unsigned int location_boundary)
+static void align_structure(list_part_t *list_part, const unsigned int location_boundary)
 {
   list_part_t *element;
   for(element=list_part;element!=NULL;element=element->next)
@@ -324,12 +324,14 @@ static void warning_geometry_ncurses(disk_t *disk_car, const unsigned int recomm
 }
 #endif
 
-static unsigned int tab_insert(uint64_t *tab, uint64_t offset, unsigned int tab_nbr)
+static unsigned int tab_insert(uint64_t *tab, const uint64_t offset, unsigned int tab_nbr)
 {
   if(tab_nbr<MAX_SEARCH_LOCATION-1)
   {
     unsigned int i,j;
-    for(i=0;i<tab_nbr && tab[i]<=offset;i++);
+    for(i=0;i<tab_nbr && tab[i]<offset;i++);
+    if(i<tab_nbr && tab[i]==offset)
+      return tab_nbr;
     tab_nbr++;
     for(j=tab_nbr;j>i;j--)
       tab[j]=tab[j-1];
@@ -382,7 +384,14 @@ static list_part_t *search_part(disk_t *disk_car, const list_part_t *list_part_o
   int ind_stop=0;
   list_part_t *list_part=NULL;
   list_part_t *list_part_bad=NULL;
-  partition_t *partition=partition_new(disk_car->arch);
+  partition_t *partition;
+  /* It's not a problem to read a little bit more than necessary */
+  const uint64_t search_location_max=td_max(disk_car->disk_size +
+      (disk_car->CHS.head+1)*disk_car->CHS.sector*disk_car->sector_size,
+      disk_car->disk_real_size);
+  const uint64_t max_disk_size_for_partition=td_max(disk_car->disk_size,
+      (uint64_t)(disk_car->CHS.cylinder+1)*(disk_car->CHS.head+1)*disk_car->CHS.sector*disk_car->sector_size);
+  partition=partition_new(disk_car->arch);
   buffer_disk=(unsigned char*)MALLOC(16*DEFAULT_SECTOR_SIZE);
   {
     /* Will search for partition at current known partition location */
@@ -402,7 +411,7 @@ static list_part_t *search_part(disk_t *disk_car, const list_part_t *list_part_o
     wattroff(stdscr, A_REVERSE);
   }
 #endif
-  aff_buffer(BUFFER_RESET,"Q");
+  screen_buffer_reset();
   log_info("\nsearch_part()\n");
   log_info("%s\n",disk_car->description(disk_car));
   if(disk_car->arch==&arch_gpt)
@@ -414,8 +423,26 @@ static list_part_t *search_part(disk_t *disk_car, const list_part_t *list_part_o
   {
     min_location=disk_car->sector_size;
     location_boundary=disk_car->sector_size;
-    /* sometime users choose Intel instead of GPT */
-    try_offset_nbr=tab_insert(try_offset,2*disk_car->sector_size+16384,try_offset_nbr);
+    /* sometimes users choose Intel instead of GPT */
+    try_offset_nbr=tab_insert(try_offset, 2*disk_car->sector_size+16384, try_offset_nbr);
+    /* sometimes users don't choose Vista by mistake */
+    try_offset_nbr=tab_insert(try_offset, 2048*512, try_offset_nbr);
+    /* try to deal with incorrect geometry */
+    /* 1/[01]/1 CHS x  16 63 */
+    try_offset_nbr=tab_insert(try_offset, 16 * 63 * disk_car->sector_size, try_offset_nbr);
+    try_offset_nbr=tab_insert(try_offset, 17 * 63 * disk_car->sector_size, try_offset_nbr);
+    try_offset_nbr=tab_insert(try_offset, 16 * disk_car->CHS.sector * disk_car->sector_size, try_offset_nbr);
+    try_offset_nbr=tab_insert(try_offset, 17 * disk_car->CHS.sector * disk_car->sector_size, try_offset_nbr);
+    /* 1/[01]/1 CHS x 240 63 */
+    try_offset_nbr=tab_insert(try_offset, 240 * 63 * disk_car->sector_size, try_offset_nbr);
+    try_offset_nbr=tab_insert(try_offset, 241 * 63 * disk_car->sector_size, try_offset_nbr);
+    try_offset_nbr=tab_insert(try_offset, 240 * disk_car->CHS.sector * disk_car->sector_size, try_offset_nbr);
+    try_offset_nbr=tab_insert(try_offset, 241 * disk_car->CHS.sector * disk_car->sector_size, try_offset_nbr);
+    /* 1/[01]/1 CHS x 255 63 */
+    try_offset_nbr=tab_insert(try_offset, 255 * 63 * disk_car->sector_size, try_offset_nbr);
+    try_offset_nbr=tab_insert(try_offset, 256 * 63 * disk_car->sector_size, try_offset_nbr);
+    try_offset_nbr=tab_insert(try_offset, 255 * disk_car->CHS.sector * disk_car->sector_size, try_offset_nbr);
+    try_offset_nbr=tab_insert(try_offset, 256 * disk_car->CHS.sector * disk_car->sector_size, try_offset_nbr);
   }
   else if(disk_car->arch==&arch_mac)
   {
@@ -443,7 +470,7 @@ static list_part_t *search_part(disk_t *disk_car, const list_part_t *list_part_o
   /* Not every sector will be examined */
   search_location_init(disk_car, location_boundary, fast_mode, search_vista_part);
   /* Scan the disk */
-  while(ind_stop==0 && search_location<disk_car->disk_real_size)
+  while(ind_stop==0 && search_location < search_location_max)
   {
     unsigned int sector_inc=0;
     static CHS_t start;
@@ -659,7 +686,7 @@ static list_part_t *search_part(disk_t *disk_car, const list_part_t *list_part_o
           aff_part_buffer(AFF_PART_BASE, disk_car,partition);
           if(interface)
           {
-            aff_buffer(BUFFER_SHOW,"Q");
+	    screen_buffer_to_interface();
           }
           if(disk_car->arch->is_part_known(partition)!=0 &&
               partition->part_size>1 &&
@@ -697,7 +724,7 @@ static list_part_t *search_part(disk_t *disk_car, const list_part_t *list_part_o
               /* TODO: Detect Linux md 1.0 software raid */
             }
             /* */
-            if(pos_fin<=disk_car->disk_size)
+            if(pos_fin <= max_disk_size_for_partition)
             {
               {
                 int insert_error=0;
@@ -790,7 +817,7 @@ static list_part_t *search_part(disk_t *disk_car, const list_part_t *list_part_o
               partition->status=STATUS_DELETED;
               if(disk_car->arch->is_part_known(partition)!=0 && partition->part_size>1 &&
                   partition->part_offset >= min_location &&
-                  partition->part_offset+partition->part_size-1 <= disk_car->disk_size)
+                  partition->part_offset+partition->part_size-1 <= max_disk_size_for_partition)
               {
                 int insert_error=0;
                 partition_t *new_partition=partition_new(NULL);
@@ -1179,7 +1206,7 @@ int interface_recovery(disk_t *disk_car, const list_part_t * list_part_org, cons
 	log_warning("Warning: the current number of heads per cylinder is %u but the correct value may be %u.\n",
 	    disk_car->CHS.head+1, head_max+1);
 #ifdef HAVE_NCURSES
-	if(*current_cmd!=NULL)
+	if(*current_cmd==NULL)
 	{
 	  warning_geometry_ncurses(disk_car, head_max+1);
 	}
@@ -1225,7 +1252,7 @@ int interface_recovery(disk_t *disk_car, const list_part_t * list_part_org, cons
       {	/* arch_none, arch_xbox, arch_gpt */
 	location_boundary=disk_car->sector_size;
       }
-      align_structure(disk_car,list_part,location_boundary);
+      align_structure(list_part, location_boundary);
     }
 
     disk_car->arch->init_structure(disk_car,list_part,verbose);
@@ -1298,7 +1325,8 @@ int interface_recovery(disk_t *disk_car, const list_part_t * list_part_org, cons
 		else
 		{
 		  use_backup(disk_car,list_part,verbose,dump_ind,expert,current_cmd);
-		  display_message("You will have to reboot for the change to take effect.\n");
+		  if(no_confirm==0)
+		    display_message("You will have to reboot for the change to take effect.\n");
 		}
 	      }
 	      else
