@@ -2,7 +2,7 @@
 
     File: intrf.c
 
-    Copyright (C) 1998-2007 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2008 Christophe GRENIER <grenier@cgsecurity.org>
 
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -86,94 +86,83 @@ extern const char *monstr[];
 static char intr_buffer_screen[MAX_LINES][LINE_LENGTH+1];
 static int intr_nbr_line=0;
 static void set_parent_directory(char *dst_directory);
-struct file_info {
-  struct td_list_head list;
-  char name[4096];
-  struct stat stat;
-};
 
-
-int aff_buffer(const buffer_cmd_t cmd, const char *_format, ...)
+int screen_buffer_add(const char *_format, ...)
 {
-  switch(cmd)
+  char tmp_line[BUFFER_LINE_LENGTH+1];
+  char *pos_in_tmp_line=tmp_line;
+  va_list ap;
+  va_start(ap,_format);
+  memset(tmp_line,'\0',sizeof(tmp_line));
+  vsnprintf(tmp_line,BUFFER_LINE_LENGTH,_format,ap);
+  va_end(ap);
+  while(pos_in_tmp_line!=NULL && (intr_nbr_line<MAX_LINES))
   {
-    case BUFFER_RESET:
-      {
-        int i;
-        intr_nbr_line=0;
-        for(i=0;i<MAX_LINES;i++)
-          memset(intr_buffer_screen[i],0,LINE_LENGTH+1);
-      }
-      break;
-    case BUFFER_ADD:
-      {
-        char tmp_line[BUFFER_LINE_LENGTH+1];
-        char *pos_in_tmp_line=tmp_line;
-        va_list ap;
-        va_start(ap,_format);
-        memset(tmp_line,'\0',sizeof(tmp_line));
-        vsnprintf(tmp_line,BUFFER_LINE_LENGTH,_format,ap);
-        va_end(ap);
-        while(pos_in_tmp_line!=NULL && (intr_nbr_line<MAX_LINES))
-        {
-          unsigned int len=strlen(intr_buffer_screen[intr_nbr_line]);
-          unsigned int nbr=LINE_LENGTH-len;
-          char *ret_ligne= strchr(pos_in_tmp_line,'\n');
-          if(ret_ligne!=NULL && ret_ligne-pos_in_tmp_line < nbr)
-              nbr=ret_ligne-pos_in_tmp_line;
-          memcpy(&intr_buffer_screen[intr_nbr_line][len], pos_in_tmp_line, nbr);
-          intr_buffer_screen[intr_nbr_line][len+nbr]='\0';
-          if(ret_ligne!=NULL)
-          {
-            if(++intr_nbr_line<MAX_LINES)
-              intr_buffer_screen[intr_nbr_line][0]='\0';
-            ret_ligne++;
-          }
-          pos_in_tmp_line=ret_ligne;
-        }
-        /*	log_trace("aff_intr_buffer_screen %d =>%s<=\n",intr_nbr_line,tmp_line); */
-        if(intr_nbr_line==MAX_LINES)
-        {
-          log_warning("Buffer can't store more than %u lines.\n", MAX_LINES);
-          intr_nbr_line++;
-        }
-      }
-      break;
-    case BUFFER_SHOW:
-#ifdef HAVE_NCURSES
-      {
-        int i;
-        int pos=intr_nbr_line-DUMP_MAX_LINES<0?0:intr_nbr_line-DUMP_MAX_LINES;
-        if(intr_nbr_line<MAX_LINES && intr_buffer_screen[intr_nbr_line][0]!='\0')
-          intr_nbr_line++;
-        /* curses interface */
-        for (i=pos; i<intr_nbr_line && i<MAX_LINES && (i-pos)<DUMP_MAX_LINES; i++)
-        {
-          wmove(stdscr,DUMP_Y+1+i-pos,DUMP_X);
-          wclrtoeol(stdscr);
-          wprintw(stdscr,"%s",intr_buffer_screen[i]);
-        }
-        wrefresh(stdscr);
-      }
-#endif
-      break;
-    case BUFFER_WRITE:
-      {
-        int i;
-        if(intr_nbr_line<MAX_LINES && intr_buffer_screen[intr_nbr_line][0]!='\0')
-          intr_nbr_line++;
-        /* to log file and stdout */
-        for(i=0;i<intr_nbr_line && i<MAX_LINES;i++)
-        {
-          printf("%s\n",intr_buffer_screen[i]);
-          log_info("%s\n",intr_buffer_screen[i]);
-        }
-      }
-      break;
+    unsigned int len=strlen(intr_buffer_screen[intr_nbr_line]);
+    unsigned int nbr=LINE_LENGTH-len;
+    char *ret_ligne= strchr(pos_in_tmp_line,'\n');
+    if(ret_ligne!=NULL && ret_ligne-pos_in_tmp_line < nbr)
+      nbr=ret_ligne-pos_in_tmp_line;
+    memcpy(&intr_buffer_screen[intr_nbr_line][len], pos_in_tmp_line, nbr);
+    intr_buffer_screen[intr_nbr_line][len+nbr]='\0';
+    if(ret_ligne!=NULL)
+    {
+      if(++intr_nbr_line<MAX_LINES)
+	intr_buffer_screen[intr_nbr_line][0]='\0';
+      ret_ligne++;
+    }
+    pos_in_tmp_line=ret_ligne;
+  }
+  /*	log_trace("aff_intr_buffer_screen %d =>%s<=\n",intr_nbr_line,tmp_line); */
+  if(intr_nbr_line==MAX_LINES)
+  {
+    log_warning("Buffer can't store more than %u lines.\n", MAX_LINES);
+    intr_nbr_line++;
   }
   return 0;
 }
 
+void screen_buffer_to_interface()
+{
+#ifdef HAVE_NCURSES
+  {
+    int i;
+    int pos=intr_nbr_line-DUMP_MAX_LINES<0?0:intr_nbr_line-DUMP_MAX_LINES;
+    if(intr_nbr_line<MAX_LINES && intr_buffer_screen[intr_nbr_line][0]!='\0')
+      intr_nbr_line++;
+    /* curses interface */
+    for (i=pos; i<intr_nbr_line && i<MAX_LINES && (i-pos)<DUMP_MAX_LINES; i++)
+    {
+      wmove(stdscr,DUMP_Y+1+i-pos,DUMP_X);
+      wclrtoeol(stdscr);
+      wprintw(stdscr,"%s",intr_buffer_screen[i]);
+    }
+    wrefresh(stdscr);
+  }
+#endif
+}
+
+void screen_buffer_to_stdout()
+{
+  int i;
+  if(intr_nbr_line<MAX_LINES && intr_buffer_screen[intr_nbr_line][0]!='\0')
+    intr_nbr_line++;
+  /* to log file and stdout */
+  for(i=0;i<intr_nbr_line && i<MAX_LINES;i++)
+  {
+    printf("%s\n",intr_buffer_screen[i]);
+    log_info("%s\n",intr_buffer_screen[i]);
+  }
+}
+
+void screen_buffer_reset()
+{
+  int i;
+  intr_nbr_line=0;
+  for(i=0;i<MAX_LINES;i++)
+    memset(intr_buffer_screen[i],0,LINE_LENGTH+1);
+}
+
 void screen_buffer_to_log()
 {
   int i;
@@ -251,9 +240,8 @@ const char *aff_part_aux(const unsigned int newline, const disk_t *disk_car, con
 #define PATH_SEP '/'
 #define SPATH_SEP "/"
 #if defined(__CYGWIN__)
+/* /cygdrive/c/ => */
 #define PATH_DRIVE_LENGTH 9
-#elif defined(DJGPP) || defined(__OS2__)
-#define PATH_DRIVE_LENGTH 3
 #endif
 
 static void set_parent_directory(char *dst_directory)
@@ -270,10 +258,12 @@ static void set_parent_directory(char *dst_directory)
   else
     dst_directory[PATH_DRIVE_LENGTH]='\0';
 #elif defined(DJGPP) || defined(__OS2__)
-  if(last_sep>PATH_DRIVE_LENGTH)
-    dst_directory[last_sep]='\0';
+  if(last_sep > 2 )
+    dst_directory[last_sep]='\0';	/* subdirectory */
+  else if(last_sep == 2 && dst_directory[3]!='\0')
+    dst_directory[3]='\0';	/* root directory */
   else
-    dst_directory[0]='\0';
+    dst_directory[0]='\0';	/* drive list */
 #else
   if(last_sep>1)
     dst_directory[last_sep]='\0';
@@ -282,8 +272,24 @@ static void set_parent_directory(char *dst_directory)
 #endif
 }
 
+static inline char *td_getcwd(char *buf, unsigned long size)
+{
+  /* buf must non-NULL*/
+#ifdef HAVE_GETCWD
+  if(getcwd(buf, size)!=NULL)
+    return buf;
+#endif
+  buf[0]='.';
+  buf[1]='\0';
+  return buf;
+}
+
 #ifdef HAVE_NCURSES
+#define INTER_DIR (LINES-25+16)
+static int vaff_txt(int line, WINDOW *window, const char *_format, va_list ap) __attribute__((format(printf, 3, 0)));
+static int wmenuUpdate(WINDOW *window, const int yinfo, int y, int x, const struct MenuItem *menuItems, const unsigned int itemLength, const char *available, const int menuType, unsigned int current);
 static void dir_aff_entry(WINDOW *window, struct file_info *dir_info);
+static int wgetch_nodelay(WINDOW *window);
 
 int get_string(char *str, int len, char *def)
 {
@@ -351,7 +357,7 @@ int get_string(char *str, int len, char *def)
       return i;
 }
 
-int wgetch_nodelay(WINDOW *window)
+static int wgetch_nodelay(WINDOW *window)
 {
   int res;
   nodelay(window,TRUE);
@@ -365,7 +371,7 @@ int wgetch_nodelay(WINDOW *window)
  * Should not be called directly. Call function menuSelect instead.
  */
 
-int wmenuUpdate(WINDOW *window, int y, int x, const struct MenuItem *menuItems, const unsigned int itemLength, const char *available, const int menuType, unsigned int current)
+static int wmenuUpdate(WINDOW *window, const int yinfo, int y, int x, const struct MenuItem *menuItems, const unsigned int itemLength, const char *available, const int menuType, unsigned int current)
 {
   unsigned int i, lmargin = x, ymargin = y;
   unsigned int lenNameMax=0;
@@ -445,7 +451,7 @@ int wmenuUpdate(WINDOW *window, int y, int x, const struct MenuItem *menuItems,
     if( menuType & MENU_VERT )
     {
       y += 1;
-      if( y >= WARNING_START )
+      if( y >= yinfo - 1)
       {
         y = ymargin;
         x += (lenName < itemLength?itemLength:lenName) + MENU_SPACING;
@@ -467,7 +473,7 @@ int wmenuUpdate(WINDOW *window, int y, int x, const struct MenuItem *menuItems,
   if(!(menuType & MENU_VERT_WARN))
   {
     const char *mcd = menuItems[current].desc;
-    mvwaddstr(window, WARNING_START + 1, (COLUMNS - strlen( mcd )) / 2, mcd );
+    mvwaddstr(window, yinfo, (COLUMNS - strlen( mcd )) / 2, mcd );
   }
   return y;
 }
@@ -475,13 +481,13 @@ int wmenuUpdate(WINDOW *window, int y, int x, const struct MenuItem *menuItems,
 /* This function takes a list of menu items, lets the user choose one *
  * and returns the value keyboard shortcut of the selected menu item  */
 
-int wmenuSelect(WINDOW *window, int y, int x, const struct MenuItem *menuItems, const unsigned int itemLength, const char *available, int menuType, unsigned int menuDefault)
+int wmenuSelect(WINDOW *window, int yinfo, int y, int x, const struct MenuItem *menuItems, const unsigned int itemLength, const char *available, int menuType, unsigned int menuDefault)
 {
   unsigned int current=menuDefault;
-  return wmenuSelect_ext(window, y, x, menuItems, itemLength, available, menuType, &current,NULL);
+  return wmenuSelect_ext(window, yinfo, y, x, menuItems, itemLength, available, menuType, &current, NULL);
 }
 
-int wmenuSelect_ext(WINDOW *window, int y, int x, const struct MenuItem *menuItems, const unsigned int itemLength, const char *available, int menuType, unsigned int *current, int *real_key)
+int wmenuSelect_ext(WINDOW *window, const int yinfo, int y, int x, const struct MenuItem *menuItems, const unsigned int itemLength, const char *available, int menuType, unsigned int *current, int *real_key)
 {
   int i, ylast = y, key = 0;
   /*
@@ -505,7 +511,7 @@ int wmenuSelect_ext(WINDOW *window, int y, int x, const struct MenuItem *menuIte
   while( key==0 )
   {
     /* Display the menu */
-    ylast = wmenuUpdate( window, y, x, menuItems, itemLength, available,
+    ylast = wmenuUpdate( window, yinfo, y, x, menuItems, itemLength, available,
         menuType, *current );
     wrefresh(window);
     /* Don't put wgetch after the following wclrtoeol */
@@ -519,7 +525,7 @@ int wmenuSelect_ext(WINDOW *window, int y, int x, const struct MenuItem *menuIte
       wmove(window, i, x );
       wclrtoeol(window);
     }
-    wmove(window, WARNING_START + 1, 0 );
+    wmove(window, yinfo, 0 );
     wclrtoeol(window);
     /* Cursor keys */
     switch(key)
@@ -604,7 +610,7 @@ int wmenuSelect_ext(WINDOW *window, int y, int x, const struct MenuItem *menuIte
     wmove(window, i, x );
     wclrtoeol(window);
   }
-  wmove(window, WARNING_START + 1, 0 );
+  wmove(window, yinfo, 0 );
   wclrtoeol(window);
   return key;
 }
@@ -624,7 +630,7 @@ int wmenuSimple(WINDOW *window,const struct MenuItem *menuItems, unsigned int me
       available[i] = menuItems[i].key;
     }
     available[i] = 0;
-    return wmenuSelect(window,18, 0, menuItems, itemLength, available, MENU_HORIZ | MENU_BUTTON, menuDefault);
+    return wmenuSelect(window, 24, 18, 0, menuItems, itemLength, available, MENU_HORIZ | MENU_BUTTON, menuDefault);
 }
 
 /* End of command menu support code */
@@ -732,7 +738,7 @@ void dump(WINDOW *window, const void *nom_dump,unsigned int lng)
           wprintw(window," ");
       }
     }
-    switch (wmenuSelect(window,INTER_DUMP_Y, INTER_DUMP_X, menuDump, 8, options, MENU_HORIZ | MENU_BUTTON | MENU_ACCEPT_OTHERS, menu))
+    switch (wmenuSelect(window, 24, INTER_DUMP_Y, INTER_DUMP_X, menuDump, 8, options, MENU_HORIZ | MENU_BUTTON | MENU_ACCEPT_OTHERS, menu))
     {
       case 'p':
       case 'P':
@@ -885,7 +891,7 @@ void dump2(WINDOW *window, const void *dump_1, const void *dump_2, const unsigne
           wprintw(window," ");
       }
     }
-    switch (wmenuSelect(window,INTER_DUMP_Y, INTER_DUMP_X, menuDump, 8, options, MENU_HORIZ | MENU_BUTTON | MENU_ACCEPT_OTHERS, menu))
+    switch (wmenuSelect(window, 24, INTER_DUMP_Y, INTER_DUMP_X, menuDump, 8, options, MENU_HORIZ | MENU_BUTTON | MENU_ACCEPT_OTHERS, menu))
     {
       case 'p':
       case 'P':
@@ -984,7 +990,7 @@ int screen_buffer_display_ext(WINDOW *window, const char *options_org, const str
     wclrtoeol(window);
     if(i<intr_nbr_line)
       wprintw(window, "Next");
-    key=wmenuSelect_ext(window,INTER_ANALYSE_MENU_Y, INTER_ANALYSE_MENU_X, (menuItems!=NULL?menuItems:menuDefault),
+    key=wmenuSelect_ext(window, 24, INTER_ANALYSE_MENU_Y, INTER_ANALYSE_MENU_X, (menuItems!=NULL?menuItems:menuDefault),
         itemLength, options, MENU_HORIZ | MENU_BUTTON | MENU_ACCEPT_OTHERS, menu,NULL);
     switch (key)
     {
@@ -1041,7 +1047,7 @@ void aff_CHS(const CHS_t * CHS)
 
 void aff_CHS_buffer(const CHS_t * CHS)
 {
-  aff_buffer(BUFFER_ADD,"%5u %3u %2u ", CHS->cylinder, CHS->head, CHS->sector);
+  screen_buffer_add("%5u %3u %2u ", CHS->cylinder, CHS->head, CHS->sector);
 }
 
 void aff_part(WINDOW *window,const unsigned int newline,const disk_t *disk_car,const partition_t *partition)
@@ -1140,6 +1146,43 @@ void not_implemented(const char *msg)
 static SCREEN *screenp=NULL;
 #endif
 
+static char *filename_to_directory(const char *filename)
+{
+  char buf[2048];
+  char *res;
+#ifdef HAVE_READLINK
+  int len;
+  len=readlink(filename,buf,sizeof(buf)-1);
+  if(len>=0)
+    buf[len]='\0';
+  else
+  {
+    strncpy(buf,filename,sizeof(buf)-1);
+    buf[sizeof(buf)-1]='\0';
+  }
+#else
+  strncpy(buf,filename,sizeof(buf)-1);
+  buf[sizeof(buf)-1]='\0';
+#endif
+  res=dirname(buf);
+#ifdef HAVE_GETCWD
+  if(res!=NULL && strcmp(res,".")==0 && getcwd(buf, sizeof(buf)-1)!=NULL)
+  {
+    buf[sizeof(buf)-1]='\0';
+    res=buf;
+  }
+#endif
+#ifdef __CYGWIN__
+  {
+    char beautifull_dst_directory[2048];
+    cygwin_conv_to_win32_path(res, beautifull_dst_directory);
+    return strdup(beautifull_dst_directory);
+  }
+#else
+  return strdup(res);
+#endif
+}
+
 int start_ncurses(const char *prog_name, const char *real_prog_name)
 {
 #if defined(DJGPP) || defined(__MINGW32__)
@@ -1154,6 +1197,7 @@ int start_ncurses(const char *prog_name, const char *real_prog_name)
 #else
   {
     int term_overwrite;
+    char *terminfo=filename_to_directory(real_prog_name);
     for(term_overwrite=0;screenp==NULL && term_overwrite<=1;term_overwrite++)
     {
 #ifdef HAVE_SETENV
@@ -1171,34 +1215,11 @@ int start_ncurses(const char *prog_name, const char *real_prog_name)
 #endif
       screenp=newterm(NULL,stdout,stdin);
 #ifdef HAVE_SETENV
-#ifdef HAVE_DIRNAME
-      if(screenp==NULL)
+      if(screenp==NULL && terminfo!=NULL)
       {
-        char buf[2048];
-#ifdef HAVE_READLINK
-        {
-          int len;
-          len=readlink(real_prog_name,buf,sizeof(buf)-1);
-          if(len>=0)
-          {
-            buf[len]='\0';
-          }
-          else
-          {
-            strncpy(buf,real_prog_name,sizeof(buf)-1);
-            buf[sizeof(buf)-1]='\0';
-          }
-        }
-#else	// HAVE_READLINK
-        {
-          strncpy(buf,real_prog_name,sizeof(buf)-1);
-          buf[sizeof(buf)-1]='\0';
-        }
-#endif	// HAVE_READLINK
-        setenv("TERMINFO",dirname(buf),1);
+        setenv("TERMINFO", terminfo, 1);
         screenp=newterm(NULL,stdout,stdin);
       }
-#endif	// HAVE_DIRNAME
       if(screenp==NULL)
       {
         setenv("TERMINFO",".",1);
@@ -1212,15 +1233,17 @@ int start_ncurses(const char *prog_name, const char *real_prog_name)
     {
       log_critical("Terminfo file is missing.\n");
 #if defined(__CYGWIN__)
-      printf("Sub-directory 'c' and the terminfo file 'c\\cygwin' is missing.\n");
+      printf("The terminfo file '%s\\c\\cygwin' is missing.\n", terminfo);
 #else
       printf("Terminfo file is missing.\n");
 #endif
       printf("Extract all files and subdirectories before running the program.\n");
       printf("Press Enter key to quit.\n");
       getchar();
+      free(terminfo);
       return 1;
     }
+    free(terminfo);
   }
 #endif
   noecho();
@@ -1239,34 +1262,31 @@ int start_ncurses(const char *prog_name, const char *real_prog_name)
   curs_set(0);
   {
     int quit=0;
-    while(LINES<25 && quit==0)
+    while(LINES>=8 && LINES<25 && quit==0)
     {
-      if(LINES>=8)
+      aff_copy(stdscr);
+      wmove(stdscr,4,0);
+      wprintw(stdscr,"%s need 25 lines to work.", prog_name);
+      wmove(stdscr,5,0);
+      wprintw(stdscr,"Please enlarge the terminal.");
+      wmove(stdscr,LINES-2,0);
+      wattrset(stdscr, A_REVERSE);
+      wprintw(stdscr,"[ Quit ]");
+      wattroff(stdscr, A_REVERSE);
+      wrefresh(stdscr);
+      switch(wgetch(stdscr))
       {
-        aff_copy(stdscr);
-        wmove(stdscr,4,0);
-        wprintw(stdscr,"%s need 25 lines to work.", prog_name);
-        wmove(stdscr,5,0);
-        wprintw(stdscr,"Please enlarge the terminal.");
-        wmove(stdscr,LINES-2,0);
-        wattrset(stdscr, A_REVERSE);
-        wprintw(stdscr,"[ Quit ]");
-        wattroff(stdscr, A_REVERSE);
-        wrefresh(stdscr);
-        switch(wgetch(stdscr))
-        {
-          case KEY_ENTER:
+	case 'q':
+	case 'Q':
+	case KEY_ENTER:
 #ifdef PADENTER
-          case PADENTER:
+	case PADENTER:
 #endif
-          case '\n':
-          case '\r':
-            quit=1;
-            break;
-        }
+	case '\n':
+	case '\r':
+	  quit=1;
+	  break;
       }
-      else
-        quit=1;
     }
   }
   if(LINES<25)
@@ -1285,9 +1305,12 @@ int end_ncurses()
   wrefresh(stdscr);
   nl();
   endwin();
+#if defined(DJGPP) || defined(__MINGW32__)
+#else
 #ifdef HAVE_DELSCREEN
   if(screenp!=NULL)
     delscreen(screenp);
+#endif
 #endif
   return 0;
 }
@@ -1333,21 +1356,23 @@ static int intrf_no_disk_ncurses(const char *prog_name)
 #ifdef HAVE_GETEUID
   if(geteuid()!=0)
   {
-    static const struct MenuItem menuSudo[]=
-    {
-      {'S',"Sudo","Use the sudo command to restart as root"},
-      {'Q',"Quit",""},
-      {0,NULL,NULL}
-    };
-    unsigned int menu=0;
-    int command;
     wprintw(stdscr,"You need to be root to use %s.\n", prog_name);
 #ifdef SUDO_BIN
-    command = wmenuSelect_ext(stdscr,21, 0, menuSudo, 8,
-        "SQ", MENU_VERT | MENU_VERT_WARN | MENU_BUTTON, &menu,NULL);
-    if(command=='s' || command=='S')
-      return 1;
-    return 0;
+    {
+      static const struct MenuItem menuSudo[]=
+      {
+	{'S',"Sudo","Use the sudo command to restart as root"},
+	{'Q',"Quit",""},
+	{0,NULL,NULL}
+      };
+      unsigned int menu=0;
+      int command;
+      command = wmenuSelect_ext(stdscr,24, 21, 0, menuSudo, 8,
+	  "SQ", MENU_VERT | MENU_VERT_WARN | MENU_BUTTON, &menu,NULL);
+      if(command=='s' || command=='S')
+	return 1;
+      return 0;
+    }
 #endif
   }
 #endif
@@ -1413,7 +1438,7 @@ static int interface_partition_type_ncurses(disk_t *disk_car)
     wprintw(stdscr,"Note: Do NOT select 'None' for media with only a single partition. It's very");
     wmove(stdscr,21,0);
     wprintw(stdscr,"rare for a drive to be 'Non-partitioned'.");
-    car=wmenuSelect_ext(stdscr,INTER_PARTITION_Y, INTER_PARTITION_X, menuOptions, 7, "IGMNSXQ", MENU_BUTTON | MENU_VERT | MENU_VERT_WARN, &menu,&real_key);
+    car=wmenuSelect_ext(stdscr, 24, INTER_PARTITION_Y, INTER_PARTITION_X, menuOptions, 7, "IGMNSXQ", MENU_BUTTON | MENU_VERT | MENU_VERT_WARN, &menu,&real_key);
     switch(car)
     {
       case 'i':
@@ -1445,9 +1470,35 @@ static int interface_partition_type_ncurses(disk_t *disk_car)
         return 1;
     }
   }
+  autoset_unit(disk_car);
   return 0;
 }
 
+#if defined(DJGPP) || defined(__OS2__)
+void get_dos_drive_list(struct td_list_head *list);
+
+void get_dos_drive_list(struct td_list_head *list)
+{
+  int i;
+  for(i='a';i<='z';i++)
+  {
+    struct file_info *new_drive;
+    new_drive=(struct file_info*)MALLOC(sizeof(*new_drive));
+    new_drive->name[0]=i;
+    new_drive->name[1]=':';
+    new_drive->name[2]=PATH_SEP;
+    new_drive->name[3]='\0';
+    new_drive->stat.st_mode=LINUX_S_IFDIR|LINUX_S_IRWXUGO;
+    td_list_add_tail(&new_drive->list, list);
+  }
+}
+#endif
+
+#define ASK_LOCATION_WAITKEY 	0
+#define ASK_LOCATION_UPDATE	1
+#define ASK_LOCATION_NEWDIR	2
+#define ASK_LOCATION_QUIT	3
+
 char *ask_location(const char*msg, const char *src_dir)
 {
   char dst_directory[4096];
@@ -1455,18 +1506,10 @@ char *ask_location(const char*msg, const char *src_dir)
   int quit;
   WINDOW *window=newwin(0,0,0,0);	/* full screen */
   aff_copy(window);
-#ifdef HAVE_GETCWD
-  if(getcwd(dst_directory, sizeof(dst_directory))==NULL)
-    strncpy(dst_directory,".",sizeof(dst_directory));
-#else
-  strncpy(dst_directory,".",sizeof(dst_directory));
-#endif
+  td_getcwd(dst_directory, sizeof(dst_directory));
   do
   {
     DIR* dir;
-    struct td_list_head *dir_current;
-    int offset=0;
-    int pos_num=0;
     static struct file_info dir_list = {
       .list = TD_LIST_HEAD_INIT(dir_list.list),
       .name = {0}
@@ -1482,18 +1525,7 @@ char *ask_location(const char*msg, const char *src_dir)
 #if defined(DJGPP) || defined(__OS2__)
     if(dst_directory[0]=='\0')
     {
-      int i;
-      for(i='a';i<='z';i++)
-      {
-        struct file_info *new_drive;
-        new_drive=(struct file_info*)MALLOC(sizeof(*new_drive));
-        new_drive->name[0]=i;
-        new_drive->name[1]=':';
-        new_drive->name[2]=PATH_SEP;
-        new_drive->name[3]='\0';
-        new_drive->stat.st_mode=LINUX_S_IFDIR|LINUX_S_IRWXUGO;
-        td_list_add_tail(&new_drive->list,&dir_list.list);
-      }
+      get_dos_drive_list(&dir_list.list);
       dir=NULL;
     }
     else
@@ -1510,6 +1542,10 @@ char *ask_location(const char*msg, const char *src_dir)
       {
         char current_file[4096];
         dir_entrie=readdir(dir);
+	/* if dir_entrie exists
+	 *   there is enough room to store the filename
+	 *   dir_entrie->d_name is ".", ".." or something that doesn't begin by a "."
+	 * */
         if(dir_entrie!=NULL
             && strlen(dst_directory)+1+strlen(dir_info->name)+1<=sizeof(current_file) &&
             (dir_entrie->d_name[0]!='.' ||
@@ -1533,50 +1569,61 @@ char *ask_location(const char*msg, const char *src_dir)
 #else
             if(stat(current_file,&dir_info->stat)==0)
 #endif
-            {
-              if(S_ISDIR(dir_info->stat.st_mode))
-              {
-                /* If the C compiler doesn't use posix definition, st_mode need to be fixed */
+	    {
 #if defined(DJGPP) || defined(__OS2__)
-                dir_info->stat.st_mode=LINUX_S_IFDIR|LINUX_S_IRWXUGO;
-#elif defined(__CYGWIN__)
-                if(strlen(dst_directory)<=PATH_DRIVE_LENGTH)
-                {
-                  dir_info->stat.st_mode=LINUX_S_IFDIR|LINUX_S_IRWXUGO;
-                  dir_info->stat.st_mtime=0;
-                  dir_info->stat.st_uid=0;
-                  dir_info->stat.st_gid=0;
-                }
+	      /* If the C library doesn't use posix definition, st_mode need to be fixed */
+	      if(S_ISDIR(dir_info->stat.st_mode))
+		dir_info->stat.st_mode=LINUX_S_IFDIR|LINUX_S_IRWXUGO;
+	      else
+		dir_info->stat.st_mode=LINUX_S_IFREG|LINUX_S_IRWXUGO;
 #endif
-                strncpy(dir_info->name,dir_entrie->d_name,sizeof(dir_info->name));
-                td_list_add_tail(&dir_info->list,&dir_list.list);
-                dir_info=(struct file_info*)MALLOC(sizeof(*dir_info));
-              }
-            }
+#ifdef __CYGWIN__
+	      /* Fix Drive list */
+	      if(strlen(dst_directory)<=PATH_DRIVE_LENGTH)
+	      {
+		dir_info->stat.st_mode=LINUX_S_IFDIR|LINUX_S_IRWXUGO;
+		dir_info->stat.st_mtime=0;
+		dir_info->stat.st_uid=0;
+		dir_info->stat.st_gid=0;
+	      }
+#endif
+	      strncpy(dir_info->name,dir_entrie->d_name,sizeof(dir_info->name));
+	      td_list_add_sorted(&dir_info->list, &dir_list.list, filesort);
+	      dir_info=(struct file_info*)MALLOC(sizeof(*dir_info));
+	    }
         }
       } while(dir_entrie!=NULL);
       free(dir_info);
       closedir(dir);
     }
-    dir_current=dir_list.list.next;
     if(dir_list.list.next!=&dir_list.list)
     {
+      struct td_list_head *dir_current=dir_list.list.next;
+      int offset=0;
+      int pos_num=0;
+      int old_LINES=LINES;
       do
       {
+	int dst_directory_ok=0;
+	if(old_LINES!=LINES)
+	{ /* Screen size has changed, reset to initial values */
+	  dir_current=dir_list.list.next;
+	  offset=0;
+	  pos_num=0;
+	  old_LINES=LINES;
+	}
         aff_copy(window);
         wmove(window,7,0);
-#ifdef PATH_DRIVE_LENGTH
 #ifdef __CYGWIN__
         if(strlen(dst_directory)<=PATH_DRIVE_LENGTH)
           wprintw(window,"To select a drive, use the arrow keys.");
         else
           wprintw(window,"To select another directory, use the arrow keys.");
-#else
-        if(strlen(dst_directory)<PATH_DRIVE_LENGTH)
+#elif defined(DJGPP) || defined(__OS2__)
+        if(dst_directory[0]=='\0')
           wprintw(window,"To select a drive, use the arrow keys.");
         else
           wprintw(window,"To select another directory, use the arrow keys.");
-#endif
 #else
         wprintw(window,"To select another directory, use the arrow keys.");
 #endif
@@ -1606,73 +1653,72 @@ char *ask_location(const char*msg, const char *src_dir)
 	  if(dir_walker!=&dir_list.list && dir_walker->next!=&dir_list.list)
 	    wprintw(window, "Next");
         }
-        {
-          int line=4;
-          char beautifull_dst_directory[4096];
-          if(strcmp(dst_directory,".")==0)
-          {
-            strncpy(beautifull_dst_directory,"the program is running from",sizeof(beautifull_dst_directory));
-            line=aff_txt(line,window,msg,src_dir,beautifull_dst_directory);
-          }
-          else
-          {
+	if(strcmp(dst_directory,".")==0)
+	{
+	  aff_txt(4, window, msg, src_dir, "the program is running from");
+	  dst_directory_ok=1;
+	}
+	else
+	{
 #ifdef __CYGWIN__
-            if(strlen(dst_directory)>PATH_DRIVE_LENGTH)
-            {
-              cygwin_conv_to_win32_path(dst_directory,beautifull_dst_directory);
-              line=aff_txt(line,window,msg,src_dir,beautifull_dst_directory);
-            }
+	  if(strlen(dst_directory)>PATH_DRIVE_LENGTH)
+	  {
+	    char beautifull_dst_directory[4096];
+	    cygwin_conv_to_win32_path(dst_directory, beautifull_dst_directory);
+	    aff_txt(4, window, msg, src_dir, beautifull_dst_directory);
+	    dst_directory_ok=1;
+	  }
+#elif defined(DJGPP) || defined(__OS2__)
+	  if(strlen(dst_directory)>0)
+	  {
+	    aff_txt(4, window, msg, src_dir, dst_directory);
+	    dst_directory_ok=1;
+	  }
 #else
-#ifdef PATH_DRIVE_LENGTH
-            if(strlen(dst_directory)>=PATH_DRIVE_LENGTH)
+	  aff_txt(4, window, msg, src_dir, dst_directory);
+	  dst_directory_ok=1;
 #endif
-            {
-              strncpy(beautifull_dst_directory,dst_directory,sizeof(beautifull_dst_directory));
-              line=aff_txt(line,window,msg,src_dir,beautifull_dst_directory);
-            }
-#endif
-          }
-        }
+	}
         wclrtoeol(window);	/* before addstr for BSD compatibility */
         wrefresh(window);
         do
         {
-          quit=0;
+          quit=ASK_LOCATION_WAITKEY;
           switch(wgetch(window))
           {
             case 'y':
             case 'Y':
-#if defined __CYGWIN__
-              if(strlen(dst_directory)>=PATH_DRIVE_LENGTH)
-#endif
+              if(dst_directory_ok>0)
               {
                 res=strdup(dst_directory);
-                quit=3;
+                quit=ASK_LOCATION_QUIT;
               }
               break;
             case 'n':
             case 'N':
               res=NULL;
-              quit=3;
+              quit=ASK_LOCATION_QUIT;
               break;
             case KEY_UP:
+	    case '8':
               if(dir_current->prev!=&dir_list.list)
               {
                 dir_current=dir_current->prev;
                 pos_num--;
                 if(pos_num<offset)
                   offset--;
-                quit=1;
+                quit=ASK_LOCATION_UPDATE;
               }
               break;
             case KEY_DOWN:
+	    case '2':
               if(dir_current->next!=&dir_list.list)
               {
                 dir_current=dir_current->next;
                 pos_num++;
                 if(pos_num>=offset+INTER_DIR)
                   offset++;
-                quit=1;
+                quit=ASK_LOCATION_UPDATE;
               }
               break;
             case KEY_PPAGE:
@@ -1684,7 +1730,7 @@ char *ask_location(const char*msg, const char *src_dir)
                   pos_num--;
                   if(pos_num<offset)
                     offset--;
-                  quit=1;
+                  quit=ASK_LOCATION_UPDATE;
                 }
               }
               break;
@@ -1697,47 +1743,56 @@ char *ask_location(const char*msg, const char *src_dir)
                   pos_num++;
                   if(pos_num>=offset+INTER_DIR)
                     offset++;
-                  quit=1;
+                  quit=ASK_LOCATION_UPDATE;
                 }
               }
               break;
+	    case KEY_LEFT:
+	    case '4':
+	      set_parent_directory(dst_directory);
+	      quit=ASK_LOCATION_NEWDIR;
+	      break;
             case KEY_RIGHT:
             case '\r':
             case '\n':
-
+	    case '6':
             case KEY_ENTER:
 #ifdef PADENTER
             case PADENTER:
 #endif
-              if(dir_current!=&dir_list.list)
-              {
-                struct file_info *dir_info;
-                dir_info=td_list_entry(dir_current, struct file_info, list);
-                if(strcmp(dir_info->name,".")==0)
-                {
-                }
-                else if(strcmp(dir_info->name,"..")==0)
-                {
-                  set_parent_directory(dst_directory);
-                  quit=2;
-                }
-                else if(strlen(dst_directory)+1+strlen(dir_info->name)+1<=sizeof(dst_directory))
-                {
+	      {
+		struct file_info *dir_info;
+		dir_info=td_list_entry(dir_current, struct file_info, list);
+		if(dir_current!=&dir_list.list &&
+		  (LINUX_S_ISDIR(dir_info->stat.st_mode) || LINUX_S_ISLNK(dir_info->stat.st_mode)))
+		if(dir_current!=&dir_list.list)
+		{
+		  if(strcmp(dir_info->name,".")==0)
+		  {
+		  }
+		  else if(strcmp(dir_info->name,"..")==0)
+		  {
+		    set_parent_directory(dst_directory);
+		    quit=ASK_LOCATION_NEWDIR;
+		  }
+		  else if(strlen(dst_directory)+1+strlen(dir_info->name)+1<=sizeof(dst_directory))
+		  {
 #if defined(DJGPP) || defined(__OS2__)
-                  if(dst_directory[0]!='\0'&&dst_directory[1]!='\0'&&dst_directory[2]!='\0'&&dst_directory[3]!='\0')
+		    if(dst_directory[0]!='\0'&&dst_directory[1]!='\0'&&dst_directory[2]!='\0'&&dst_directory[3]!='\0')
 #else
-                    if(dst_directory[1]!='\0')
+		      if(dst_directory[1]!='\0')
 #endif
-                      strcat(dst_directory,SPATH_SEP);
-                  strcat(dst_directory,dir_info->name);
-                  quit=2;
-                }
-              }
+			strcat(dst_directory,SPATH_SEP);
+		    strcat(dst_directory,dir_info->name);
+		    quit=ASK_LOCATION_NEWDIR;
+		  }
+		}
+	      }
               break;
 
           }
-        } while(quit==0);
-      } while(quit==1);
+        } while(quit==ASK_LOCATION_WAITKEY && old_LINES==LINES);
+      } while(quit==ASK_LOCATION_UPDATE || old_LINES!=LINES);
       {
         struct td_list_head *dir_walker = NULL;
         struct td_list_head *dir_walker_next = NULL;
@@ -1753,9 +1808,9 @@ char *ask_location(const char*msg, const char *src_dir)
     else
     {
       set_parent_directory(dst_directory);
-      quit=2;
+      quit=ASK_LOCATION_NEWDIR;
     }
-  } while(quit==2);
+  } while(quit==ASK_LOCATION_NEWDIR);
   delwin(window);
   (void) clearok(stdscr, TRUE);
 #ifdef HAVE_TOUCHWIN
@@ -1764,7 +1819,7 @@ char *ask_location(const char*msg, const char *src_dir)
   return res;
 }
 
-int vaff_txt(int line, WINDOW *window, const char *_format, va_list ap)
+static int vaff_txt(int line, WINDOW *window, const char *_format, va_list ap)
 {
   char buffer[1024];
   int i;
@@ -1854,16 +1909,8 @@ int ask_confirmation(const char*_format, ...)
 char *ask_location(const char*msg, const char *src_dir)
 {
   char dst_directory[4096];
-  char *res=NULL;
-  int quit;
-#ifdef HAVE_GETCWD
-  if(getcwd(dst_directory, sizeof(dst_directory))==NULL)
-    strncpy(dst_directory,".",sizeof(dst_directory));
-#else
-  strncpy(dst_directory,".",sizeof(dst_directory));
-#endif
-  res=strdup(dst_directory);
-  return res;
+  td_getcwd(dst_directory, sizeof(dst_directory));
+  return strdup(dst_directory);
 }
 #endif
 
@@ -1909,7 +1956,7 @@ void aff_part_buffer(const unsigned int newline,const disk_t *disk_car,const par
 {
   const char *msg;
   msg=aff_part_aux(newline, disk_car, partition);
-  aff_buffer(BUFFER_ADD,"%s\n", msg);
+  screen_buffer_add("%s\n", msg);
 }
 
 void log_CHS_from_LBA(const disk_t *disk_car, const unsigned long int pos_LBA)
@@ -1964,6 +2011,7 @@ int interface_partition_type(disk_t *disk_car, const int verbose, char**current_
         {
           (*current_cmd)+=strlen(arch_list[i]->part_name_option);
           disk_car->arch=arch_list[i];
+	  autoset_unit(disk_car);
           keep_asking=1;
         }
       if(strncmp(*current_cmd, "ask_type", 8)==0)
