@@ -20,5 +20,5 @@
 
  */
 
-int io_redir_add_redir(t_param_disk *disk_car, const uint64_t org_offset, const unsigned int size, const uint64_t new_offset, const void *mem);
-int io_redir_del_redir(t_param_disk *disk_car, uint64_t org_offset);
+int io_redir_add_redir(disk_t *disk_car, const uint64_t org_offset, const unsigned int size, const uint64_t new_offset, const void *mem);
+int io_redir_del_redir(disk_t *disk_car, uint64_t org_offset);
