@@ -35,10 +35,11 @@
 #include "jfs.h"
 #include "fnctdsk.h"
 #include "log.h"
+#include "guid_cpy.h"
 
-static int test_JFS(disk_t *disk_car, const struct jfs_superblock *sb,partition_t *partition,const int debug, const int dump_ind);
+static int test_JFS(disk_t *disk_car, const struct jfs_superblock *sb,partition_t *partition,const int verbose, const int dump_ind);
 
-int check_JFS(disk_t *disk_car,partition_t *partition,const int debug)
+int check_JFS(disk_t *disk_car,partition_t *partition,const int verbose)
 {
   unsigned char *buffer=(unsigned char*)MALLOC(JFS_SUPERBLOCK_SIZE);
   if(disk_car->read(disk_car,JFS_SUPERBLOCK_SIZE, buffer, partition->part_offset+64*512)!=0)
@@ -46,21 +47,21 @@ int check_JFS(disk_t *disk_car,partition_t *partition,const int debug)
     free(buffer);
     return 1;
   }
-  if(test_JFS(disk_car,(struct jfs_superblock*)buffer,partition,debug,0)!=0)
+  if(test_JFS(disk_car,(struct jfs_superblock*)buffer,partition,verbose,0)!=0)
   {
     free(buffer);
     return 1;
   }
-  set_JFS_info(disk_car,(struct jfs_superblock*)buffer,partition,debug,0);
+  set_JFS_info(disk_car,(struct jfs_superblock*)buffer,partition,verbose,0);
   free(buffer);
   return 0;
 }
 
-int set_JFS_info(disk_t *disk_car, const struct jfs_superblock *sb,partition_t *partition,const int debug, const int dump_ind)
+int set_JFS_info(disk_t *disk_car, const struct jfs_superblock *sb,partition_t *partition,const int verbose, const int dump_ind)
 {
-  snprintf(partition->info,sizeof(partition->info),"JFS %u",(unsigned int)sb->s_version);
-  partition->name[0]='\0';
-  if(sb->s_version==1)
+  snprintf(partition->info,sizeof(partition->info),"JFS %u",(unsigned int)le32(sb->s_version));
+  partition->fsname[0]='\0';
+  if(le32(sb->s_version)==1)
   {
     set_part_name(partition,sb->s_fpack,11);
   }
@@ -70,22 +71,26 @@ int set_JFS_info(disk_t *disk_car, const struct jfs_superblock *sb,partition_t *
 /*
 Primary superblock is at 0x8000
 */
-int recover_JFS(disk_t *disk_car, const struct jfs_superblock *sb,partition_t *partition,const int debug, const int dump_ind)
+int recover_JFS(disk_t *disk_car, const struct jfs_superblock *sb,partition_t *partition,const int verbose, const int dump_ind)
 {
-  if(test_JFS(disk_car,sb,partition,debug,dump_ind)!=0)
+  if(test_JFS(disk_car,sb,partition,verbose,dump_ind)!=0)
     return 1;
-  set_JFS_info(disk_car,sb,partition,debug,dump_ind);
+  set_JFS_info(disk_car,sb,partition,verbose,dump_ind);
   partition->part_type_i386=P_LINUX;
   partition->part_type_sun=PSUN_LINUX;
   partition->part_type_mac=PMAC_LINUX;
-  partition->part_size=(uint64_t)sb->s_pbsize*le64(sb->s_size)+
-    sb->s_bsize*(le24(sb->s_fsckpxd.len)+le24(sb->s_logpxd.len));
-  partition->boot_sector=0;
+  partition->part_type_gpt=GPT_ENT_TYPE_LINUX_DATA;
+  partition->part_size=(uint64_t)le32(sb->s_pbsize) * le64(sb->s_size) +
+    le32(sb->s_bsize) * (le24(sb->s_fsckpxd.len)+le24(sb->s_logpxd.len));
+  partition->sborg_offset=64*512;
+  partition->sb_size=JFS_SUPERBLOCK_SIZE;
+  partition->sb_offset=0;
   partition->blocksize=le32(sb->s_bsize);
-  if(debug>0)
+  guid_cpy(&partition->part_uuid, (const efi_guid_t *)&sb->s_uuid);
+  if(verbose>0)
   {
     log_info("\n");
-    log_info("recover_JFS: s_blocksize=%lu\n",partition->blocksize);
+    log_info("recover_JFS: s_blocksize=%u\n",partition->blocksize);
     log_info("recover_JFS: s_size %lu\n",(long unsigned int)le64(sb->s_size));
     log_info("recover_JFS: s_fsckpxd.len:%d\n", (int)le24(sb->s_fsckpxd.len));
     log_info("recover_JFS: s_logpxd.len:%d\n", (int)le24(sb->s_logpxd.len));
@@ -94,7 +99,7 @@ int recover_JFS(disk_t *disk_car, const struct jfs_superblock *sb,partition_t *p
   return 0;
 }
 
-static int test_JFS(disk_t *disk_car, const struct jfs_superblock *sb,partition_t *partition,const int debug, const int dump_ind)
+static int test_JFS(disk_t *disk_car, const struct jfs_superblock *sb,partition_t *partition,const int verbose, const int dump_ind)
 {
   if(memcmp(sb->s_magic,"JFS1",4)!=0)
     return 1;
@@ -105,7 +110,7 @@ static int test_JFS(disk_t *disk_car, const struct jfs_superblock *sb,partition_
     dump_log(sb,DEFAULT_SECTOR_SIZE);
   }
   /*
-  if( sb->s_agsize >= (1 << L2BPERDMAP) ) {
+  if( le32(sb->s_agsize) >= (1 << L2BPERDMAP) ) {
     return 2;
   }
   if(partition->part_size!=0 && (partition->part_size<le64(sb->s_size)))
