@@ -48,6 +48,7 @@
 #include <stdio.h>
 #include "types.h"
 #include "common.h"
+#include "log.h"
 
 #if defined(linux) && defined(_IO) && !defined(BLKGETSIZE)
 #	define BLKGETSIZE _IO(0x12,96) /* Get device size in 512byte blocks. */
@@ -80,7 +81,7 @@ static int ntfs_device_testdisk_io_close(struct ntfs_device *dev)
 static s64 ntfs_device_testdisk_io_seek(struct ntfs_device *dev, s64 offset,
 		int whence)
 {
-  t_my_data *my_data=(t_my_data*)dev->d_private;
+  my_data_t *my_data=(my_data_t*)dev->d_private;
   switch(whence)
   {
     case SEEK_SET:
@@ -99,7 +100,7 @@ static s64 ntfs_device_testdisk_io_seek(struct ntfs_device *dev, s64 offset,
 static s64 ntfs_device_testdisk_io_read(struct ntfs_device *dev, void *buf,
 		s64 count)
 {
-  t_my_data *my_data=(t_my_data*)dev->d_private;
+  my_data_t *my_data=(my_data_t*)dev->d_private;
   if(my_data->disk_car->read(my_data->disk_car,count,buf,my_data->partition->part_offset+my_data->offset))
     return 0;
   my_data->offset+=count;
@@ -109,35 +110,22 @@ static s64 ntfs_device_testdisk_io_read(struct ntfs_device *dev, void *buf,
 static s64 ntfs_device_testdisk_io_write(struct ntfs_device *dev, const void *buf,
 		s64 count)
 {
-  t_my_data *my_data=(t_my_data*)dev->d_private;
+  my_data_t *my_data=(my_data_t*)dev->d_private;
   if(my_data->disk_car->write(my_data->disk_car,count,buf,my_data->partition->part_offset+my_data->offset))
     return 0;
   my_data->offset+=count;
   return count;
 }
 
-static s64 ntfs_device_testdisk_io_pread(struct ntfs_device *dev, void *buf,
-		s64 count, s64 offset)
-{
-	return ntfs_pread(dev, offset, count, buf);
-}
-
-static s64 ntfs_device_testdisk_io_pwrite(struct ntfs_device *dev, const void *buf,
-		s64 count, s64 offset)
-{
-	return ntfs_pwrite(dev, offset, count, buf);
-}
-
 static int ntfs_device_testdisk_io_sync(struct ntfs_device *dev)
 {
-	ecrit_rapport("ntfs_device_testdisk_io_sync() unimplemented\n");
-	errno = ENOTSUP;
-	return -1;
+  my_data_t *my_data=(my_data_t*)dev->d_private;
+  return my_data->disk_car->sync(my_data->disk_car);
 }
 
 static int ntfs_device_testdisk_io_stat(struct ntfs_device *dev, struct stat *buf)
 {
-	ecrit_rapport("ntfs_device_testdisk_io_stat() unimplemented\n");
+	log_warning("ntfs_device_testdisk_io_stat() unimplemented\n");
 	errno = ENOTSUP;
 	return -1;
 }
@@ -145,7 +133,7 @@ static int ntfs_device_testdisk_io_stat(struct ntfs_device *dev, struct stat *bu
 static int ntfs_device_testdisk_io_ioctl(struct ntfs_device *dev, int request,
 		void *argp)
 {
-	ecrit_rapport( "ntfs_device_testdisk_io_ioctl() unimplemented\n");
+	log_warning( "ntfs_device_testdisk_io_ioctl() unimplemented\n");
 	errno = ENOTSUP;
 	return -1;
 }
@@ -159,8 +147,8 @@ struct ntfs_device_operations ntfs_device_testdisk_io_ops = {
 	.seek		= ntfs_device_testdisk_io_seek,
 	.read		= ntfs_device_testdisk_io_read,
 	.write		= ntfs_device_testdisk_io_write,
-	.pread		= ntfs_device_testdisk_io_pread,
-	.pwrite		= ntfs_device_testdisk_io_pwrite,
+	.pread		= NULL,
+	.pwrite		= NULL,
 	.sync		= ntfs_device_testdisk_io_sync,
 	.stat		= ntfs_device_testdisk_io_stat,
 	.ioctl		= ntfs_device_testdisk_io_ioctl,
