@@ -2,7 +2,7 @@
 
     File: photorec.c
 
-    Copyright (C) 1998-2007 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2008 Christophe GRENIER <grenier@cgsecurity.org>
 
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -66,6 +66,9 @@
 #else
 #include <stdio.h>
 #endif
+#ifdef HAVE_JPEGLIB_H
+#include <jpeglib.h>
+#endif
 #include "dir.h"
 #include "filegen.h"
 #include "photorec.h"
@@ -79,6 +82,10 @@
 #include "phrecn.h"
 #include "hdaccess.h"
 #include "sudo.h"
+#include "phcfg.h"
+#include "misc.h"
+#include "ext2_dir.h"
+#include "ntfs_dir.h"
 
 /* #define DEBUG_FILE_FINISH */
 /* #define DEBUG_UPDATE_SEARCH_SPACE */
@@ -97,7 +104,12 @@ extern const file_hint_t file_hint_ace;
 extern const file_hint_t file_hint_addressbook;
 extern const file_hint_t file_hint_aif;
 extern const file_hint_t file_hint_all;
+extern const file_hint_t file_hint_als;
+extern const file_hint_t file_hint_amd;
+extern const file_hint_t file_hint_amr;
+extern const file_hint_t file_hint_arj;
 extern const file_hint_t file_hint_asf;
+extern const file_hint_t file_hint_asm;
 extern const file_hint_t file_hint_au;
 extern const file_hint_t file_hint_bkf;
 extern const file_hint_t file_hint_blend;
@@ -105,6 +117,7 @@ extern const file_hint_t file_hint_bmp;
 extern const file_hint_t file_hint_bz2;
 extern const file_hint_t file_hint_cab;
 extern const file_hint_t file_hint_cam;
+extern const file_hint_t file_hint_chm;
 extern const file_hint_t file_hint_cm;
 extern const file_hint_t file_hint_crw;
 extern const file_hint_t file_hint_ctg;
@@ -116,6 +129,8 @@ extern const file_hint_t file_hint_dir;
 extern const file_hint_t file_hint_djv;
 extern const file_hint_t file_hint_doc;
 extern const file_hint_t file_hint_dpx;
+extern const file_hint_t file_hint_drw;
+extern const file_hint_t file_hint_ds2;
 extern const file_hint_t file_hint_dsc;
 extern const file_hint_t file_hint_dss;
 extern const file_hint_t file_hint_dta;
@@ -128,23 +143,33 @@ extern const file_hint_t file_hint_evt;
 extern const file_hint_t file_hint_exe;
 extern const file_hint_t file_hint_ext2_sb;
 extern const file_hint_t file_hint_fcp;
+extern const file_hint_t file_hint_fcs;
 extern const file_hint_t file_hint_fh10;
 extern const file_hint_t file_hint_fh5;
+extern const file_hint_t file_hint_fits;
 extern const file_hint_t file_hint_flac;
 extern const file_hint_t file_hint_flv;
+extern const file_hint_t file_hint_frm;
 extern const file_hint_t file_hint_fs;
 extern const file_hint_t file_hint_gif;
+extern const file_hint_t file_hint_gho;
+extern const file_hint_t file_hint_gpg;
 extern const file_hint_t file_hint_gz;
+extern const file_hint_t file_hint_ifo;
 extern const file_hint_t file_hint_imb;
 extern const file_hint_t file_hint_indd;
+extern const file_hint_t file_hint_iso;
 extern const file_hint_t file_hint_itunes;
 extern const file_hint_t file_hint_jpg;
 extern const file_hint_t file_hint_kdb;
 extern const file_hint_t file_hint_lnk;
+extern const file_hint_t file_hint_m2ts;
 extern const file_hint_t file_hint_max;
 extern const file_hint_t file_hint_mb;
+extern const file_hint_t file_hint_mcd;
 extern const file_hint_t file_hint_mdb;
 extern const file_hint_t file_hint_mdf;
+extern const file_hint_t file_hint_mfg;
 extern const file_hint_t file_hint_mid;
 extern const file_hint_t file_hint_mkv;
 extern const file_hint_t file_hint_mov;
@@ -157,14 +182,18 @@ extern const file_hint_t file_hint_njx;
 extern const file_hint_t file_hint_ogg;
 extern const file_hint_t file_hint_one;
 extern const file_hint_t file_hint_orf;
+extern const file_hint_t file_hint_paf;
 extern const file_hint_t file_hint_pap;
 extern const file_hint_t file_hint_pct;
 extern const file_hint_t file_hint_pcx;
 extern const file_hint_t file_hint_pdf;
+extern const file_hint_t file_hint_pfx;
 extern const file_hint_t file_hint_png;
 extern const file_hint_t file_hint_prc;
+extern const file_hint_t file_hint_prt;
 extern const file_hint_t file_hint_ps;
 extern const file_hint_t file_hint_psd;
+extern const file_hint_t file_hint_psp;
 extern const file_hint_t file_hint_pst;
 extern const file_hint_t file_hint_ptb;
 extern const file_hint_t file_hint_qbb;
@@ -181,15 +210,19 @@ extern const file_hint_t file_hint_riff;
 extern const file_hint_t file_hint_rm;
 extern const file_hint_t file_hint_rns;
 extern const file_hint_t file_hint_rpm;
+extern const file_hint_t file_hint_sib;
 extern const file_hint_t file_hint_sit;
 extern const file_hint_t file_hint_skp;
 extern const file_hint_t file_hint_spe;
 extern const file_hint_t file_hint_spss;
+extern const file_hint_t file_hint_sqlite;
+extern const file_hint_t file_hint_stl;
 extern const file_hint_t file_hint_stuffit;
 extern const file_hint_t file_hint_swf;
 extern const file_hint_t file_hint_tar;
 extern const file_hint_t file_hint_tib;
 extern const file_hint_t file_hint_tiff;
+extern const file_hint_t file_hint_tph;
 extern const file_hint_t file_hint_fasttxt;
 extern const file_hint_t file_hint_txt;
 extern const file_hint_t file_hint_vmdk;
@@ -199,6 +232,8 @@ extern const file_hint_t file_hint_x3f;
 extern const file_hint_t file_hint_xcf;
 extern const file_hint_t file_hint_xm;
 extern const file_hint_t file_hint_xsv;
+extern const file_hint_t file_hint_veg;
+extern const file_hint_t file_hint_wks;
 extern const file_hint_t file_hint_zip;
 
 static alloc_data_t *update_search_space(const file_recovery_t *file_recovery, alloc_data_t *list_search_space, alloc_data_t **new_current_search_space, uint64_t *offset, const unsigned int blocksize);
@@ -303,12 +338,12 @@ static alloc_data_t *update_search_space(const file_recovery_t *file_recovery, a
   return list_search_space;
 }
 
-alloc_data_t *del_search_space(alloc_data_t *list_search_space, uint64_t start, uint64_t end)
+alloc_data_t *del_search_space(alloc_data_t *list_search_space, const uint64_t start, const uint64_t end)
 {
   return update_search_space_aux(list_search_space, start, end, NULL, NULL);
 }
 
-static alloc_data_t *update_search_space_aux(alloc_data_t *list_search_space, uint64_t start, uint64_t end, alloc_data_t **new_current_search_space, uint64_t *offset)
+static alloc_data_t *update_search_space_aux(alloc_data_t *list_search_space, const uint64_t start, const uint64_t end, alloc_data_t **new_current_search_space, uint64_t *offset)
 {
   struct td_list_head *search_walker = NULL;
 #ifdef DEBUG_UPDATE_SEARCH_SPACE
@@ -333,8 +368,9 @@ static alloc_data_t *update_search_space_aux(alloc_data_t *list_search_space, ui
 #endif
     if(current_search_space->start==start)
     {
-      if(end<current_search_space->end)
-      { /* current_search_space->start==start end<current_search_space->end */
+      const uint64_t pivot=current_search_space->end+1;
+      if(end+1<current_search_space->end)
+      { /* current_search_space->start==start end+1<current_search_space->end */
         if(offset!=NULL && new_current_search_space!=NULL &&
             current_search_space->start<=*offset && *offset<=end)
         {
@@ -346,7 +382,6 @@ static alloc_data_t *update_search_space_aux(alloc_data_t *list_search_space, ui
         return list_search_space;
       }
       /* current_search_space->start==start current_search_space->end<=end */
-      start=current_search_space->end+1;
       if(list_search_space==current_search_space)
         list_search_space=td_list_entry(current_search_space->list.next, alloc_data_t, list);
       if(offset!=NULL && new_current_search_space!=NULL &&
@@ -357,14 +392,15 @@ static alloc_data_t *update_search_space_aux(alloc_data_t *list_search_space, ui
       }
       td_list_del(search_walker);
       free(current_search_space);
-      return update_search_space_aux(list_search_space, start,end, new_current_search_space, offset);
+      return update_search_space_aux(list_search_space, pivot, end, new_current_search_space, offset);
     }
     if(current_search_space->end==end)
     {
+      const uint64_t pivot=current_search_space->start-1;
 #ifdef DEBUG_UPDATE_SEARCH_SPACE
       log_trace("current_search_space->end==end\n");
 #endif
-      if(current_search_space->start<start)
+      if(current_search_space->start+1<start)
       { /* current_search_space->start<start current_search_space->end==end */
         if(offset!=NULL && new_current_search_space!=NULL &&
             start<=*offset && *offset<=current_search_space->end)
@@ -376,7 +412,6 @@ static alloc_data_t *update_search_space_aux(alloc_data_t *list_search_space, ui
         return list_search_space;
       }
       /* start<=current_search_space->start current_search_space->end==end */
-      end=current_search_space->start-1;
       if(list_search_space==current_search_space)
         list_search_space=td_list_entry(current_search_space->list.next, alloc_data_t, list);
       if(offset!=NULL && new_current_search_space!=NULL &&
@@ -387,18 +422,19 @@ static alloc_data_t *update_search_space_aux(alloc_data_t *list_search_space, ui
       }
       td_list_del(search_walker);
       free(current_search_space);
-      return update_search_space_aux(list_search_space, start,end, new_current_search_space, offset);
+      return update_search_space_aux(list_search_space, start, pivot, new_current_search_space, offset);
     }
     if(start < current_search_space->start && current_search_space->start <= end)
     {
-      list_search_space=update_search_space_aux(list_search_space,
-          start, current_search_space->start-1,  new_current_search_space, offset);
-      return update_search_space_aux(list_search_space, current_search_space->start, end, new_current_search_space, offset);
+      const uint64_t pivot=current_search_space->start;
+      list_search_space=update_search_space_aux(list_search_space, start, pivot-1,  new_current_search_space, offset);
+      return update_search_space_aux(list_search_space, pivot, end, new_current_search_space, offset);
     }
     if(start <= current_search_space->end && current_search_space->end < end)
     {
-      list_search_space=update_search_space_aux(list_search_space, start, current_search_space->end, new_current_search_space, offset);
-      return update_search_space_aux(list_search_space, current_search_space->end+1, end, new_current_search_space, offset);
+      const uint64_t pivot=current_search_space->end;
+      list_search_space=update_search_space_aux(list_search_space, start, pivot, new_current_search_space, offset);
+      return update_search_space_aux(list_search_space, pivot+1, end, new_current_search_space, offset);
     }
     if(current_search_space->start < start && end < current_search_space->end)
     {
@@ -420,20 +456,20 @@ static alloc_data_t *update_search_space_aux(alloc_data_t *list_search_space, ui
   return list_search_space;
 }
 
-alloc_data_t *init_search_space(const partition_t *partition, const disk_t *disk_car)
+void init_search_space(alloc_data_t *list_search_space, const disk_t *disk_car, const partition_t *partition)
 {
-  alloc_data_t *list_search_space;
-  list_search_space=(alloc_data_t*)MALLOC(sizeof(*list_search_space));
-  list_search_space->start=partition->part_offset;
-  list_search_space->end=partition->part_offset+partition->part_size-1;
-  if(list_search_space->end > disk_car->disk_size-1)
-    list_search_space->end = disk_car->disk_size-1;
-  if(list_search_space->end > disk_car->disk_real_size-1)
-    list_search_space->end = disk_car->disk_real_size-1;
-  list_search_space->file_stat=NULL;
-  list_search_space->list.prev=&list_search_space->list;
-  list_search_space->list.next=&list_search_space->list;
-  return list_search_space;
+  alloc_data_t *new_sp;
+  new_sp=(alloc_data_t*)MALLOC(sizeof(*new_sp));
+  new_sp->start=partition->part_offset;
+  new_sp->end=partition->part_offset+partition->part_size-1;
+  if(new_sp->end > disk_car->disk_size-1)
+    new_sp->end = disk_car->disk_size-1;
+  if(new_sp->end > disk_car->disk_real_size-1)
+    new_sp->end = disk_car->disk_real_size-1;
+  new_sp->file_stat=NULL;
+  new_sp->list.prev=&new_sp->list;
+  new_sp->list.next=&new_sp->list;
+  td_list_add_tail(&new_sp->list, &list_search_space->list);
 }
 
 void free_list_search_space(alloc_data_t *list_search_space)
@@ -452,6 +488,7 @@ void free_list_search_space(alloc_data_t *list_search_space)
 void reset_file_recovery(file_recovery_t *file_recovery)
 {
   file_recovery->filename[0]='\0';
+  file_recovery->time=0;
   file_recovery->file_stat=NULL;
   file_recovery->handle=NULL;
   file_recovery->file_size=0;
@@ -487,7 +524,7 @@ unsigned int photorec_mkdir(const char *recup_dir, const unsigned int initial_di
       if(mkdir(working_recup_dir, 0775)!=0 && errno==EEXIST)
 #endif
 #else
-#warn You need a mkdir function!
+#warning You need a mkdir function!
 #endif
       {
 	dir_num++;
@@ -575,7 +612,7 @@ unsigned int remove_used_space(disk_t *disk_car, const partition_t *partition, a
     return ntfs_remove_used_space(disk_car, partition, list_search_space);
 #endif
 #ifdef HAVE_LIBEXT2FS
-  else if(partition->upart_type==UP_EXT2 || partition->upart_type==UP_EXT3)
+  else if(partition->upart_type==UP_EXT2 || partition->upart_type==UP_EXT3 || partition->upart_type==UP_EXT4)
     return ext2_remove_used_space(disk_car, partition, list_search_space);
 #endif
   return 0;
@@ -747,7 +784,7 @@ int main( int argc, char **argv )
 {
   int i;
   int use_sudo=0;
-  int help=0, verbose=0;
+  int help=0, version=0, verbose=0;
   int create_log=TD_LOG_NONE;
   int run_setlocale=1;
   int testdisk_mode=TESTDISK_O_RDONLY|TESTDISK_O_READAHEAD_32K;
@@ -768,116 +805,144 @@ int main( int argc, char **argv )
 #endif
   file_enable_t list_file_enable[]=
   {
-    { .enable=1, .file_hint=&file_hint_7z   },
-    { .enable=1, .file_hint=&file_hint_a    },
-    { .enable=1, .file_hint=&file_hint_accdb},
-    { .enable=1, .file_hint=&file_hint_ace  },
-    { .enable=1, .file_hint=&file_hint_addressbook},
-    { .enable=1, .file_hint=&file_hint_aif  },
-    { .enable=1, .file_hint=&file_hint_all  },
-    { .enable=1, .file_hint=&file_hint_asf  },
-    { .enable=1, .file_hint=&file_hint_au   },
-    { .enable=1, .file_hint=&file_hint_bkf  },
-    { .enable=1, .file_hint=&file_hint_blend },
-    { .enable=1, .file_hint=&file_hint_bmp  },
-    { .enable=1, .file_hint=&file_hint_bz2  },
-    { .enable=1, .file_hint=&file_hint_cab  },
-    { .enable=1, .file_hint=&file_hint_cam  },
-    { .enable=1, .file_hint=&file_hint_cm  },
-    { .enable=1, .file_hint=&file_hint_crw  },
-    { .enable=1, .file_hint=&file_hint_ctg  },
-    { .enable=1, .file_hint=&file_hint_cwk  },
-    { .enable=1, .file_hint=&file_hint_dat  },
+    { .enable=0, .file_hint=&file_hint_7z   },
+    { .enable=0, .file_hint=&file_hint_a    },
+    { .enable=0, .file_hint=&file_hint_accdb},
+    { .enable=0, .file_hint=&file_hint_ace  },
+    { .enable=0, .file_hint=&file_hint_addressbook},
+    { .enable=0, .file_hint=&file_hint_aif  },
+    { .enable=0, .file_hint=&file_hint_all  },
+    { .enable=0, .file_hint=&file_hint_als  },
+    { .enable=0, .file_hint=&file_hint_amd  },
+    { .enable=0, .file_hint=&file_hint_amr  },
+    { .enable=0, .file_hint=&file_hint_arj  },
+    { .enable=0, .file_hint=&file_hint_asf  },
+    { .enable=0, .file_hint=&file_hint_asm  },
+    { .enable=0, .file_hint=&file_hint_au   },
+    { .enable=0, .file_hint=&file_hint_bkf  },
+    { .enable=0, .file_hint=&file_hint_blend },
+    { .enable=0, .file_hint=&file_hint_bmp  },
+    { .enable=0, .file_hint=&file_hint_bz2  },
+    { .enable=0, .file_hint=&file_hint_cab  },
+    { .enable=0, .file_hint=&file_hint_cam  },
+    { .enable=0, .file_hint=&file_hint_chm  },
+    { .enable=0, .file_hint=&file_hint_cm  },
+    { .enable=0, .file_hint=&file_hint_crw  },
+    { .enable=0, .file_hint=&file_hint_ctg  },
+    { .enable=0, .file_hint=&file_hint_cwk  },
+    { .enable=0, .file_hint=&file_hint_dat  },
     { .enable=0, .file_hint=&file_hint_dbf  },
-    { .enable=1, .file_hint=&file_hint_dim  },
-    { .enable=1, .file_hint=&file_hint_dir  },
-    { .enable=1, .file_hint=&file_hint_djv  },
-    { .enable=1, .file_hint=&file_hint_doc  },
-    { .enable=1, .file_hint=&file_hint_dpx  },
-    { .enable=1, .file_hint=&file_hint_dsc  },
-    { .enable=1, .file_hint=&file_hint_dss  },
+    { .enable=0, .file_hint=&file_hint_dim  },
+    { .enable=0, .file_hint=&file_hint_dir  },
+    { .enable=0, .file_hint=&file_hint_djv  },
+    { .enable=0, .file_hint=&file_hint_drw  },
+    { .enable=0, .file_hint=&file_hint_doc  },
+    { .enable=0, .file_hint=&file_hint_dpx  },
+    { .enable=0, .file_hint=&file_hint_ds2  },
+    { .enable=0, .file_hint=&file_hint_dsc  },
+    { .enable=0, .file_hint=&file_hint_dss  },
     { .enable=0, .file_hint=&file_hint_dta  },
-    { .enable=1, .file_hint=&file_hint_dump },
-    { .enable=1, .file_hint=&file_hint_dv   },
-    { .enable=1, .file_hint=&file_hint_dwg  },
-    { .enable=1, .file_hint=&file_hint_elf  },
-    { .enable=1, .file_hint=&file_hint_emf  },
-    { .enable=1, .file_hint=&file_hint_evt  },
-    { .enable=1, .file_hint=&file_hint_exe  },
-    { .enable=1, .file_hint=&file_hint_ext2_sb },
-    { .enable=1, .file_hint=&file_hint_fcp  },
-    { .enable=1, .file_hint=&file_hint_fh10  },
-    { .enable=1, .file_hint=&file_hint_fh5  },
-    { .enable=1, .file_hint=&file_hint_flac },
-    { .enable=1, .file_hint=&file_hint_flv  },
-    { .enable=1, .file_hint=&file_hint_fs   },
-    { .enable=1, .file_hint=&file_hint_gif  },
-    { .enable=1, .file_hint=&file_hint_gz   },
-    { .enable=1, .file_hint=&file_hint_imb  },
-    { .enable=1, .file_hint=&file_hint_indd  },
-    { .enable=1, .file_hint=&file_hint_itunes  },
-    { .enable=1, .file_hint=&file_hint_kdb  },
-    { .enable=1, .file_hint=&file_hint_lnk  },
-    { .enable=1, .file_hint=&file_hint_jpg  },
-    { .enable=1, .file_hint=&file_hint_max  },
-    { .enable=1, .file_hint=&file_hint_mb   },
-    { .enable=1, .file_hint=&file_hint_mdb  },
-    { .enable=1, .file_hint=&file_hint_mdf  },
-    { .enable=1, .file_hint=&file_hint_mid  },
-    { .enable=1, .file_hint=&file_hint_mkv  },
-    { .enable=1, .file_hint=&file_hint_mov  },
-    { .enable=1, .file_hint=&file_hint_mp3  },
-    { .enable=1, .file_hint=&file_hint_mpg  },
-    { .enable=1, .file_hint=&file_hint_mrw  },
-    { .enable=1, .file_hint=&file_hint_mus  },
-    { .enable=1, .file_hint=&file_hint_mysql },
-    { .enable=1, .file_hint=&file_hint_njx  },
-    { .enable=1, .file_hint=&file_hint_ogg  },
-    { .enable=1, .file_hint=&file_hint_one  },
-    { .enable=1, .file_hint=&file_hint_orf  },
-    { .enable=1, .file_hint=&file_hint_pap  },
-    { .enable=1, .file_hint=&file_hint_pct  },
-    { .enable=1, .file_hint=&file_hint_pcx  },
-    { .enable=1, .file_hint=&file_hint_pdf  },
-    { .enable=1, .file_hint=&file_hint_png  },
-    { .enable=1, .file_hint=&file_hint_prc  },
-    { .enable=1, .file_hint=&file_hint_ps   },
-    { .enable=1, .file_hint=&file_hint_psd  },
-    { .enable=1, .file_hint=&file_hint_pst  },
-    { .enable=1, .file_hint=&file_hint_ptb  },
-    { .enable=1, .file_hint=&file_hint_qbb  },
-    { .enable=1, .file_hint=&file_hint_qdf  },
-    { .enable=1, .file_hint=&file_hint_qxd  },
-    { .enable=1, .file_hint=&file_hint_ra  },
-    { .enable=1, .file_hint=&file_hint_raf  },
-    { .enable=1, .file_hint=&file_hint_rar  },
-    { .enable=1, .file_hint=&file_hint_raw  },
-    { .enable=1, .file_hint=&file_hint_rdc  },
-    { .enable=1, .file_hint=&file_hint_reg  },
-    { .enable=1, .file_hint=&file_hint_res  },
-    { .enable=1, .file_hint=&file_hint_riff },
-    { .enable=1, .file_hint=&file_hint_rm   },
-    { .enable=1, .file_hint=&file_hint_rns  },
-    { .enable=1, .file_hint=&file_hint_rpm  },
-    { .enable=1, .file_hint=&file_hint_sit  },
-    { .enable=1, .file_hint=&file_hint_skp  },
-    { .enable=1, .file_hint=&file_hint_spe  },
-    { .enable=1, .file_hint=&file_hint_spss },
-    { .enable=1, .file_hint=&file_hint_stuffit  },
-    { .enable=1, .file_hint=&file_hint_swf  },
-    { .enable=1, .file_hint=&file_hint_tar  },
-    { .enable=1, .file_hint=&file_hint_tib  },
-    { .enable=1, .file_hint=&file_hint_tiff },
-    { .enable=1, .file_hint=&file_hint_fasttxt  },
-    { .enable=1, .file_hint=&file_hint_txt  },
-    { .enable=1, .file_hint=&file_hint_vmdk },
-    { .enable=1, .file_hint=&file_hint_wpd  },
-    { .enable=1, .file_hint=&file_hint_wmf  },
-    { .enable=1, .file_hint=&file_hint_x3f  },
-    { .enable=1, .file_hint=&file_hint_xcf  },
-    { .enable=1, .file_hint=&file_hint_xm   },
-    { .enable=1, .file_hint=&file_hint_xsv  },
-    { .enable=1, .file_hint=&file_hint_zip  },
+    { .enable=0, .file_hint=&file_hint_dump },
+    { .enable=0, .file_hint=&file_hint_dv   },
+    { .enable=0, .file_hint=&file_hint_dwg  },
+    { .enable=0, .file_hint=&file_hint_elf  },
+    { .enable=0, .file_hint=&file_hint_emf  },
+    { .enable=0, .file_hint=&file_hint_evt  },
+    { .enable=0, .file_hint=&file_hint_exe  },
+    { .enable=0, .file_hint=&file_hint_ext2_sb },
+    { .enable=0, .file_hint=&file_hint_fcp  },
+    { .enable=0, .file_hint=&file_hint_fcs  },
+    { .enable=0, .file_hint=&file_hint_fh10  },
+    { .enable=0, .file_hint=&file_hint_fh5  },
+    { .enable=0, .file_hint=&file_hint_fits },
+    { .enable=0, .file_hint=&file_hint_flac },
+    { .enable=0, .file_hint=&file_hint_flv  },
+    { .enable=0, .file_hint=&file_hint_frm  },
+    { .enable=0, .file_hint=&file_hint_fs   },
+    { .enable=0, .file_hint=&file_hint_gho  },
+    { .enable=0, .file_hint=&file_hint_gif  },
+    { .enable=0, .file_hint=&file_hint_gpg  },
+    { .enable=0, .file_hint=&file_hint_gz   },
+    { .enable=0, .file_hint=&file_hint_ifo  },
+    { .enable=0, .file_hint=&file_hint_imb  },
+    { .enable=0, .file_hint=&file_hint_indd  },
+    { .enable=0, .file_hint=&file_hint_iso  },
+    { .enable=0, .file_hint=&file_hint_itunes  },
+    { .enable=0, .file_hint=&file_hint_jpg  },
+    { .enable=0, .file_hint=&file_hint_kdb  },
+    { .enable=0, .file_hint=&file_hint_lnk  },
+    { .enable=0, .file_hint=&file_hint_m2ts },
+    { .enable=0, .file_hint=&file_hint_max  },
+    { .enable=0, .file_hint=&file_hint_mb   },
+    { .enable=0, .file_hint=&file_hint_mcd  },
+    { .enable=0, .file_hint=&file_hint_mdb  },
+    { .enable=0, .file_hint=&file_hint_mdf  },
+    { .enable=0, .file_hint=&file_hint_mfg  },
+    { .enable=0, .file_hint=&file_hint_mid  },
+    { .enable=0, .file_hint=&file_hint_mkv  },
+    { .enable=0, .file_hint=&file_hint_mov  },
+    { .enable=0, .file_hint=&file_hint_mp3  },
+    { .enable=0, .file_hint=&file_hint_mpg  },
+    { .enable=0, .file_hint=&file_hint_mrw  },
+    { .enable=0, .file_hint=&file_hint_mus  },
+    { .enable=0, .file_hint=&file_hint_mysql },
+    { .enable=0, .file_hint=&file_hint_njx  },
+    { .enable=0, .file_hint=&file_hint_ogg  },
+    { .enable=0, .file_hint=&file_hint_one  },
+    { .enable=0, .file_hint=&file_hint_orf  },
+    { .enable=0, .file_hint=&file_hint_paf  },
+    { .enable=0, .file_hint=&file_hint_pap  },
+    { .enable=0, .file_hint=&file_hint_pct  },
+    { .enable=0, .file_hint=&file_hint_pcx  },
+    { .enable=0, .file_hint=&file_hint_pdf  },
+    { .enable=0, .file_hint=&file_hint_pfx  },
+    { .enable=0, .file_hint=&file_hint_png  },
+    { .enable=0, .file_hint=&file_hint_prc  },
+    { .enable=0, .file_hint=&file_hint_prt  },
+    { .enable=0, .file_hint=&file_hint_ps   },
+    { .enable=0, .file_hint=&file_hint_psd  },
+    { .enable=0, .file_hint=&file_hint_psp  },
+    { .enable=0, .file_hint=&file_hint_pst  },
+    { .enable=0, .file_hint=&file_hint_ptb  },
+    { .enable=0, .file_hint=&file_hint_qbb  },
+    { .enable=0, .file_hint=&file_hint_qdf  },
+    { .enable=0, .file_hint=&file_hint_qxd  },
+    { .enable=0, .file_hint=&file_hint_ra  },
+    { .enable=0, .file_hint=&file_hint_raf  },
+    { .enable=0, .file_hint=&file_hint_rar  },
+    { .enable=0, .file_hint=&file_hint_raw  },
+    { .enable=0, .file_hint=&file_hint_rdc  },
+    { .enable=0, .file_hint=&file_hint_reg  },
+    { .enable=0, .file_hint=&file_hint_res  },
+    { .enable=0, .file_hint=&file_hint_riff },
+    { .enable=0, .file_hint=&file_hint_rm   },
+    { .enable=0, .file_hint=&file_hint_rns  },
+    { .enable=0, .file_hint=&file_hint_rpm  },
+    { .enable=0, .file_hint=&file_hint_sib  },
+    { .enable=0, .file_hint=&file_hint_sit  },
+    { .enable=0, .file_hint=&file_hint_skp  },
+    { .enable=0, .file_hint=&file_hint_spe  },
+    { .enable=0, .file_hint=&file_hint_spss },
+    { .enable=0, .file_hint=&file_hint_sqlite	},
+    { .enable=0, .file_hint=&file_hint_stl  },
+    { .enable=0, .file_hint=&file_hint_stuffit  },
+    { .enable=0, .file_hint=&file_hint_swf  },
+    { .enable=0, .file_hint=&file_hint_tar  },
+    { .enable=0, .file_hint=&file_hint_tib  },
+    { .enable=0, .file_hint=&file_hint_tiff },
+    { .enable=0, .file_hint=&file_hint_tph  },
+    { .enable=0, .file_hint=&file_hint_fasttxt  },
+    { .enable=0, .file_hint=&file_hint_txt  },
+    { .enable=0, .file_hint=&file_hint_vmdk },
+    { .enable=0, .file_hint=&file_hint_wpd  },
+    { .enable=0, .file_hint=&file_hint_wmf  },
+    { .enable=0, .file_hint=&file_hint_x3f  },
+    { .enable=0, .file_hint=&file_hint_xcf  },
+    { .enable=0, .file_hint=&file_hint_xm   },
+    { .enable=0, .file_hint=&file_hint_xsv  },
+    { .enable=0, .file_hint=&file_hint_veg  },
+    { .enable=0, .file_hint=&file_hint_wks  },
+    { .enable=0, .file_hint=&file_hint_zip  },
     { .enable=0, .file_hint=NULL }
   };
   /* random (weak is ok) is need fot GPT */
@@ -911,7 +976,7 @@ int main( int argc, char **argv )
       int len=strlen(argv[i+1]);
       if(argv[i+1][len-1]=='\\' || argv[i+1][len-1]=='/')
       {
-        char *new_recup_dir=MALLOC(len+strlen(DEFAULT_RECUP_DIR)+1);
+        char *new_recup_dir=(char *)MALLOC(len+strlen(DEFAULT_RECUP_DIR)+1);
         strcpy(new_recup_dir,argv[i+1]);
         strcat(new_recup_dir,DEFAULT_RECUP_DIR);
         recup_dir=new_recup_dir;	/* small memory leak */
@@ -927,6 +992,9 @@ int main( int argc, char **argv )
     else if((strcmp(argv[i],"/help")==0) || (strcmp(argv[i],"-help")==0) || (strcmp(argv[i],"--help")==0) ||
       (strcmp(argv[i],"/h")==0) || (strcmp(argv[i],"-h")==0))
       help=1;
+    else if((strcmp(argv[i],"/version")==0) || (strcmp(argv[i],"-version")==0) || (strcmp(argv[i],"--version")==0) ||
+      (strcmp(argv[i],"/v")==0) || (strcmp(argv[i],"-v")==0))
+      version=1;
     else if((strcmp(argv[i],"/nosetlocale")==0) || (strcmp(argv[i],"-nosetlocale")==0))
       run_setlocale=0;
     else if(strcmp(argv[i],"/cmd")==0)
@@ -961,9 +1029,30 @@ int main( int argc, char **argv )
         list_disk=insert_new_disk(list_disk,disk_car);
     }
   }
+  if(version!=0)
+  {
+    printf("\n");
+    printf("Version: %s\n", VERSION);
+    printf("Compiler: %s\n", get_compiler());
+    printf("ext2fs lib: %s, ntfs lib: %s, ewf lib: %s, libjpeg: ",
+	td_ext2fs_version(), td_ntfs_version(), td_ewf_version());
+#if defined(HAVE_LIBJPEG)
+#if defined(JPEG_LIB_VERSION)
+    printf("%u", JPEG_LIB_VERSION);
+#else
+    printf("yes");
+#endif
+#else
+    printf("none");
+#endif
+    printf("\n");
+    printf("OS: %s\n" , get_os());
+    return 0;
+  }
   if(help!=0)
   {
-    printf("\nUsage: photorec [/log] [/debug] [/d recup_dir] [file or device]\n"\
+    printf("\nUsage: photorec [/log] [/debug] [/d recup_dir] [file.dd|file.e01|device]\n"\
+	"       photorec /version\n" \
         "\n" \
         "/log          : create a photorec.log file\n" \
         "/debug        : add debug information\n" \
@@ -974,7 +1063,7 @@ int main( int argc, char **argv )
         "If you have problems with PhotoRec or bug reports, please contact me.\n");
     return 0;
   }
-  aff_buffer(BUFFER_RESET,"Q");
+  screen_buffer_reset();
 #ifdef HAVE_SETLOCALE
   if(run_setlocale>0)
   {
@@ -994,15 +1083,19 @@ int main( int argc, char **argv )
     return 1;
 #endif
   create_log=log_open("photorec.log", create_log, 1, "PhotoRec", argc, argv);
-  log_info("PhotoRec %s, Data Recovery Utility, %s\nChristophe GRENIER <grenier@cgsecurity.org>\nhttp://www.cgsecurity.org\n",VERSION,TESTDISKDATE);
-  log_info(TESTDISK_OS);
-  log_info(" (ewf lib: %s)\n", td_ewf_version());
-#if 0
-#if defined(HAVE_LIBJPEG) && defined(JPEG_LIB_VERSION)
-  log_info(", libjpeg: %u)\n", JPEG_LIB_VERSION);
+  log_info("PhotoRec %s, Data Recovery Utility, %s\nChristophe GRENIER <grenier@cgsecurity.org>\nhttp://www.cgsecurity.org\n", VERSION, TESTDISKDATE);
+  log_info("OS: %s\n" , get_os());
+  log_info("Compiler: %s\n", get_compiler());
+  log_info("ext2fs lib: %s, ntfs lib: %s, ewf lib: %s, libjpeg: ",
+      td_ext2fs_version(), td_ntfs_version(), td_ewf_version());
+#if defined(HAVE_LIBJPEG)
+#if defined(JPEG_LIB_VERSION)
+  log_info("%u", JPEG_LIB_VERSION);
 #else
-  log_info(", libjpeg: none)\n");
+  log_info("yes");
 #endif
+#else
+  log_info("none");
 #endif
   log_info("\n");
 #if defined(__CYGWIN__) || defined(__MINGW32__) || defined(DJGPP)
@@ -1040,6 +1133,7 @@ int main( int argc, char **argv )
 	  disk->description(disk), disk->sector_size, disk->model);
   }
   log_info("\n");
+  file_options_load(list_file_enable);
   use_sudo=do_curses_photorec(verbose, recup_dir, list_disk, list_file_enable, cmd_device, &cmd_run);
 #ifdef HAVE_NCURSES
   end_ncurses();
@@ -1206,6 +1300,8 @@ int file_finish(file_recovery_t *file_recovery, const char *recup_dir, const int
     //    log_debug("%s %llu\n",file_recovery->filename,(long long unsigned)file_recovery->file_size);
     if(file_recovery->file_size>0)
     {
+      if(file_recovery->time!=0 && file_recovery->time!=(time_t)-1)
+	set_date(file_recovery->filename, file_recovery->time, file_recovery->time);
       if((++(*file_nbr))%MAX_FILES_PER_DIR==0)
       {
         *dir_num=photorec_mkdir(recup_dir,*dir_num+1);
