@@ -2,7 +2,7 @@
 
     File: phrecn.c
 
-    Copyright (C) 1998-2007 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2008 Christophe GRENIER <grenier@cgsecurity.org>
 
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -47,6 +47,7 @@
 #else
 #include <stdio.h>
 #endif
+#include <errno.h>
 #include "fnctdsk.h"
 #include "dir.h"
 #include "fat_dir.h"
@@ -61,11 +62,13 @@
 #include "log.h"
 #include "hdaccess.h"
 #include "file_tar.h"
+#include "phcfg.h"
+#include "ext2grp.h"
 
 /* #define DEBUG */
 /* #define DEBUG_GET_NEXT_SECTOR */
 /* #define DEBUG_BF */
-#define READ_SIZE 256*512
+#define READ_SIZE 1024*512
 #define INTER_MENU_DISK 10
 
 extern const file_hint_t file_hint_tar;
@@ -84,6 +87,7 @@ static int photorec(disk_t *disk_car, partition_t *partition, const int verbose,
 static void interface_options_photorec(int *paranoid, int *allow_partial_last_cylinder, int *keep_corrupted_file, unsigned int *mode_ext2, unsigned int *expert, unsigned int *lowmem, char**current_cmd);
 static int photorec_bf_aux(disk_t *disk_car, partition_t *partition, const int paranoid, const char *recup_dir, const int interface, file_stat_t *file_stats, unsigned int *file_nbr, file_recovery_t *file_recovery, unsigned int blocksize, alloc_data_t *list_search_space, alloc_data_t *current_search_space, const time_t real_start_time, unsigned int *dir_num, const photorec_status_t status, const unsigned int pass);
 static void interface_file_select(file_enable_t *files_enable, char**current_cmd);
+static int interface_cannot_create_file(void);
 
 /* ==================== INLINE FUNCTIONS ========================= */
 /* Check if the block looks like an indirect/double-indirect block */
@@ -254,42 +258,47 @@ void aff_copy(WINDOW *window)
 
 static int ask_mode_ext2(const disk_t *disk_car, const partition_t *partition, unsigned int *mode_ext2, unsigned int *carve_free_space_only)
 {
-  static struct MenuItem menuMode[]=
+  static const struct MenuItem menuMode[]=
     {
-      {'E',"EXT2/EXT3","EXT2/EXT3 filesystem"},
+      {'E',"ext2/ext3","ext2/ext3/ext4 filesystem"},
       {'O',"Other","FAT/NTFS/HFS+/ReiserFS/..."},
       {0,NULL,NULL}
     };
-  static struct MenuItem menuFAT16[]=
+  static const struct MenuItem menuFAT16[]=
   {
     {'F',"Free", "Scan for files from FAT16 unallocated space only"},
     {'W',"Whole","Extract files from whole partition"},
     {0,NULL,NULL}
   };
-  static struct MenuItem menuFAT32[]=
+  static const struct MenuItem menuFAT32[]=
   {
     {'F',"Free", "Scan for file from FAT32 unallocated space only"},
     {'W',"Whole","Extract files from whole partition"},
     {0,NULL,NULL}
   };
-  static struct MenuItem menuNTFS[]=
+#ifdef HAVE_LIBNTFS
+  static const struct MenuItem menuNTFS[]=
   {
     {'F',"Free", "Scan for file from NTFS unallocated space only"},
     {'W',"Whole","Extract files from whole partition"},
     {0,NULL,NULL}
   };
-  static struct MenuItem menuEXT2[]=
+#endif
+#ifdef HAVE_LIBEXT2FS
+  static const struct MenuItem menuEXT2[]=
   {
     {'F',"Free", "Scan for file from ext2/ext3 unallocated space only"},
     {'W',"Whole","Extract files from whole partition"},
     {0,NULL,NULL}
   };
+#endif
   const char *options="EO";
   WINDOW *window;
   unsigned int menu;
   int command;
   if(partition->upart_type==UP_EXT2 ||
-      partition->upart_type==UP_EXT3)
+      partition->upart_type==UP_EXT3 ||
+      partition->upart_type==UP_EXT4)
     menu=0;
   else
     menu=1;
@@ -301,14 +310,13 @@ static int ask_mode_ext2(const disk_t *disk_car, const partition_t *partition, u
   waddstr(window,"To recover lost files, PhotoRec need to know the filesystem type where the");
   wmove(window,7,0);
   waddstr(window,"file were stored:");
-  command = wmenuSelect_ext(window,8, 0, menuMode, 11,
+  command = wmenuSelect_ext(window, 24, 8, 0, menuMode, 11,
       options, MENU_VERT | MENU_VERT_WARN | MENU_BUTTON, &menu,NULL);
   *mode_ext2=(command=='E' || command=='e');
   if(*mode_ext2>0)
   {
-    log_info("EXT2/EXT3 mode activated.\n");
+    log_info("ext2/ext3/ext4 mode activated.\n");
   }
-  *carve_free_space_only=0;
   /*
   if((*mode_ext2)!=0)
     return 0;
@@ -322,25 +330,24 @@ static int ask_mode_ext2(const disk_t *disk_car, const partition_t *partition, u
     wclrtoeol(window);
     waddstr(window,"Please choose if all space need to be analysed:");
     if(partition->upart_type==UP_FAT16)
-      command = wmenuSelect_ext(window,8, 0, menuFAT16, 11,
+      command = wmenuSelect_ext(window, 24, 8, 0, menuFAT16, 11,
 	  options, MENU_VERT | MENU_VERT_WARN | MENU_BUTTON, &menu,NULL);
     else if(partition->upart_type==UP_FAT32)
-      command = wmenuSelect_ext(window,8, 0, menuFAT32, 11,
+      command = wmenuSelect_ext(window, 24, 8, 0, menuFAT32, 11,
 	  options, MENU_VERT | MENU_VERT_WARN | MENU_BUTTON, &menu,NULL);
 #ifdef HAVE_LIBNTFS
     else if(partition->upart_type==UP_NTFS)
-      command = wmenuSelect_ext(window,8, 0, menuNTFS, 11,
+      command = wmenuSelect_ext(window, 24, 8, 0, menuNTFS, 11,
 	  options, MENU_VERT | MENU_VERT_WARN | MENU_BUTTON, &menu,NULL);
 #endif
 #ifdef HAVE_LIBEXT2FS
-    else if(partition->upart_type==UP_EXT2 || partition->upart_type==UP_EXT3)
-      command = wmenuSelect_ext(window,8, 0, menuEXT2, 11,
+    else if(partition->upart_type==UP_EXT2 || partition->upart_type==UP_EXT3 || partition->upart_type==UP_EXT4)
+      command = wmenuSelect_ext(window, 24, 8, 0, menuEXT2, 11,
 	  options, MENU_VERT | MENU_VERT_WARN | MENU_BUTTON, &menu,NULL);
 #endif
     else
       command='W';
-    if(command=='F' || command=='f')
-      *carve_free_space_only=1;
+    *carve_free_space_only=(command=='F' || command=='f')?1:0;
     if(*carve_free_space_only>0)
     {
       log_info("Carve free space only.\n");
@@ -354,9 +361,10 @@ static unsigned int menu_choose_blocksize(unsigned int blocksize, const unsigned
 {
   int command;
   unsigned int menu=0;
-  const char *optionsBlocksize="51248736";
-  static struct MenuItem menuBlocksize[]=
+  const char *optionsBlocksize="S51248736";
+  static const struct MenuItem menuBlocksize[]=
   {
+	{'S',"256",""},
 	{'5',"512",""},
 	{'1',"1024",""},
 	{'2',"2048",""},
@@ -369,32 +377,35 @@ static unsigned int menu_choose_blocksize(unsigned int blocksize, const unsigned
   };
   switch(sector_size)
   {
-    case 1024: optionsBlocksize+=1; break;
-    case 2048: optionsBlocksize+=2; break;
-    case 4096: optionsBlocksize+=3; break;
-    case 8192: optionsBlocksize+=4; break;
-    case 16384: optionsBlocksize+=5;break;
-    case 32768: optionsBlocksize+=6; break;
-    case 65536: optionsBlocksize+=7; break;
+    case 512: optionsBlocksize+=1; break;
+    case 1024: optionsBlocksize+=2; break;
+    case 2048: optionsBlocksize+=3; break;
+    case 4096: optionsBlocksize+=4; break;
+    case 8192: optionsBlocksize+=5; break;
+    case 16384: optionsBlocksize+=6;break;
+    case 32768: optionsBlocksize+=7; break;
+    case 65536: optionsBlocksize+=8; break;
   }
   switch(blocksize)
   {
-    case 512: menu=0; break;
-    case 1024: menu=1; break;
-    case 2048: menu=2; break;
-    case 4096: menu=3; break;
-    case 8192: menu=4; break;
-    case 16384: menu=5; break;
-    case 32768: menu=6; break;
-    case 65536: menu=7; break;
+    case 256: menu=0; break;
+    case 512: menu=1; break;
+    case 1024: menu=2; break;
+    case 2048: menu=3; break;
+    case 4096: menu=4; break;
+    case 8192: menu=5; break;
+    case 16384: menu=6; break;
+    case 32768: menu=7; break;
+    case 65536: menu=8; break;
   }
   aff_copy(stdscr);
   wmove(stdscr,INTER_PARTITION_Y-1,0);
   wprintw(stdscr,"Please select the block size, press Enter when done.");
-  command = wmenuSelect_ext(stdscr,INTER_PARTITION_Y, INTER_PARTITION_X, menuBlocksize, 7,
+  command = wmenuSelect_ext(stdscr, 24, INTER_PARTITION_Y, INTER_PARTITION_X, menuBlocksize, 7,
       optionsBlocksize, MENU_VERT| MENU_BUTTON|MENU_VERT_WARN, &menu,NULL);
   switch(command)
   {
+    case 'S': blocksize=256; break;
     case '5': blocksize=512; break;
     case '1': blocksize=1024; break;
     case '2': blocksize=2048; break;
@@ -462,7 +473,7 @@ static int photorec_bf(disk_t *disk_car, partition_t *partition, const int verbo
   int pass2=pass;
   read_size=((*blocksize)>8192?(*blocksize):8192);
   buffer_size=(*blocksize)+READ_SIZE;
-  buffer_start=MALLOC(buffer_size);
+  buffer_start=(unsigned char *)MALLOC(buffer_size);
   for(search_walker=list_search_space->list.prev, n=search_walker->prev;
       search_walker!=&list_search_space->list && ind_stop==0;
       search_walker=n,n=search_walker->prev)
@@ -541,7 +552,7 @@ static int photorec_bf(disk_t *disk_car, partition_t *partition, const int verbo
         {
           if(!(file_recovery.handle=fopen(file_recovery.filename,"w+b")))
           { 
-            log_critical("Cannot create file %s\n", file_recovery.filename);
+            log_critical("Cannot create file %s: %s\n", file_recovery.filename, strerror(errno));
             ind_stop=2;
           }
         }
@@ -650,10 +661,10 @@ static int photorec_bf_aux(disk_t *disk_car, partition_t *partition, const int p
   file_recovery->handle=fopen(file_recovery->filename, "w+b");
   if(file_recovery->handle==NULL)
   {
-    log_critical("Brute Force : Cannot create file %s\n", file_recovery->filename);
+    log_critical("Brute Force : Cannot create file %s: %s\n", file_recovery->filename, strerror(errno));
     return 2;
   }
-  block_buffer=(unsigned char *) malloc(sizeof(unsigned char)*blocksize);
+  block_buffer=(unsigned char *) MALLOC(sizeof(unsigned char)*blocksize);
 
   current_search_space=start_search_space;
   /* We have offset==start_search_space->start==file_recovery->location.start */
@@ -799,6 +810,31 @@ static int photorec_bf_aux(disk_t *disk_car, partition_t *partition, const int p
   return ind_stop;
 }
 
+#if defined(__CYGWIN__) || defined(__MINGW32__)
+/* Live antivirus protection may open file as soon as they are created by *
+ * PhotoRec. PhotoRec will not be able to overwrite a file as long as the *
+ * antivirus is scanning it, so let's wait a little bit if the creation   *
+ * failed. */
+
+#ifndef HAVE_SLEEP
+#define sleep(x) Sleep((x)*1000)
+#endif
+
+static FILE *fopen_with_retry(const char *path, const char *mode)
+{
+  FILE *handle;
+  if((handle=fopen(path, mode))!=NULL)
+    return handle;
+  sleep(1);
+  if((handle=fopen(path, mode))!=NULL)
+    return handle;
+  sleep(2);
+  if((handle=fopen(path, mode))!=NULL)
+    return handle;
+  return NULL;
+}
+#endif
+
 static int photorec_aux(disk_t *disk_car, partition_t *partition, const int verbose, const int paranoid, const char *recup_dir, const int interface, file_stat_t *file_stats, unsigned int *file_nbr, unsigned int *blocksize, alloc_data_t *list_search_space, const time_t real_start_time, unsigned int *dir_num, const photorec_status_t status, const unsigned int pass, const unsigned int expert, const unsigned int lowmem)
 {
   uint64_t offset=0;
@@ -809,7 +845,7 @@ static int photorec_aux(disk_t *disk_car, partition_t *partition, const int verb
   time_t previous_time;
   int ind_stop=0;
   unsigned int buffer_size;
-  unsigned int read_size;
+  const unsigned int read_size=((*blocksize)>65536?(*blocksize):65536);
   alloc_data_t *current_search_space;
   file_recovery_t file_recovery;
   static alloc_data_t list_file={
@@ -818,10 +854,8 @@ static int photorec_aux(disk_t *disk_car, partition_t *partition, const int verb
   static list_cluster_t list_cluster= {
     .list = TD_LIST_HEAD_INIT(list_cluster.list)
   };
-#define READ_SIZE 256*512
-  read_size=((*blocksize)>8192?(*blocksize):8192);
   buffer_size=(*blocksize)+READ_SIZE;
-  buffer_start=MALLOC(buffer_size);
+  buffer_start=(unsigned char *)MALLOC(buffer_size);
   buffer_olddata=buffer_start;
   buffer=buffer_olddata+(*blocksize);
   reset_file_recovery(&file_recovery);
@@ -912,7 +946,7 @@ static int photorec_aux(disk_t *disk_car, partition_t *partition, const int verb
             if(file_recovery.file_stat->file_hint==&file_hint_dir)
             {
               file_data_t *dir_list;
-              dir_list=dir_fat_aux(buffer,read_size,0);
+              dir_list=dir_fat_aux(buffer,read_size,0,0);
               if(dir_list!=NULL)
               {
                 if(verbose>0)
@@ -941,9 +975,14 @@ static int photorec_aux(disk_t *disk_car, partition_t *partition, const int verb
         }
         if(file_recovery.file_stat->file_hint->recover==1 && status!=STATUS_FIND_OFFSET)
         {
-          if(!(file_recovery.handle=fopen(file_recovery.filename,"w+b")))
+#if defined(__CYGWIN__) || defined(__MINGW32__)
+          file_recovery.handle=fopen_with_retry(file_recovery.filename,"w+b");
+#else
+          file_recovery.handle=fopen(file_recovery.filename,"w+b");
+#endif
+          if(!file_recovery.handle)
           { 
-            log_critical("Cannot create file %s\n", file_recovery.filename);
+            log_critical("Cannot create file %s: %s\n", file_recovery.filename, strerror(errno));
             ind_stop=2;
           }
         }
@@ -981,7 +1020,7 @@ static int photorec_aux(disk_t *disk_car, partition_t *partition, const int verb
         {
           if(fwrite(buffer,*blocksize,1,file_recovery.handle)<1)
           { 
-            log_critical("Cannot write file %s\n", file_recovery.filename);
+            log_critical("Cannot write file %s:%s\n", file_recovery.filename, strerror(errno));
             ind_stop=3;
           }
         }
@@ -1078,7 +1117,7 @@ static int photorec_aux(disk_t *disk_car, partition_t *partition, const int verb
         }
 #endif
       }
-      if(interface!=0)
+      if(interface!=0 && ind_stop==0)
       {
         time_t current_time;
         current_time=time(NULL);
@@ -1160,7 +1199,7 @@ static void recovery_finished(const unsigned int file_nbr, const char *recup_dir
 #endif
 
 
-static void free_search_space(alloc_data_t *list_search_space)
+void free_search_space(alloc_data_t *list_search_space)
 {
   struct td_list_head *search_walker = NULL;
   struct td_list_head *search_walker_next = NULL;
@@ -1173,10 +1212,71 @@ static void free_search_space(alloc_data_t *list_search_space)
   }
 }
 
+#if defined(HAVE_NCURSES) && (defined(__CYGWIN__) || defined(__MINGW32__))
+static int interface_cannot_create_file(void)
+{
+  static const struct MenuItem menuMain[]=
+  {
+    { 'C', "Continue", "Continue the recovery."},
+    { 'Q', "Quit", "Abort the recovery."},
+    { 0,NULL,NULL}
+  };
+  unsigned int menu=1;
+  int car;
+  aff_copy(stdscr);
+  wmove(stdscr,4,0);
+  wprintw(stdscr,"PhotoRec has been unable to create new file.");
+  wmove(stdscr,5,0);
+  wprintw(stdscr,"This problem may be due to antivirus blocking write access while scanning files created by PhotoRec.");
+  wmove(stdscr,6,0);
+  wprintw(stdscr,"If possible, temporary disable your antivirus live protection.");
+  car= wmenuSelect_ext(stdscr, 24, INTER_MAIN_Y, INTER_MAIN_X, menuMain, 10,
+      "CQ", MENU_VERT | MENU_VERT_WARN | MENU_BUTTON, &menu,NULL);
+  if(car=='c' || car=='C')
+    return 0;
+  return 1;
+}
+#else
+static int interface_cannot_create_file(void)
+{
+  return 1;
+}
+#endif
+
+static file_stat_t * init_file_stats(file_enable_t *files_enable)
+{
+  file_stat_t *file_stats;
+  file_enable_t *file_enable;
+  unsigned int enable_count=1;	/* Lists are terminated by NULL */
+  for(file_enable=files_enable;file_enable->file_hint!=NULL;file_enable++)
+  {
+    if(file_enable->enable>0)
+    {
+      enable_count++;
+    }
+  }
+  file_stats=(file_stat_t *)MALLOC(enable_count * sizeof(file_stat_t));
+  enable_count=0;
+  for(file_enable=files_enable;file_enable->file_hint!=NULL;file_enable++)
+  {
+    if(file_enable->enable>0)
+    {
+      file_stats[enable_count].file_hint=file_enable->file_hint;
+      file_stats[enable_count].not_recovered=0;
+      file_stats[enable_count].recovered=0;
+      if(file_enable->file_hint->register_header_check!=NULL)
+	file_enable->file_hint->register_header_check(&file_stats[enable_count]);
+      enable_count++;
+    }
+  }
+  file_stats[enable_count].file_hint=NULL;
+  return file_stats;
+}
+
 static int photorec(disk_t *disk_car, partition_t *partition, const int verbose, const int paranoid, char *recup_dir, const int keep_corrupted_file, const int interface, file_enable_t *files_enable, unsigned int mode_ext2, char **current_cmd, alloc_data_t *list_search_space, unsigned int blocksize, const unsigned int expert, const unsigned int lowmem, const unsigned int carve_free_space_only)
 {
   char *new_recup_dir=NULL;
-  file_stat_t *file_stats=NULL;
+  file_stat_t *file_stats;
   time_t real_start_time;
   unsigned int file_nbr=0;
   unsigned int dir_num=1;
@@ -1184,67 +1284,14 @@ static int photorec(disk_t *disk_car, partition_t *partition, const int verbose,
   unsigned int pass;
   unsigned int blocksize_is_known=0;
   photorec_status_t status;
-  aff_buffer(BUFFER_RESET,"Q");
+  screen_buffer_reset();
   log_info("\nAnalyse\n");
   log_partition(disk_car,partition);
-  if(blocksize==0 || td_list_empty(&list_search_space->list))
-  {
+  if(blocksize==0)
     blocksize=disk_car->sector_size;
-    blocksize_is_known=0;
-  }
   else
     blocksize_is_known=1;
-
-  if(td_list_empty(&list_search_space->list))
-  {
-    alloc_data_t *tmp=init_search_space(partition,disk_car);
-    td_list_add_tail(&tmp->list, &list_search_space->list);
-    if(carve_free_space_only>0)
-    {
-      blocksize=remove_used_space(disk_car, partition, list_search_space);
-      if(blocksize==0)
-        blocksize=disk_car->sector_size;
-      else
-        blocksize_is_known=1;
-    }
-  }
-  else
-  { /* Correct the values */
-    struct td_list_head *search_walker = NULL;
-    td_list_for_each(search_walker, &list_search_space->list)
-    {
-      alloc_data_t *current_search_space;
-      current_search_space=td_list_entry(search_walker, alloc_data_t, list);
-      current_search_space->start=current_search_space->start*disk_car->sector_size;
-      current_search_space->end=current_search_space->end*disk_car->sector_size+disk_car->sector_size-1;
-    }
-  }
-  {
-    file_enable_t *file_enable;
-    unsigned int enable_count=1;	/* Lists are terminated by NULL */
-    for(file_enable=files_enable;file_enable->file_hint!=NULL;file_enable++)
-    {
-      if(file_enable->enable>0)
-      {
-        enable_count++;
-      }
-    }
-    file_stats=MALLOC(enable_count * sizeof(file_stat_t));
-    enable_count=0;
-    for(file_enable=files_enable;file_enable->file_hint!=NULL;file_enable++)
-    {
-      if(file_enable->enable>0)
-      {
-        file_stats[enable_count].file_hint=file_enable->file_hint;
-        file_stats[enable_count].not_recovered=0;
-        file_stats[enable_count].recovered=0;
-        if(file_enable->file_hint->register_header_check!=NULL)
-          file_enable->file_hint->register_header_check(&file_stats[enable_count]);
-        enable_count++;
-      }
-    }
-    file_stats[enable_count].file_hint=NULL;
-  }
+  file_stats=init_file_stats(files_enable);
 
   real_start_time=time(NULL);
   dir_num=photorec_mkdir(recup_dir,dir_num);
@@ -1281,16 +1328,32 @@ static int photorec(disk_t *disk_car, partition_t *partition, const int verbose,
     }
 #endif
     if(status==STATUS_FIND_OFFSET && blocksize_is_known>0)
+    {
+#ifdef HAVE_NCURSES
+      if(expert>0)
+      {
+	uint64_t offset=0;
+	if(!td_list_empty(&list_search_space->list))
+	{
+	  alloc_data_t *tmp;
+	  tmp=td_list_entry(list_search_space->list.next, alloc_data_t, list);
+	  offset=tmp->start%blocksize;
+	}
+	blocksize=menu_choose_blocksize(blocksize, disk_car->sector_size, &offset);
+	update_blocksize(blocksize,list_search_space, offset);
+      }
+#endif
       ind_stop=0;
+    }
     else if(status==STATUS_EXT2_ON_BF || status==STATUS_EXT2_OFF_BF)
     {
       ind_stop=photorec_bf(disk_car, partition, verbose, paranoid, recup_dir, interface, file_stats, &file_nbr, &blocksize, list_search_space, real_start_time, &dir_num, status, pass,expert, lowmem);
-      session_save(list_search_space, disk_car, partition, files_enable, blocksize, verbose);
+      session_save(list_search_space, disk_car, partition, files_enable, blocksize, paranoid, keep_corrupted_file, mode_ext2, expert, lowmem, carve_free_space_only, verbose);
     }
     else
     {
       ind_stop=photorec_aux(disk_car, partition, verbose, paranoid, recup_dir, interface, file_stats, &file_nbr, &blocksize, list_search_space, real_start_time, &dir_num, status, pass,expert, lowmem);
-      session_save(list_search_space, disk_car, partition, files_enable, blocksize, verbose);
+      session_save(list_search_space, disk_car, partition, files_enable, blocksize, paranoid, keep_corrupted_file, mode_ext2, expert, lowmem, carve_free_space_only, verbose);
     }
     if(ind_stop==3)
     { /* no more space */
@@ -1302,7 +1365,7 @@ static int photorec(disk_t *disk_car, partition_t *partition, const int verbose,
       else
       {
         free(new_recup_dir);
-        new_recup_dir=MALLOC(strlen(res)+1+strlen(DEFAULT_RECUP_DIR)+1);
+        new_recup_dir=(char *)MALLOC(strlen(res)+1+strlen(DEFAULT_RECUP_DIR)+1);
         strcpy(new_recup_dir,res);
         strcat(new_recup_dir,"/");
         strcat(new_recup_dir,DEFAULT_RECUP_DIR);
@@ -1315,6 +1378,11 @@ static int photorec(disk_t *disk_car, partition_t *partition, const int verbose,
       status=STATUS_QUIT;
 #endif
     }
+    else if(ind_stop==2)
+    {
+      if(interface_cannot_create_file()!=0)
+	status=STATUS_QUIT;
+    }
     else if(ind_stop>0)
     {
       status=STATUS_QUIT;
@@ -1412,7 +1480,18 @@ static int photorec(disk_t *disk_car, partition_t *partition, const int verbose,
   return 0;
 }
 
+static int spacerange_cmp(const struct td_list_head *a, const struct td_list_head *b)
+{
+  const alloc_data_t *space_a=td_list_entry(a, const alloc_data_t, list);
+  const alloc_data_t *space_b=td_list_entry(b, const alloc_data_t, list);
+  if(space_a->start < space_b->start)
+    return -1;
+  if(space_a->start > space_b->start)
+    return 1;
+  return space_a->end - space_b->end;
+}
 
+enum { INIT_SPACE_WHOLE, INIT_SPACE_PREINIT, INIT_SPACE_EXT2_GROUP, INIT_SPACE_EXT2_INODE };
 
 static void menu_photorec(disk_t *disk_car, const int verbose, const char *recup_dir, file_enable_t *file_enable, char **current_cmd, alloc_data_t*list_search_space)
 {
@@ -1431,11 +1510,12 @@ static void menu_photorec(disk_t *disk_car, const int verbose, const char *recup
   unsigned int lowmem=0;
   unsigned int carve_free_space_only=0;
   int done=0;
+  int mode_init_space=(td_list_empty(&list_search_space->list)?INIT_SPACE_WHOLE:INIT_SPACE_PREINIT);
 #ifdef HAVE_NCURSES
   int command;
   int offset=0;
   unsigned int menu=0;
-  static struct MenuItem menuMain[]=
+  static const struct MenuItem menuMain[]=
   {
 	{'S',"Search","Start file recovery"},
 	{'O',"Options","Modify options"},
@@ -1488,7 +1568,7 @@ static void menu_photorec(disk_t *disk_car, const int verbose, const char *recup
 	  res=ask_location("Do you want to save recovered files in %s%s ? [Y/N]\nDo not choose to write the files to the same partition they were stored on.","");
 	  if(res!=NULL)
 	  {
-	    char *new_recup_dir=MALLOC(strlen(res)+1+strlen(DEFAULT_RECUP_DIR)+1);
+	    char *new_recup_dir=(char *)MALLOC(strlen(res)+1+strlen(DEFAULT_RECUP_DIR)+1);
 	    strcpy(new_recup_dir,res);
 	    strcat(new_recup_dir,"/");
 	    strcat(new_recup_dir,DEFAULT_RECUP_DIR);
@@ -1498,7 +1578,26 @@ static void menu_photorec(disk_t *disk_car, const int verbose, const char *recup
 	  }
 	}
 	if(res!=NULL)
-	  photorec(disk_car, current_element->part, verbose, paranoid, res, keep_corrupted_file,1,file_enable,mode_ext2,current_cmd,list_search_space,blocksize,expert, lowmem, carve_free_space_only);
+	{
+	  partition_t *partition=current_element->part;
+	  if(mode_init_space==INIT_SPACE_EXT2_GROUP)
+	  {
+	    blocksize=ext2_fix_group(list_search_space, disk_car, partition);
+	  }
+	  else if(mode_init_space==INIT_SPACE_EXT2_INODE)
+	  {
+	    blocksize=ext2_fix_inode(list_search_space, disk_car, partition);
+	  }
+	  if(td_list_empty(&list_search_space->list))
+	  {
+	    init_search_space(list_search_space, disk_car, partition);
+	  }
+	  if(carve_free_space_only>0)
+	  {
+	    blocksize=remove_used_space(disk_car, partition, list_search_space);
+	  }
+	  photorec(disk_car, partition, verbose, paranoid, res, keep_corrupted_file,1,file_enable,mode_ext2,current_cmd,list_search_space,blocksize,expert, lowmem, carve_free_space_only);
+	}
 	if(res!=recup_dir)
 	  free(res);
       }
@@ -1532,6 +1631,60 @@ static void menu_photorec(disk_t *disk_car, const int verbose, const char *recup
       {	/* Start interactive mode */
 	*current_cmd=NULL;
       }
+      else if(strncmp(*current_cmd,"wholespace",10)==0)
+      {
+	(*current_cmd)+=10;
+	carve_free_space_only=0;
+      }
+      else if(strncmp(*current_cmd,"freespace",9)==0)
+      {
+	(*current_cmd)+=9;
+	carve_free_space_only=1;
+      }
+      else if(strncmp(*current_cmd,"ext2_group,",11)==0)
+      {
+	unsigned int groupnr;
+	(*current_cmd)+=11;
+	mode_ext2=1;
+	groupnr=atoi(*current_cmd);
+	while(*current_cmd[0]!=',' && *current_cmd[0]!='\0')
+	  (*current_cmd)++;
+	if(mode_init_space==INIT_SPACE_WHOLE)
+	  mode_init_space=INIT_SPACE_EXT2_GROUP;
+	if(mode_init_space==INIT_SPACE_EXT2_GROUP)
+	{
+          alloc_data_t *new_free_space;
+          new_free_space=(alloc_data_t*)MALLOC(sizeof(*new_free_space));
+          /* Temporary storage, values need to be multiplied by group size and aligned */
+          new_free_space->start=groupnr;
+          new_free_space->end=groupnr;
+          new_free_space->file_stat=NULL;
+          if(td_list_add_sorted_uniq(&new_free_space->list, &list_search_space->list, spacerange_cmp))
+	    free(new_free_space);
+        }
+      }
+      else if(strncmp(*current_cmd,"ext2_inode,",11)==0)
+      {
+	unsigned int inodenr;
+	(*current_cmd)+=11;
+	mode_ext2=1;
+	inodenr=atoi(*current_cmd);
+	while(*current_cmd[0]!=',' && *current_cmd[0]!='\0')
+	  (*current_cmd)++;
+	if(mode_init_space==INIT_SPACE_WHOLE)
+	  mode_init_space=INIT_SPACE_EXT2_INODE;
+	if(mode_init_space==INIT_SPACE_EXT2_INODE)
+	{
+          alloc_data_t *new_free_space;
+          new_free_space=(alloc_data_t*)MALLOC(sizeof(*new_free_space));
+          /* Temporary storage, values need to be multiplied by group size and aligned */
+          new_free_space->start=inodenr;
+          new_free_space->end=inodenr;
+          new_free_space->file_stat=NULL;
+          if(td_list_add_sorted_uniq(&new_free_space->list, &list_search_space->list, spacerange_cmp))
+	    free(new_free_space);
+        }
+      }
       else if(isdigit(*current_cmd[0]))
       {
 	unsigned int order;
@@ -1574,7 +1727,7 @@ static void menu_photorec(disk_t *disk_car, const int verbose, const char *recup
 	  aff_part(stdscr,AFF_PART_ORDER|AFF_PART_STATUS,disk_car,element->part);
 	}
       }
-      command = wmenuSelect(stdscr,INTER_SELECT_Y, INTER_SELECT_X, menuMain, 8,
+      command = wmenuSelect(stdscr, 24, INTER_SELECT_Y, INTER_SELECT_X, menuMain, 8,
 	  (expert==0?"SOFQ":"SOFGQ"), MENU_HORIZ | MENU_BUTTON | MENU_ACCEPT_OTHERS, menu);
       switch(command)
       {
@@ -1604,9 +1757,11 @@ static void menu_photorec(disk_t *disk_car, const int verbose, const char *recup
 	  break;
 	case 's':
 	case 'S':
-	  ask_mode_ext2(disk_car, current_element->part, &mode_ext2, &carve_free_space_only);
+	  if(current_element!=NULL)
 	  {
 	    char *res;
+	    partition_t *partition=current_element->part;
+	    ask_mode_ext2(disk_car, partition, &mode_ext2, &carve_free_space_only);
 	    menu=0;
 	    if(recup_dir!=NULL)
 	      res=recup_dir;
@@ -1615,7 +1770,7 @@ static void menu_photorec(disk_t *disk_car, const int verbose, const char *recup
 	      res=ask_location("Do you want to save recovered files in %s%s ? [Y/N]\nDo not choose to write the files to the same partition they were stored on.","");
 	      if(res!=NULL)
 	      {
-		char *new_recup_dir=MALLOC(strlen(res)+1+strlen(DEFAULT_RECUP_DIR)+1);
+		char *new_recup_dir=(char *)MALLOC(strlen(res)+1+strlen(DEFAULT_RECUP_DIR)+1);
 		strcpy(new_recup_dir,res);
 		strcat(new_recup_dir,"/");
 		strcat(new_recup_dir,DEFAULT_RECUP_DIR);
@@ -1625,7 +1780,17 @@ static void menu_photorec(disk_t *disk_car, const int verbose, const char *recup
 	      }
 	    }
 	    if(res!=NULL)
-	      photorec(disk_car, current_element->part, verbose, paranoid, res, keep_corrupted_file,1,file_enable,mode_ext2, current_cmd, list_search_space,blocksize,expert, lowmem, carve_free_space_only);
+	    {
+	      if(td_list_empty(&list_search_space->list))
+	      {
+		init_search_space(list_search_space, disk_car, partition);
+	      }
+	      if(carve_free_space_only>0)
+	      {
+		blocksize=remove_used_space(disk_car, partition, list_search_space);
+	      }
+	      photorec(disk_car, partition, verbose, paranoid, res, keep_corrupted_file,1,file_enable,mode_ext2, current_cmd, list_search_space,blocksize,expert, lowmem, carve_free_space_only);
+	    }
 	    if(res!=recup_dir)
 	      free(res);
 	  }
@@ -1675,7 +1840,7 @@ static void photorec_disk_selection_ncurses(int verbose, const char *recup_dir,
   int pos_num=0;
   const list_disk_t *element_disk;
   const list_disk_t *current_disk=list_disk;
-  static struct MenuItem menuMain[]=
+  static const struct MenuItem menuMain[]=
   {
     { 'P', "Previous",""},
     { 'N', "Next","" },
@@ -1739,7 +1904,7 @@ static void photorec_disk_selection_ncurses(int verbose, const char *recup_dir,
       wmove(stdscr,line++,0);
       wprintw(stdscr,"detection, and install the latest OS patches and disk drivers."); 
     }
-    command = wmenuSelect_ext(stdscr,INTER_MAIN_Y, INTER_MAIN_X, menuMain, 8,
+    command = wmenuSelect_ext(stdscr, 24, INTER_MAIN_Y, INTER_MAIN_X, menuMain, 8,
 	options, MENU_HORIZ | MENU_BUTTON | MENU_ACCEPT_OTHERS, &menu,&real_key);
     switch(command)
     {
@@ -1801,10 +1966,13 @@ static void photorec_disk_selection_ncurses(int verbose, const char *recup_dir,
 
 int do_curses_photorec(int verbose, const char *recup_dir, const list_disk_t *list_disk, file_enable_t *file_enable, char *cmd_device, char **current_cmd)
 {
-  const list_disk_t *current_disk=list_disk;
   static alloc_data_t list_search_space={
     .list = TD_LIST_HEAD_INIT(list_search_space.list)
   };
+  if(list_disk==NULL)
+  {
+    return intrf_no_disk("PhotoRec");
+  }
   if(cmd_device==NULL)
   {
     char *saved_device=NULL;
@@ -1823,22 +1991,30 @@ int do_curses_photorec(int verbose, const char *recup_dir, const list_disk_t *li
       free_list_search_space(&list_search_space);
     }
   }
-  if(cmd_device!=NULL)
+  if(cmd_device!=NULL && *current_cmd!=NULL)
   {
     const list_disk_t *element_disk;
+    disk_t *disk=NULL;
     for(element_disk=list_disk;element_disk!=NULL;element_disk=element_disk->next)
     {
       if(strcmp(element_disk->disk->device,cmd_device)==0)
-	current_disk=element_disk;
+	disk=element_disk->disk;
+    }
+    if(disk==NULL)
+    {
+      return intrf_no_disk("PhotoRec");
+    }
+    {
+      /* disk sector size is now known, fix the sector ranges */
+      struct td_list_head *search_walker = NULL;
+      td_list_for_each(search_walker, &list_search_space.list)
+      {
+	alloc_data_t *current_search_space;
+	current_search_space=td_list_entry(search_walker, alloc_data_t, list);
+	current_search_space->start=current_search_space->start*disk->sector_size;
+	current_search_space->end=current_search_space->end*disk->sector_size+disk->sector_size-1;
+      }
     }
-  }
-  if(current_disk==NULL)
-  {
-    return intrf_no_disk("PhotoRec");
-  }
-  if(*current_cmd!=NULL)
-  {
-    disk_t *disk=current_disk->disk;
     autodetect_arch(disk);
     if(interface_partition_type(disk, verbose, current_cmd)==0)
       menu_photorec(disk, verbose, recup_dir, file_enable, current_cmd, &list_search_space);
@@ -1901,7 +2077,7 @@ static void interface_options_photorec_ncurses(int *paranoid, int *allow_partial
 
      */
     aff_copy(stdscr);
-    car=wmenuSelect_ext(stdscr,INTER_OPTION_Y, INTER_OPTION_X, menuOptions, 0, "PAKELQ", MENU_VERT|MENU_VERT_ARROW2VALID, &menu,&real_key);
+    car=wmenuSelect_ext(stdscr, 24, INTER_OPTION_Y, INTER_OPTION_X, menuOptions, 0, "PAKELQ", MENU_VERT|MENU_VERT_ARROW2VALID, &menu,&real_key);
     switch(car)
     {
       case 'p':
@@ -1949,26 +2125,52 @@ static void interface_options_photorec(int *paranoid, int *allow_partial_last_cy
     {
       while(*current_cmd[0]==',')
 	(*current_cmd)++;
-      if(strncmp(*current_cmd,"mode_ext2",9)==0)
+      /* paranoid, longer option first */
+      if(strncmp(*current_cmd,"paranoid_no",11)==0)
       {
-	(*current_cmd)+=9;
-	*mode_ext2=1;
+	(*current_cmd)+=11;
+	*paranoid=0;
       }
-      else if(strncmp(*current_cmd,"expert",6)==0)
+      else if(strncmp(*current_cmd,"paranoid_bf",11)==0)
       {
-	(*current_cmd)+=6;
-	*expert=1;
+	(*current_cmd)+=11;
+	*paranoid=2;
       }
-      else if(strncmp(*current_cmd,"lowmem",6)==0)
+      else if(strncmp(*current_cmd,"paranoid",8)==0)
       {
-	(*current_cmd)+=6;
-	*lowmem=1;
+	(*current_cmd)+=8;
+	*paranoid=1;
+      }
+      /* TODO: allow_partial_last_cylinder */
+      /* keep_corrupted_file */
+      else if(strncmp(*current_cmd,"keep_corrupted_file_no",22)==0)
+      {
+	(*current_cmd)+=22;
+	*keep_corrupted_file=0;
       }
       else if(strncmp(*current_cmd,"keep_corrupted_file",19)==0)
       {
 	(*current_cmd)+=19;
 	*keep_corrupted_file=1;
       }
+      /* mode_ext2 */
+      else if(strncmp(*current_cmd,"mode_ext2",9)==0)
+      {
+	(*current_cmd)+=9;
+	*mode_ext2=1;
+      }
+      /* expert */
+      else if(strncmp(*current_cmd,"expert",6)==0)
+      {
+	(*current_cmd)+=6;
+	*expert=1;
+      }
+      /* lowmem */
+      else if(strncmp(*current_cmd,"lowmem",6)==0)
+      {
+	(*current_cmd)+=6;
+	*lowmem=1;
+      }
       else
 	keep_asking=0;
     } while(keep_asking>0);
@@ -1991,13 +2193,18 @@ static void interface_options_photorec(int *paranoid, int *allow_partial_last_cy
 }
 
 #ifdef HAVE_NCURSES
+#define INTER_FSELECT_X	0
+#define INTER_FSELECT_Y	LINES-2
+#define INTER_FSELECT	LINES-10
+
 static void interface_file_select_ncurses(file_enable_t *files_enable)
 {
   int current_element_num=0;
   int offset=0;
-  int rewrite=1;
+  int old_LINES=0;	/* Screen will be cleared */
   unsigned int menu=0;
-  static struct MenuItem menuAdv[]=
+  int enable_status=files_enable[0].enable;
+  static const struct MenuItem menuAdv[]=
   {
     {'q',"Quit","Return to main menu"},
     {0,NULL,NULL}
@@ -2006,20 +2213,22 @@ static void interface_file_select_ncurses(file_enable_t *files_enable)
   {
     int i;
     int command;
-    if(rewrite!=0)
+    if(old_LINES!=LINES)
     {
       aff_copy(stdscr);
-      wmove(stdscr,5,0);
+      wmove(stdscr,4,0);
       wprintw(stdscr,"PhotoRec will try to locate the following files");
-      rewrite=0;
+      current_element_num=0;
+      offset=0;
+      old_LINES=LINES;
     }
-    wmove(stdscr,5+1,4);
+    wmove(stdscr,5,4);
     wclrtoeol(stdscr);
     if(offset>0)
       wprintw(stdscr,"Previous");
-    for(i=offset;files_enable[i].file_hint!=NULL && ((i-offset)<INTER_SELECT);i++)
+    for(i=offset;files_enable[i].file_hint!=NULL && i<offset+INTER_FSELECT;i++)
     {
-      wmove(stdscr,5+2+i-offset,0);
+      wmove(stdscr,6+i-offset,0);
       wclrtoeol(stdscr);	/* before addstr for BSD compatibility */
       if(i==current_element_num)
       {
@@ -2037,15 +2246,34 @@ static void interface_file_select_ncurses(file_enable_t *files_enable)
 	    files_enable[i].file_hint->description);
       }
     }
-    wmove(stdscr,5+2+INTER_SELECT,4);
+    wmove(stdscr,6+INTER_FSELECT,4);
     wclrtoeol(stdscr);	/* before addstr for BSD compatibility */
     if(files_enable[i].file_hint!=NULL)
       wprintw(stdscr,"Next");
-    command = wmenuSelect(stdscr,INTER_SELECT_Y, INTER_SELECT_X, menuAdv, 8,
+    wmove(stdscr,6+INTER_FSELECT+1,0);
+    wclrtoeol(stdscr);
+    wprintw(stdscr,"Press ");
+    if(has_colors())
+      wbkgdset(stdscr,' ' | A_BOLD | COLOR_PAIR(0));
+    wprintw(stdscr,"s");
+    if(has_colors())
+      wbkgdset(stdscr,' ' | COLOR_PAIR(0));
+    if(enable_status==0)
+      wprintw(stdscr," for default selection, ");
+    else
+      wprintw(stdscr," to disable all file famillies, ");
+    if(has_colors())
+      wbkgdset(stdscr,' ' | A_BOLD | COLOR_PAIR(0));
+    wprintw(stdscr,"b");
+    if(has_colors())
+      wbkgdset(stdscr,' ' | COLOR_PAIR(0));
+    wprintw(stdscr," to save the settings");
+    command = wmenuSelect(stdscr, LINES-1, INTER_FSELECT_Y, INTER_FSELECT_X, menuAdv, 8,
 	"q", MENU_BUTTON | MENU_ACCEPT_OTHERS, menu);
     switch(command)
     {
       case KEY_UP:
+      case '8':
 	if(current_element_num>0)
 	{
 	  current_element_num--;
@@ -2054,7 +2282,8 @@ static void interface_file_select_ncurses(file_enable_t *files_enable)
 	}
 	break;
       case KEY_PPAGE:
-	for(i=0;(i<INTER_SELECT) && (current_element_num>0);i++)
+      case '9':
+	for(i=0;i<INTER_FSELECT-1 && current_element_num>0;i++)
 	{
 	  current_element_num--;
 	  if(current_element_num<offset)
@@ -2062,18 +2291,20 @@ static void interface_file_select_ncurses(file_enable_t *files_enable)
 	}
 	break;
       case KEY_DOWN:
+      case '2':
 	if(files_enable[current_element_num+1].file_hint!=NULL)
 	{
 	  current_element_num++;
-	  if(current_element_num>=offset+INTER_SELECT)
+	  if(current_element_num>=offset+INTER_FSELECT)
 	    offset++;
 	}
 	break;
       case KEY_NPAGE:
-	for(i=0;(i<INTER_SELECT) && (files_enable[current_element_num+1].file_hint!=NULL);i++)
+      case '3':
+	for(i=0;i<INTER_FSELECT-1 && (files_enable[current_element_num+1].file_hint!=NULL);i++)
 	{
 	  current_element_num++;
-	  if(current_element_num>=offset+INTER_SELECT)
+	  if(current_element_num>=offset+INTER_FSELECT)
 	    offset++;
 	}
 	break;
@@ -2084,15 +2315,34 @@ static void interface_file_select_ncurses(file_enable_t *files_enable)
       case '-':
       case 'x':
       case 'X':
+      case '4':
+      case '5':
+      case '6':
 	files_enable[current_element_num].enable=1-files_enable[current_element_num].enable;
 	break;
       case 's':
       case 'S':
 	{
-	  file_enable_t *file_enable;
-	  int enable_status=1-files_enable[0].enable;
-	  for(file_enable=&files_enable[0];file_enable->file_hint!=NULL;file_enable++)
-	    file_enable->enable=enable_status;
+	  enable_status=1-enable_status;
+	  if(enable_status==0)
+	  {
+	    file_enable_t *file_enable;
+	    for(file_enable=&files_enable[0];file_enable->file_hint!=NULL;file_enable++)
+	      file_enable->enable=0;
+	  }
+	  else
+	    reset_list_file_enable(files_enable);
+	}
+	break;
+      case 'b':
+      case 'B':
+	if(file_options_save(files_enable)<0)
+	{
+	  display_message("Failed to save the settings.");
+	}
+	else
+	{
+	  display_message("Settings recorded successfully.");
 	}
 	break;
       case 'q':
