@@ -104,6 +104,6 @@ struct format40_super {
   char sb_unused[432];
 } __attribute__((packed));
 
-int check_rfs(disk_t *disk_car,partition_t *partition,const int debug);
-int recover_rfs(disk_t *disk_car, const struct reiserfs_super_block *sb,partition_t *partition,const int debug, const int dump_ind);
+int check_rfs(disk_t *disk_car,partition_t *partition,const int verbose);
+int recover_rfs(disk_t *disk_car, const struct reiserfs_super_block *sb,partition_t *partition,const int verbose, const int dump_ind);
 
