@@ -103,6 +103,6 @@ struct xfs_sb
 	uint32_t	sb_features2;	/* additonal feature bits */
 } __attribute__ ((__packed__));
 
-int check_xfs(disk_t *disk_car,partition_t *partition,const int debug);
-int recover_xfs(disk_t *disk_car, const struct xfs_sb *sb,partition_t *partition,const int debug, const int dump_ind);
+int check_xfs(disk_t *disk_car,partition_t *partition,const int verbose);
+int recover_xfs(disk_t *disk_car, const struct xfs_sb *sb,partition_t *partition,const int verbose, const int dump_ind);
 
